#include "stdafx.h"
#include <d3dx9math.h>
#include "dx_control.h"


//p}N
#define SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }
#define SAFE_RELEASE(p)      { if(p) { (p)->Release(); (p)=NULL; } }


//RXgN^
dx_control::dx_control()
	: pD3D( NULL )
	, pDevice( NULL )
	, bDeviceLost( FALSE )
	, sBgColor( D3DCOLOR_XRGB(0,0,0) )
	, bUpdateSize( FALSE )
	, iMaxLightCount( 0 )
{
	::ZeroMemory( &this->param, sizeof( D3DPRESENT_PARAMETERS ) );
	this->sScreen.cx = 640;
	this->sScreen.cy = 480;
}

//fXgN^
dx_control::~dx_control()
{
	Dispose();		//[`
}

//[`
HRESULT dx_control::Dispose()
{
	SAFE_RELEASE(this->pDevice);
	SAFE_RELEASE(this->pD3D);

	return TRUE;
}



//Direct3D9
HRESULT dx_control::Init(HWND hWnd, BOOL bWin)
{
	HRESULT	hr;

	if( NULL == ( this->pD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) )
		return E_FAIL;

	D3DPRESENT_PARAMETERS &d3dpp = this->param;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.BackBufferCount = 1;
	d3dpp.BackBufferWidth = this->sScreen.cx;
	d3dpp.BackBufferHeight = this->sScreen.cy;
	d3dpp.hDeviceWindow = NULL;
	d3dpp.Windowed = bWin;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	// Z obt@̎쐬
	d3dpp.EnableAutoDepthStencil = TRUE;
	d3dpp.AutoDepthStencilFormat = D3DFMT_D16;

	d3dpp.MultiSampleType = D3DMULTISAMPLE_NONE;
	d3dpp.MultiSampleQuality = 0;
	d3dpp.Flags = 0;
	d3dpp.FullScreen_RefreshRateInHz = D3DPRESENT_RATE_DEFAULT;
	d3dpp.PresentationInterval = D3DPRESENT_INTERVAL_DEFAULT;
	d3dpp.BackBufferFormat = D3DFMT_UNKNOWN;

	struct init_state
	{
		D3DDEVTYPE	eDevType;
		long		lVertexProcess;
	} InitSt[] =
	{
		{ D3DDEVTYPE_HAL, D3DCREATE_HARDWARE_VERTEXPROCESSING },
		{ D3DDEVTYPE_HAL, D3DCREATE_SOFTWARE_VERTEXPROCESSING },
		{ D3DDEVTYPE_REF, D3DCREATE_HARDWARE_VERTEXPROCESSING },
		{ D3DDEVTYPE_REF, D3DCREATE_SOFTWARE_VERTEXPROCESSING },
	};

	int iInitCount = sizeof(InitSt) / sizeof(InitSt[ 0 ]);
	for ( int i = 0; i < iInitCount; i++ )
	{
		if( SUCCEEDED( hr = this->pD3D->CreateDevice(
			0,
			InitSt[ i ].eDevType,
			hWnd,
			InitSt[ i ].lVertexProcess,
			&this->param,
			&this->pDevice )))
		{
			D3DCAPS9 sD3dcap;
			this->pD3D->GetDeviceCaps(
				D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, &sD3dcap );

			// ő僉Cgݒ
			this->iMaxLightCount = sD3dcap.MaxActiveLights;

			return hr;
		}
	}

	return E_FAIL;
}

#if 0
void dbOutString( const char * fmt, ... )
{
	char buf[1024];
	va_list ap;
	va_start(ap, fmt);
	_vsnprintf_s( buf, 1024, fmt, ap );
	va_end(ap);
	OutputDebugStringA(buf);
}

static DWORD s_dwCount = 0;

// Jn
static void StartCount()
{
	s_dwCount = ::GetTickCount();
}
// JE^\
static void ReportCount( const char *pName )
{
	dbOutString( "%s[%d]\n", pName, ::GetTickCount() - s_dwCount );
	// dbOutString͖̕
	s_dwCount = ::GetTickCount();
}
#endif


//`
HRESULT dx_control::Render( void (*pFunc)( IDirect3DDevice9 *pDevice, void *pvData ), void *pvUserData )
{

	HRESULT	hr = 0;

	if ( this->bDeviceLost )
	{
		if ( this->pDevice->TestCooperativeLevel() != D3DERR_DEVICENOTRESET )
			return 0;

		this->param.BackBufferWidth = this->sScreen.cx;
		this->param.BackBufferHeight = this->sScreen.cy;

		if ( FAILED( hr = this->pDevice->Reset( &this->param ) ) )
		{
			return 0;
		}

		this->bDeviceLost = FALSE;
		// TCYύXƃfoCXXgdȂꍇ
		this->bUpdateSize = false;
	}


	if ( this->bUpdateSize )
	{
		this->param.BackBufferWidth = this->sScreen.cx;
		this->param.BackBufferHeight = this->sScreen.cy;

		if ( FAILED( hr = this->pDevice->Reset( &this->param ) ) )
		{
			return 0;
		}
		this->bUpdateSize = false;
	}



	if ( FAILED( this->pDevice->Clear(
							0,
							NULL,
							D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER,
							this->sBgColor,
							1.0f, 0 ) ) )
	{
		return E_FAIL;
	}

	//V[Jn
	if ( SUCCEEDED( hr = this->pDevice->BeginScene() ) )
	{
		if ( pFunc != NULL )
		{
			D3DXMATRIX mView;
			D3DXMATRIX mProjection;

			// vWFNV
			D3DXMatrixOrthoOffCenterLH( &mProjection,
							0.0f,
							(float)this->sScreen.cx,		// KF{Client Width
							(float)this->sScreen.cy,		// KF{Client Hight
							0.0f,
							0.0f,
							10.0f);

			// r[|[g
			D3DXMatrixLookAtLH( &mView, 
				&D3DXVECTOR3( 0.0f, 0.0f, 0.0f ),	// _
				&D3DXVECTOR3( 0.0f, 0.0f, 1.0f ),	// ʒu
				&D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );	// UpVector


			pDevice->SetTransform( D3DTS_VIEW, &mView );
			pDevice->SetTransform( D3DTS_PROJECTION, &mProjection );


			// }eAĝŃCgONɂB
			// AACgf[^͑SOFFɂB
			pDevice->SetRenderState( D3DRS_LIGHTING, TRUE );
		//	pDevice->SetRenderState( D3DRS_LIGHTING, FALSE );

			pDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

			// At@uhIy[V
			pDevice->SetRenderState( D3DRS_BLENDOP, D3DBLENDOP_ADD );

			pDevice->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCALPHA );
			pDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );



			pDevice->SetRenderState( D3DRS_SPECULARENABLE, FALSE );

			// ȉ͑Să}eA\[Xɂ
			pDevice->SetRenderState( D3DRS_AMBIENTMATERIALSOURCE, D3DMCS_MATERIAL );
			pDevice->SetRenderState( D3DRS_DIFFUSEMATERIALSOURCE, D3DMCS_MATERIAL );
			pDevice->SetRenderState( D3DRS_EMISSIVEMATERIALSOURCE, D3DMCS_MATERIAL );

//			pDevice->SetRenderState( D3DRS_COLORVERTEX, TRUE );
			pDevice->SetRenderState( D3DRS_COLORVERTEX, FALSE );

			pDevice->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_ARGB( 255,255,255,255 ));
			

			pDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );
			pDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
			pDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_CURRENT );

			pDevice->SetSamplerState( 0, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP );
			pDevice->SetSamplerState( 0, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP );

			pDevice->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
			pDevice->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );


			// őSẴCg񂷂悤ɂ
			for ( int i = 0; i < this->iMaxLightCount; i++ )
			{
				this->pDevice->LightEnable( i, FALSE );
			}


			D3DXMATRIX mTexture;
			D3DXMatrixIdentity( &mTexture );
			this->pDevice->SetTransform( D3DTS_TEXTURE0, &mTexture );

			static D3DMATERIAL9 sMaterial = 
			{
				{ 255.0f, 255.0f, 255.0f, 255.0f },/* Diffuse color RGBA */
				{ 255.0f, 255.0f, 255.0f, 255.0f },/* Ambient color RGB */
				{ 255.0f, 255.0f, 255.0f, 255.0f },/* Specular 'shininess' */
				{ 255.0f, 255.0f, 255.0f, 255.0f },/* Emissive color RGB */
				1.0f							   /* Sharpness if specular highlight */
			};

			this->pDevice->SetMaterial( &sMaterial );

			pFunc( this->pDevice, pvUserData );
		}

		hr = this->pDevice->EndScene();
	}

//	StartCount();

	//obNobt@vC}obt@֓]
	if ( this->pDevice->Present( NULL, NULL, NULL, NULL ) == D3DERR_DEVICELOST )
	{
		this->bDeviceLost = TRUE;
	}
	// JE^\
//	ReportCount( "Present" );

	return hr;
}
