﻿#include "stdafx.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "WavFormatCtrl.h"
#include "ChunkInfo.h"

using namespace glb;
namespace dx
{
	// コンストラクタ
	WavFormatCtrl::WavFormatCtrl ()
		: fStart( 0.0f )			// 再生開始タイム
		, fTime( 0.0f )				// 再生時間
		, iWavSize( 0 )				// WAVサイズ取得・設定
		, lWavOffset( 0 )			// 自由に使える
		, bEndLimitEnable( false )	// 途中終了フラグ
		, iStartPos( 0 )			// ストリームの再生開始位置
		, iEndPos( 0 )				// ストリームの再生終了位置
	{
	}

	// コンストラクタ(クローン用)
	WavFormatCtrl::WavFormatCtrl( WavFormatCtrl *pBase )
		: fStart( 0.0f )			// 再生開始タイム
		, fTime( 0.0f )				// 再生時間
		, iWavSize( 0 )				// WAVサイズ取得・設定
		, lWavOffset( 0 )			// 自由に使える
		, bEndLimitEnable( false )	// 途中終了フラグ
		, iStartPos( 0 )			// ストリームの再生開始位置
		, iEndPos( 0 )				// ストリームの再生終了位置
	{
		this->sWavFormat = pBase->sWavFormat;
		this->fStart		= pBase->fStart;			// 再生開始タイム
		this->fTime		= pBase->fTime;				// 再生時間
		this->lWavOffset		= pBase->lWavOffset;				// 自由に使える
		this->bEndLimitEnable = pBase->bEndLimitEnable;	// 途中終了フラグ
		this->iStartPos	= pBase->iStartPos;			// ストリームの再生開始位置
		this->iEndPos	= pBase->iEndPos;			// ストリームの再生終了位置
		this->iWavSize	= pBase->iWavSize;
	}


	// 再生開始時間
	void WavFormatCtrl::Start( float value )
	{
		fStart = value;
		this->ResetPos();
	}

	// 再生時間
	void WavFormatCtrl::Time( float value )
	{
		fTime = value;
		this->ResetPos();
	}


	// クローン作成
	WavFormatCtrl *WavFormatCtrl::Clone()
	{
		return evc_new WavFormatCtrl( this );
	}


	// １秒が何バイトかを取得
	int WavFormatCtrl::GetSecByte() const
	{
		return (int)(this->SampleSize() * this->sWavFormat.nSamplesPerSec);
	}

	// 開始、終了位置を再設定
	void WavFormatCtrl::ResetPos()
	{
		float fSec = (float)this->GetSecByte();

		this->iStartPos = (int)(fSec * this->fStart);
//		this->iStartPos -= (int)((float)this->iStartPos % this->SampleSize());
		this->iStartPos -= this->iStartPos % (int)this->SampleSize();
		
		this->iEndPos = (int)( fSec * (this->fStart + this->fTime) );
//		this->iEndPos -= (int)((float)this->iEndPos % this->SampleSize());
		this->iEndPos -= this->iEndPos % (int)this->SampleSize();
	}


	// ストリームを指定の再生位置に移動(時間指定)
	bool WavFormatCtrl::SetTime( est::istream &inStream, float fTime )
	{
		long lStart = (long)( fTime * this->GetSecByte() );
		// 開始位置はSampleSizeで割り切れる箇所からでないといけない。
		if ( lStart > 0 )
		{
			lStart -= lStart % (long)this->SampleSize();
		}

		inStream.seekg( ( (long)this->WavOffset() +	// ファイル内の波形の先頭位置
							  this->StartPos() +	// 管理サウンドの先頭位置
							  lStart ),			 // 算出されたスタート位置
							  est::ios::beg );
		return true;
	}

	
	// 登録
	void WavFormatCtrl::Register( est::istream &inStream )
	{
		// ファイルサイズを取得
		inStream.seekg( 0, est::ios::end );
		int iStreamMax = inStream.tellg();

		inStream.seekg( 0, est::ios::beg );

		int iDmy;
		inStream >> iDmy;	// "RIFF"
		inStream >> iDmy;	// size
		inStream >> iDmy;	// WAVE

		// wavフォーマットInfo	
		ChunkInfo sChunkWork;

		while ( iStreamMax > inStream.tellg() )
		{
			sChunkWork.Register( inStream );

			switch ( sChunkWork.iChunkID )
			{
			// フォーマットなら
			case ChunkInfo::Fmt:
				{
					WAVEFORMATEX sWavFormat;
					inStream >> sWavFormat.wFormatTag;
					inStream >> sWavFormat.nChannels;
					inStream >> sWavFormat.nSamplesPerSec;
					inStream >> sWavFormat.nAvgBytesPerSec;
					inStream >> sWavFormat.nBlockAlign;
					inStream >> sWavFormat.wBitsPerSample;
					// 現状、DirectXではPcm以外サポートしていない。
					GLB_ASSERT( sWavFormat.wFormatTag == WAVE_FORMAT_PCM );
					// wavヘッダの登録
					this->sWavFormat = sWavFormat;
				}
				break;
			// データなら
			case ChunkInfo::Data:
					// Tagには波形データ開始オフセットを入れる。
					this->WavOffset( (long)sChunkWork.lOffset );
					// 波形データサイズ
					this->WavSize( sChunkWork.iChunkSize );
					break;
				// それ以外
				default:
					break;
			}

			sChunkWork.NextChunk( inStream );
		}

		this->ResetPos();

		// ストリームをデータの先頭位置へ
		this->SetTime( inStream, 0.0f );
	}

}
