﻿#ifndef INCLUDE_WAVFORMATCTRL_H
#define INCLUDE_WAVFORMATCTRL_H
#include <windows.h>
#include <mmsystem.h>
#include <dsound.h>
#include "glb/glbDefine.h"

namespace dx
{
	class WavFormatCtrl
	{
	private:
		float			fStart;				// 再生開始タイム
		float			fTime;				// 再生時間
		int				iWavSize;			// WAVサイズ取得・設定
		long			lWavOffset;			// 自由に使える
		bool			bEndLimitEnable;	// 途中終了フラグ
		int				iStartPos;			// ストリームの再生開始位置
		WAVEFORMATEX	sWavFormat;
		int				iEndPos;			// ストリームの再生終了位置

	public:
		int WavSize() const			{ return this->iWavSize; }
		void WavSize( int value )	{ this->iWavSize = value;}

		long WavOffset() const		{ return lWavOffset; }
		void WavOffset( long value ){ this->lWavOffset = value; }
	
		// 終了位置の有効を設定・取得
		// 終了位置が無効の場合は最後まで再生
		bool EndLimit() const	 	{ return bEndLimitEnable; }
		void EndLimit( bool value ) { this->bEndLimitEnable = value; }

		// 開始位置をバイト位置で取得
		int StartPos() const		{ return iStartPos; }

		// 終了位置をバイト位置で取得
		int EndPos() const 			{ return iEndPos; }

		WAVEFORMATEX *Format()		{ return &this->sWavFormat; }

		// 波形の最後のデータ位置
		long LastWavePos() const	{ return this->lWavOffset + this->iWavSize;}


		// １サンプルのバイトが１byte以下の場合(例：4bit)があるので
		// フロートで表現
		float SampleSize() const 
		{
			// サンプルのバイト×チャンネル数
			return (float)this->sWavFormat.wBitsPerSample / 8.0f * this->sWavFormat.nChannels;
		}

		// 再生開始時間
		float Start() const { return fStart; }
		void Start( float value );

		// 再生時間
		float Time() const { return fTime; }
		void Time( float value );


		// コンストラクタ
		WavFormatCtrl ();

		// クローン作成
		WavFormatCtrl *Clone();

		// １秒が何バイトかを取得
		int GetSecByte() const;

		// ストリームを指定の再生位置に移動(時間指定)
		bool SetTime( est::istream &inStream, float fTime );
	
		// 登録
		void Register( est::istream &inStream );
	
	private :
		// コンストラクタ(クローン用)
		WavFormatCtrl( WavFormatCtrl *pBase );

		// 開始、終了位置を再設定
		void ResetPos();
	};
}

#endif // INCLUDE_WAVFORMATCTRL_H