#ifndef INCLUDE_TSMATRIX_H
#define INCLUDE_TSMATRIX_H
#include <cmath>
#include "glb/IGLMatrix.h"


class tsMatrix : public IGLMatrix
{
public:
	float m11, m12, m13,
		  m21, m22, m23, 
		  m31, m32, m33;


	// RXgN^
	tsMatrix ()
	{
		this->Identity();
	}

	// Pʃ}gNXɐݒ
	virtual void Identity()
	{
		this->m11 = this->m22 = this->m33 = 1.0f;
		this->m12 = this->m13 = this->m21 = this->m23 = this->m31 = this->m32 = 0.0f;
	}


	// |Z
	virtual void MulR( const IGLMatrix *pmTarget )
	{
		tsMatrix m;
		m.Copy( (tsMatrix *)pmTarget );
		float r11 = this->m11 * m.m11 + this->m12 * m.m21 + this->m13 * m.m31;
		float r12 = this->m11 * m.m12 + this->m12 * m.m22 + this->m13 * m.m32;
		float r13 = this->m11 * m.m13 + this->m12 * m.m23 + this->m13 * m.m33;
		float r21 = this->m21 * m.m11 + this->m22 * m.m21 + this->m23 * m.m31;
		float r22 = this->m21 * m.m12 + this->m22 * m.m22 + this->m23 * m.m32;
		float r23 = this->m21 * m.m13 + this->m22 * m.m23 + this->m23 * m.m33;
		float r31 = this->m31 * m.m11 + this->m32 * m.m21 + this->m33 * m.m31;
		float r32 = this->m31 * m.m12 + this->m32 * m.m22 + this->m33 * m.m32;
		float r33 = this->m31 * m.m13 + this->m32 * m.m23 + this->m33 * m.m33;
		this->m11 = r11;
		this->m12 = r12;
		this->m13 = r13;
		this->m21 = r21;
		this->m22 = r22;
		this->m23 = r23;
		this->m31 = r31;
		this->m32 = r32;
		this->m33 = r33;
	}

	// |Z(t)
	virtual void Mul( const IGLMatrix *pmTarget )
	{
		tsMatrix m;
		m.Copy( (tsMatrix *)pmTarget );

		float r11 = m.m11 * this->m11 + m.m12 * this->m21 + m.m13 * this->m31;
		float r12 = m.m11 * this->m12 + m.m12 * this->m22 + m.m13 * this->m32;
		float r13 = m.m11 * this->m13 + m.m12 * this->m23 + m.m13 * this->m33;
		float r21 = m.m21 * this->m11 + m.m22 * this->m21 + m.m23 * this->m31;
		float r22 = m.m21 * this->m12 + m.m22 * this->m22 + m.m23 * this->m32;
		float r23 = m.m21 * this->m13 + m.m22 * this->m23 + m.m23 * this->m33;
		float r31 = m.m31 * this->m11 + m.m32 * this->m21 + m.m33 * this->m31;
		float r32 = m.m31 * this->m12 + m.m32 * this->m22 + m.m33 * this->m32;
		float r33 = m.m31 * this->m13 + m.m32 * this->m23 + m.m33 * this->m33;
		this->m11 = r11;
		this->m12 = r12;
		this->m13 = r13;
		this->m21 = r21;
		this->m22 = r22;
		this->m23 = r23;
		this->m31 = r31;
		this->m32 = r32;
		this->m33 = r33;
	}


	// ʒuݒ
	virtual void SetPos(float fX, float fY)
	{
		this->Identity();
		this->m31 = fX;
		this->m32 = fY;
	}
	virtual void SetPos( const glbVector2 *pPos )
	{
		this->SetPos( pPos->X, pPos->Y );
	}


	// XP[ݒ
	virtual void SetScale(float fX, float fY)
	{
		this->m11 = fX;
		this->m22 = fY;
		this->m33 = 1.0f;
		m12 = m13 = m21 = m23 = m31 = m32 = 0.0f;
	}
	virtual void SetScale( const glbVector2 *pScale )
	{
		this->SetScale( pScale->X, pScale->Y );
	}

	// ]Fy
	virtual void SetRotZ(float fZ)
	{
		this->Identity();
		float fSin = std::sin( -fZ );
		float fCos = std::cos( -fZ );

		this->m11 = fCos;
		this->m21 = fSin;
		this->m12 = -fSin;
		this->m22 = fCos;
	}


	// eNX`}gNXpUVZbg
	virtual void SetUV( float fU, float fV )
	{
		this->Identity();
		this->m31 = fU;
		this->m32 = fV;
	}
	virtual void SetUV( const glbVector2 *pUV )
	{
		this->SetUV( pUV->X, pUV->Y );
	}

	// t}gNXɕϊ
	virtual void Invert ()
	{
		float a = this->m22 * this->m33 - this->m23 * this->m32;
		float b = this->m13 * this->m32 - this->m12 * this->m33;
		float c = this->m12 * this->m23 - this->m13 * this->m22;

		float fDetA = 1.0f / (this->m11 * a + this->m21 * b + this->m31 * c);

		float d = this->m23*this->m31 - this->m21*this->m33;
		float e = this->m11*this->m33 - this->m13*this->m31;
		float f = this->m13*this->m21 - this->m11*this->m23;
		float g = this->m21*this->m32 - this->m22*this->m31;
		float h = this->m12*this->m31 - this->m11*this->m32;
		float i = this->m11*this->m22 - this->m12*this->m21;

		this->m11 = a * fDetA;
		this->m12 = b * fDetA;
		this->m13 = c * fDetA;
		this->m21 = d * fDetA;
		this->m22 = e * fDetA;
		this->m23 = f * fDetA;
		this->m31 = g * fDetA;
		this->m32 = h * fDetA;
		this->m33 = i * fDetA;
	}

	// lRs[
	virtual void Copy( const IGLMatrix *pMatrix )
	{
		const tsMatrix *pm = (tsMatrix *)pMatrix;
		this->m11=pm->m11; this->m12=pm->m12; this->m13=pm->m13;
		this->m21=pm->m21; this->m22=pm->m22; this->m23=pm->m23;
		this->m31=pm->m31; this->m32=pm->m32; this->m33=pm->m33;
	}


	// glbVector2 * Matrix
	virtual glbVector2 TransformCoordinate ( const glbVector2 *pMstPos ) const
	{
		glbVector2 vRet;
		vRet.X = this->m11 * pMstPos->X + this->m21 * pMstPos->Y + this->m31 /* * pMstPos->Z*/;
		vRet.Y = this->m12 * pMstPos->X + this->m22 * pMstPos->Y + this->m32 /* * pMstPos->Z*/;
//			vRet.Z = this->m13 * pMstPos->X + this->m23 * pMstPos->Y + this->m33 * pMstPos->Z;

		return vRet;
	}

};
#endif // INCLUDE_TSMATRIX_H
