#include "glbPch.h"
#include "FePointInfo.h"
#include "evc/file/IseStateIO.h"
#include "AnimType/FpAnimHermite.h"
#include "AnimType/FpAnimLiner.h"
#include "AnimType/FpAnimPoint.h"
#include "AnimType/FpAnimSaw.h"

using namespace glb;
using namespace evc;
namespace evc
{
	IFeAnimBase *FePointInfo::AnimFunc()
	{
		return this->pAnimFunc;
	}

	void FePointInfo::CreateAnimFunc(FeAnim::FeAnimType eAnimType)
	{
		GLB_SAFE_DELETE( this->pAnimFunc );

		switch (eAnimType)
		{
			case FeAnim::Liner:
				this->pAnimFunc = evc_new FpAnimLiner();
				break;
			case FeAnim::Point:
				this->pAnimFunc = evc_new FpAnimPoint();
				break;
			case FeAnim::Hermite:
				this->pAnimFunc = evc_new FpAnimHermite();
				break;
			case FeAnim::Saw:
				this->pAnimFunc = evc_new FpAnimSaw();
				break;
	
			case FeAnim::Non:
			default:
				this->pAnimFunc = 0;
				break;
		}
	}

	FePointInfo::FePointInfo()
		: fRate( 0.0f )
		, fValue( 0.0f )
		, pAnimFunc( null )
	{
	}

	FePointInfo::FePointInfo(float fInRate, float fInValue)
		: fRate( fInRate )
		, fValue( fInValue )
		, pAnimFunc( null )
	{
	}

	FePointInfo::FePointInfo(IseStateInput *pInput)
		: fRate( 0.0f )
		, fValue( 0.0f )
		, pAnimFunc( null )
	{
		this->Load(pInput);
	}


	FePointInfo::~FePointInfo()
	{
		GLB_SAFE_DELETE( this->pAnimFunc );
	}



	void FePointInfo::Load(IseStateInput *pInput)
	{
		this->fRate = pInput->ReadFloat();
		this->fValue = pInput->ReadFloat();
	
		FeAnim::FeAnimType eAnimType = pInput->ReadEnum<FeAnim,FeAnim::FeAnimType>();
		this->CreateAnimFunc(eAnimType);
		if (this->pAnimFunc != 0)
		{
			this->pAnimFunc->Load(pInput);
		}
	}

	float FePointInfo::GetPos(float fFrame, FePointInfo *pBfoInfo)
	{
		return (this->pAnimFunc == 0) ? pBfoInfo->Value(): this->pAnimFunc->GetFramePos(fFrame, pBfoInfo, this);
	}

	float FePointInfo::GetPos(float fFrame, const est::vector<FePointInfo*> &lisPoint)
	{
		FePointInfo *pBfoInfo = lisPoint.front();
		for (std::vector<FePointInfo*>::const_iterator pNowInfo = lisPoint.begin(); pNowInfo != lisPoint.end(); pNowInfo++)
		{
			if ((*pNowInfo)->Rate() > fFrame)
			{
				return (*pNowInfo)->GetPos(fFrame, pBfoInfo);
			}
			pBfoInfo = *pNowInfo;
		}
	
		// Ō̃f[^Ԃ
		return lisPoint.back()->Value();
	}
}
