#include "glbPch.h"

#include "evc/scene/seSceneManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/IO/siIOManager.h"
#include "evc/IO/siResource.h"
#include "evc/IO/siFile.h"


using namespace glb;
namespace evc
{

	siResourceBase::siResourceBase( const glb::glbString *strInName )
		: bAutoLoad( true )
		, pFile( null )
		, eState( siResourceState::Unused )
	{
		this->strName = *strInName;
	}


	bool siResourceBase::IsEffective() const
	{
		return (this->eState == siResourceState::Effective);
	}



	void siResourceBase::Load(seSceneManager *pSceneManager)
	{
		// ɏςȂ疳
		if (this->IsEffective())
			return;
	
		switch (this->State())
		{
			// ݏȂ
			case siResourceState::Loading:
			case siResourceState::Initializing:
			case siResourceState::Effective:
				break;
	
			// f[^SȂԂȂ̂Ń[h
			case siResourceState::Unused:
			case siResourceState::Invalidity:
			{
				GLB_ASSERT(this->pFile == null);
				siIOManager *pIOManager = seRootScene::Instance()->IOManager();
				this->pFile = pIOManager->CreateLoader(pSceneManager, this->Name());

				// t@C[hɎs
				this->eState = ( this->pFile == null ) ? siResourceState::Invalidity
													   : siResourceState::Loading;
				break;
			}
			default:
				break;
		}
	
	}

	void siResourceBase::Update()
	{
		switch (this->eState)
		{
			case siResourceState::Unused:
				break;
			// [h
			case siResourceState::Loading:
				GLB_ASSERT(this->pFile != null);
	
				// [hԍXVB
				this->pFile->Update();
				// [hIĂȂ
				if (!this->pFile->IsLoadFinish())
					break;
				// Ɉڍs
				this->eState = siResourceState::Initializing;
				break;
	
			// 
			case siResourceState::Initializing:
				switch (this->Init(this->pFile->Buffer()) )
				{
				case siResourceInit::Success:			// 
					// [_[
					GLB_SAFE_DELETE( this->pFile );
					this->eState = siResourceState::Effective;
					break;
				case siResourceInit::Continuation:		// 
					break;
				case siResourceInit::Failure:			// s
					this->eState = siResourceState::Invalidity;
					break;
				}
				break;
	
			case siResourceState::Effective:
				break;
			case siResourceState::Invalidity:
				break;
		}
	}

	void siResourceBase::Cancel()
	{
		// [_[I
		GLB_SAFE_DELETE( this->pFile );

		// ς݂Ȃ
		if (this->IsEffective())
		{
			delete this;
		}
		this->eState = siResourceState::Unused;
	}

	void siResourceBase::Register( const est::vector<glb::Byte> *abData)
	{
		if (this->IsEffective()) // ɏς
			return;
		this->eState = (this->Init(abData)) ? siResourceState::Effective : siResourceState::Invalidity;
	}




	siResourceSound::siResourceSound( const glb::glbString *pcName )
		: siResourceBase(pcName)
	{
	}


	siResourceTexture::siResourceTexture( const glb::glbString *pcName )
		: siResourceBase(pcName)
	{
	}
}
