#include "glbPch.h"
#include "glb/glbDefine.h"
#include "evc/common/seMath.h"
#include "evc/common/seMatrixWork.h"
#include <math.h>

using namespace glb;

namespace evc
{
	void seMath::Transformation2D( IGLMatrix *pmRet, 
									const glbVector2 *pCenter, 
									const glbVector2 *pScale, 
									float fRot, 
									const glbVector2 *pTranslation)
	{
		// }gNX[NT|[gNX
		seUseWorkMatrix mw;

		IGLMatrix *mp = mw.GetMatrix();
		IGLMatrix *ms = mw.GetMatrix();
		IGLMatrix *mc = mw.GetMatrix();
		IGLMatrix *mci = mw.GetMatrix();
		IGLMatrix *mr = mw.GetMatrix();

		mp->SetPos( pTranslation );
		ms->SetScale( pScale );
		mr->SetRotZ(fRot);
		mc->SetPos( pCenter );
		mci->Copy(mc);
		mci->Invert();

		// o͗p
		pmRet->Copy(mp);
		pmRet->Mul(mc);
		pmRet->Mul(mr);
		pmRet->Mul(ms);
		pmRet->Mul(mci);
	}

	glbVector2 seMath::GetHermiteCurvePoint( const glbVector2 *pPos1, const glbVector2 *pVec1, 
											const glbVector2 *pPos2, const glbVector2 *pVec2, float fRate )
	{
		float r2 = fRate * fRate;
		float ua = (fRate + fRate - 3) * r2;
	
		// xIɑ̗pB(͈󂳂Ȃ:C++ł͂H)
	#if 1
		float fV1 = (fRate - 1) * r2;
		float fP0 = ua + 1;
		float fV0 = fV1 - r2 + fRate;
		float fP1 = -ua;
		return glbVector2( pPos1->X * fP0 + pVec1->X * fV0 + pPos2->X * fP1 + pVec2->X * fV1, 
							pPos1->Y * fP0 + pVec1->Y * fV0 + pPos2->Y * fP1 + pVec2->Y * fV1);
	#else
		float fV1 = (fRate - 1) * r2;
		sPos1->Scale(ua + 1);
		sVec1->Scale(fV1 - r2 + fRate);
		sPos2->Scale(-ua);
		sVec2->Scale(fV1);
	
		sPos1->Add(sVec1);
		sPos1->Add(sVec2);
		sPos1->Add(sPos2);
		return sPos1;
	#endif
	}

	glbVector2 seMath::GetHermiteCurvePoint(
					const glbVector2 *pPos1, const glbVector2 *pVec1, 
					const glbVector2 *pPos2, const glbVector2 *pVec2,
					float fRate, float fPower )
	{
		float r2 = fRate * fRate;
		float ua = (fRate + fRate - 3) * r2;

		float fV1 = (fRate - 1) * r2;
		float fP0 = ua + 1;
		float fV0 = fV1 - r2 + fRate;
		float fP1 = -ua;

		fV1 *= fPower;
		fV0 *= fPower;
		return glbVector2( pPos1->X * fP0 + pVec1->X * fV0 + pPos2->X * fP1 + pVec2->X * fV1, 
						   pPos1->Y * fP0 + pVec1->Y * fV0 + pPos2->Y * fP1 + pVec2->Y * fV1);
	}


	/**
	 * _烉WA擾
	 */
	float seMath::GetRot( const glbVector2 *pStartPos, const glbVector2 *pEndPos )
	{
		glbVector2 v = *pEndPos;
		v.Sub(*pStartPos);
		return atan2f( v.Y, v.X );
	}



	bool seMath::IsHitRectangle( const glbVector2 *pAMin, const glbVector2 *pAMax,
								 const glbVector2 *pBMin, const glbVector2 *pBMax)
	{
		if (pAMin->X > pBMax->X) return false;
		if (pAMax->X < pBMin->X) return false;
		if (pAMin->Y > pBMax->Y) return false;
		if (pAMax->Y < pBMin->Y) return false;
		return true;
	}

	float seMath::GetDistance( const glbVector2 *p1, const glbVector2 *p2)
	{
		float dx = p1->X - p2->X;
		float dy = p1->Y - p2->Y;
		return ::sqrt(dx * dx + dy * dy);
	}

	bool seMath::CheckSide( const glbVector2 *pStartPos, const glbVector2 *pEndPos, const glbVector2 *pCheckPos )
	{
		return (pEndPos->X - pStartPos->X) * (pCheckPos->Y - pStartPos->Y) - (pCheckPos->X - pStartPos->X) * (pEndPos->Y - pStartPos->Y) >= 0.0f ? false : true;
	}

	bool seMath::IsInRect( const est::vector<glbVector2> *pvPoint, const glbVector2 *pPos )
	{
		int iPointNum = pvPoint->size();
		// |CgQȉȂ疳
		if (iPointNum < 3)
			return false;
	
		// Ō̓_n_܂ł̍E
		bool bSide = seMath::CheckSide( &(*pvPoint)[iPointNum - 1], &(*pvPoint)[0], pPos);
	
		// SĂ̔肪ȂtrueȊOfalse
		for (int i = 0; i < (iPointNum - 1); i++)
		{
			if (bSide != seMath::CheckSide( &(*pvPoint)[i], &(*pvPoint)[i + 1], pPos))
				return false;
		}
		return true;
	}

	bool seMath::IsInRect( const glbVector2 *pSize, 
							const IGLMatrix *pmParent, 
							const glbVector2 *pPos)
	{
		// }gNX[NT|[gNX
		seUseWorkMatrix gw;

		IGLMatrix *m = gw.GetMatrix();
		m->Copy(pmParent);
		m->Invert();
		glbVector2 sPos = m->TransformCoordinate(pPos);

		// Size}CiX̏ꍇ͒`ЂԂĂƂ݂Ȃ
		// B
		if (sPos.X < glb::MinValue(pSize->X, 0.0f) ) return false;
		if (sPos.X > glb::MaxValue(pSize->X, 0.0f) ) return false;
		if (sPos.Y < glb::MinValue(pSize->Y, 0.0f) ) return false;
		if (sPos.Y > glb::MaxValue(pSize->Y, 0.0f) ) return false;

		// qbg
		return true;
	}


	bool seMath::IsCrossLine( const glbVector2 *pA1, const glbVector2 *pA2, 
							  const glbVector2 *pB1, const glbVector2 *pB2 )
	{
		// _̂Ƃ͌ĂȂƂ
		// ɒ[ɋ߂ꍇŕ\
		if (seMath::GetDistance( pA1, pA2) < 0.0001f) return false;
		if (seMath::GetDistance( pB1, pB2) < 0.0001f) return false;
	
		glbVector2 vA = glbVector2( pA2->X - pA1->X, pA2->Y - pA1->Y );
		glbVector2 vB = glbVector2( pB2->X - pB1->X, pB2->Y - pB1->Y );
	
		// Oς̒
		float fDeno = vA.X * vB.Y - vA.Y * vB.X;
		// sȂƂ͌ĂȂƂɂ
		// floaẗׁAȂsɋ߂ꍇƂB
		float fDenoPower = (fDeno < 0.0f)? -fDeno : fDeno;
		if ( fDenoPower < 0.0001f)
			return false;
	
		glbVector2 v1 = glbVector2( pA1->X - pB1->X, pA1->Y - pB1->Y );
	
		float t = (v1.Y * vB.X - v1.X * vB.Y) / fDeno;
		if (t < 0.0f || t > 1.0f) return false;
		float s = (v1.Y * vA.X - v1.X * vA.Y) / fDeno;
		if (s < 0.0f || s > 1.0f) return false;
	
		return true;
	}

	bool seMath::IsHitPoly( const est::vector<glbVector2> *paList,
							const est::vector<glbVector2> *pbList )
	{
		if (paList == null) return false;
		if (pbList == null) return false;
	
		if (paList->size() <= 0) return false;
		if (pbList->size() <= 0) return false;
	
		glbVector2 aMin = (*paList)[0];
		glbVector2 aMax = (*paList)[0];
		glbVector2 bMin = (*pbList)[0];
		glbVector2 bMax = (*pbList)[0];
	
		// Wϊ
		for (int i = 1; i < (int)paList->size(); i++)
		{
			// ŏEő咸_쐬
			aMin.X = glb::MinValue((*paList)[i].X, aMin.X);
			aMin.Y = glb::MinValue((*paList)[i].Y, aMin.Y);
			aMax.X = glb::MaxValue((*paList)[i].X, aMax.X);
			aMax.Y = glb::MaxValue((*paList)[i].Y, aMax.Y);
		}
	
		for (int i = 1; i < (int)pbList->size(); i++)
		{
			bMin.X = glb::MinValue((*pbList)[i].X, bMin.X);
			bMin.Y = glb::MinValue((*pbList)[i].Y, bMin.Y);
			bMax.X = glb::MaxValue((*pbList)[i].X, bMax.X);
			bMax.Y = glb::MaxValue((*pbList)[i].Y, bMax.Y);
		}
	
		// oEfBO{bNX`FbN
		if (!seMath::IsHitRectangle( &aMin, &aMax, &bMin, &bMax))
			return false;
	
	
		// ӓm̌`FbN
		const glbVector2 *pBfoAPos = &(*paList)[paList->size() - 1];
		for ( int i = 0; i < (int)paList->size() - 1; i++ )
		{
			const glbVector2 *pAVector = &(*paList)[ i ];

			const glbVector2 *pBfoBPos = &(*pbList)[pbList->size() - 1];
			for ( int j = 0; j < (int)pbList->size() - 1; j++ )
			{
				const glbVector2 *pBVector = &(*pbList)[ j ];

				// mȂ
				if (seMath::IsCrossLine(pBfoAPos, pAVector, pBfoBPos, pBVector))
					return true;
	
				pBfoBPos = pBVector;
			}
			pBfoAPos = pAVector;
		}
	
		// ǂ炩̋`ЕɊSɓĂ܂Ă邩`FbN
		// ӂ͌ȂĂ_`ɂꍇB
		for ( int i = 0; i < (int)pbList->size(); i++ )
		{
			if (seMath::IsInRect(paList, &(*pbList)[i]))
				return true;
		}
	
		for ( int i = 0; i < (int)paList->size(); i++ )
		{
			if (seMath::IsInRect(pbList, &(*paList)[i]))
				return true;
		}
	
		// qbgĂȂB
		return false;
	}
}
