#include "glbPch.h"
#include "evCmd.h"
#include "evCmdBase.h"
#include "evManager.h"
#include "evInputControl.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siIOManager.h"
#include "evc/common/seMath.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seObject/ObjectParts/seoScene.h"

using namespace glb;
namespace evc
{


	static bool DoInTopRect(IevCmdUI *cmdUI, evCmd *target)
	{
		if (!cmdUI->Target()->IsVisible()) return false;
		return (cmdUI->Target()->Hit() == seHit::TopHit);
	}

	static bool DoInRect(IevCmdUI *cmdUI, evCmd *target)
	{
		if (!cmdUI->Target()->IsVisible()) return false;
		switch (cmdUI->Target()->Hit())
		{
		case seHit::TopHit:
		case seHit::Hit:
			return true;
		default:
			break;
		}
		return false;
	}

	static bool DoOutRect(IevCmdUI *cmdUI, evCmd *target)
	{
		if (!cmdUI->Target()->IsVisible())
			return false;
		return (cmdUI->Target()->Hit() == seHit::Non);
	}

	static bool DoHitRect(IevCmdUI *cmdUI, evCmd *target)
	{
		int iElemID = target->Data()[0]->Value();
		const seElemManager *em = cmdUI->SceneManager()->ElemManager();
		seElem *elem = em->seIDManager<seElem*>::Find(iElemID);

		if ((elem == null) || (!elem->IsVisible())) return false;
		seElem *elemTarget = cmdUI->Target();
		if ((elemTarget == null) || (!(elemTarget->IsVisible())))
			return false;

		est::vector<glbVector2> asPos;
		elem->GetRect(asPos);
		est::vector<glbVector2> asTargetPos;
		elemTarget->GetRect(asTargetPos);

		return seMath::IsHitPoly( &asPos, &asTargetPos);
	}

	static bool DoInRectPoint(IevCmdUI *cmdUI, evCmd *target)
	{
		if ((cmdUI->Target() == null) || (!cmdUI->Target()->IsVisible()))
			return false;
		glbVector2 pos( static_cast<float>(target->Data()[0]->Value()),
						static_cast<float>(target->Data()[1]->Value()));
		return cmdUI->Target()->IsInRect(&pos);
	}

	static bool DoOutRectPoint(IevCmdUI *cmdUI, evCmd *target)
	{
		if ((cmdUI->Target() == null) || (!cmdUI->Target()->IsVisible()))
			return false;
		glbVector2 pos( static_cast<float>(target->Data()[0]->Value()),
						static_cast<float>(target->Data()[1]->Value()) );
		return !cmdUI->Target()->IsInRect(&pos);
	}

	static bool DoIsState(IevCmdUI *cmdUI, evCmd *target)
	{
		evdVariableStateItem *item = static_cast<evdVariableStateItem*>(target->Data()[0]);
		if (item->DataVariable() == null) return false;
		return item->DataVariable()->IsOn(item->ItemID());
	}

	static bool DoIsNotState(IevCmdUI *cmdUI, evCmd *target)
	{
		evdVariableStateItem *item = static_cast<evdVariableStateItem*>(target->Data()[0]);
		if (item->DataVariable() == null) return false;
		return !item->DataVariable()->IsOn(item->ItemID());
	}

	static bool DoComparisonValue(IevCmdUI *cmdUI, evCmd *target)
	{
		int dat1 = target->Data()[0]->Value();
		int dat2 = target->Data()[1]->Value();
		switch (target->Data()[1]->Operator())
		{
		case evOperator::Over: // >
			return (dat1 > dat2) ? true : false;
		case evOperator::Under: // <
			return (dat1 < dat2) ? true : false;
		case evOperator::Same: // ==
			return (dat1 == dat2) ? true : false;
		case evOperator::Diff: // !=
			return (dat1 != dat2) ? true : false;
		case evOperator::SameOrOver: // !=
			return (dat1 >= dat2) ? true : false;
		case evOperator::SameOrUnder: // !=
			return (dat1 <= dat2) ? true : false;
		default:
			break;
		}
		return false;
	}

	static bool DoIsPush(IevCmdUI *cmdUI, evCmd *target)
	{
		return evInputControl::Instance().IsPush(static_cast<ButtonType::Type>(target->Data()[0]->Value()));
	}

	static bool DoIsFirstPush(IevCmdUI *cmdUI, evCmd *target)
	{
		return evInputControl::Instance().IsFirstPush(static_cast<ButtonType::Type>(target->Data()[0]->Value()));
	}

	static bool DoIsPop(IevCmdUI *cmdUI, evCmd *target)
	{
		return !evInputControl::Instance().IsPush(static_cast<ButtonType::Type>(target->Data()[0]->Value()));
	}

	static bool DoIsStartAnim(IevCmdUI *cmdUI, evCmd *target)
	{
		evCmdData *data = target->Data()[0];
		return cmdUI->SceneManager()->IsPlayNextAnim(data->Value());
	}

	static bool DoMovePoint(IevCmdUI *cmdUI, evCmd *target)
	{
		return evInputControl::Instance().IsMovePointer();
	}




	// ߍ쐬
	evCmdCond *evCmdList::CraeteCmd(evCond::Type eCmd, IevCmdUI *ui)
	{
		evCmdCond *cmd = new evCmdCond();
		cmd->CmdType( eCmd );
	
		// ̃\bhł͓f[^͍ȂB
		// [hAҏWɍB
	
		switch (eCmd)
		{
			// `F|C^ƍŏɃqbg
			case evCond::InTopRect:
				cmd->pOnFunc = DoInTopRect;
				break;
			// `F|C^ƃqbgĂ
			case evCond::InRect:
				cmd->pOnFunc = DoInRect;
				break;
			// `F|C^qbgĂȂ
			case evCond::OutRect:
				cmd->pOnFunc = DoOutRect;
				break;
			// `F`ƃqbgĂ
			case evCond::HitRect:
				cmd->pOnFunc = DoHitRect;
				break;
			// `FCӂ̍WƃqbgĂ
			case evCond::InRectPoint:
				cmd->pOnFunc = DoInRectPoint;
				break;
			// `FCӂ̍WƃqbgĂȂ
			case evCond::OutRectPoint:
				cmd->pOnFunc = DoOutRectPoint;
				break;
			// Xe[^XϐFĂ
			case evCond::IsState:
				cmd->pOnFunc = DoIsState;
				break;
	
			// Xe[^XϐFĂȂ
			case evCond::IsNotState:
				cmd->pOnFunc = DoIsNotState;
				break;
	
			// lr
			case evCond::ComparisonValue:
				cmd->pOnFunc = DoComparisonValue;
				break;
	
			// {^F
			case evCond::IsPush:
				cmd->pOnFunc = DoIsPush;
				break;
			// {^Fn߂
			case evCond::IsFirstPush:
				cmd->pOnFunc = DoIsFirstPush;
				break;
			// {^F
			case evCond::IsPop:
				cmd->pOnFunc = DoIsPop;
				break;
	
			// Aj[VFJn
			case evCond::IsStartAnim:
				cmd->pOnFunc = DoIsStartAnim;
				break;
			// |CgFړ
			case evCond::MovePoint:
				cmd->pOnFunc = DoMovePoint;
				break;
	
			// Rg
			case evCond::Comment:
				break;
			// ̑(Ȃ)
			default:
				break;
		}
	
		// ߕ
		return cmd;
	}






	static bool DoAnimPlay(IevCmdUI *cmdUI, evCmd *target)
	{
		cmdUI->SceneManager()->RequestAnim(target->Data()[0]->Value());
		return true;
	}

	static bool DoAnimPause(IevCmdUI *cmdUI, evCmd *target)
	{
		cmdUI->Target()->StopAnim( true );
		return true;
	}

	static bool DoAnimContinue(IevCmdUI *cmdUI, evCmd *target)
	{
		cmdUI->Target()->StopAnim( false );
		cmdUI->Target()->StopAnimAll( false );
		return true;
	}

	static bool DoSceneAnim(IevCmdUI *cmdUI, evCmd *target)
	{
		evdString *s = static_cast<evdString*>(target->Data()[0]);
		seObjectWork *objWork = cmdUI->Target()->ObjectWork();
		if (objWork->Object()->ElemType() == emObject::Scene)
		{
			seoSceneWork *scene = static_cast<seoSceneWork*>(objWork);
			scene->LocalScene()->RequestAnim(s->Text());
		}
		return true;
	}

	static bool DoSetStatus(IevCmdUI *cmdUI, evCmd *target)
	{
		evdVariableStateItem *item = static_cast<evdVariableStateItem*>(target->Data()[0]);
		evVariableState *st = item->DataVariable();
		if (st == null) return true;
		st->Value( st->Value() | item->ItemID() );
		return true;
	}

	static bool DoRemoveStatus(IevCmdUI *cmdUI, evCmd *target)
	{
		evdVariableStateItem *item = static_cast<evdVariableStateItem*>(target->Data()[0]);
		evVariableState *st = item->DataVariable();
		if (st == null) return true;
		st->Value( st->Value() & ~item->ItemID() );
		return true;
	}

	static bool DoClearStatus(IevCmdUI *cmdUI, evCmd *target)
	{
		evdVariableState *item = static_cast<evdVariableState*>(target->Data()[0]);
		evVariableState *st = item->DataVariable();
		if (st == null) return true;
		st->Value( 0 );
		return true;
	}

	static bool DoOperatorValue(IevCmdUI *cmdUI, evCmd *target)
	{
		int iArgValue = target->Data()[1]->Value();
		evCmdData *pCmdData = target->Data()[ 0 ];

		switch (target->Data()[1]->Operator())
		{
		case evOperator::Add: // +=
			pCmdData->Value( pCmdData->Value() + iArgValue );
			break;
		case evOperator::Sub: // -=
			pCmdData->Value( pCmdData->Value() - iArgValue );
			break;
		case evOperator::Set: // =
			pCmdData->Value( iArgValue );
			break;
		case evOperator::Mul: // *=
			pCmdData->Value( pCmdData->Value() * iArgValue );
			break;
		case evOperator::Div: // /=
			pCmdData->Value( pCmdData->Value() / iArgValue );
			break;
		case evOperator::Quotient: // %=
			pCmdData->Value( pCmdData->Value() % iArgValue );
			break;
		default:
			break;
		}
		return true;
	}

	static bool DoDebugString(IevCmdUI *cmdUI, evCmd *target)
	{
		evdString *s = static_cast<evdString*>(target->Data()[0]);
		seRootScene::Instance()->IOManager()->DebugString(s->Text());
		return true;
	}

	static bool DoDbWriteNum(IevCmdUI *cmdUI, evCmd *target)
	{
		// 
		// dynamic_cast
#if 0

		StringBuilder *sb = new StringBuilder();
		evCmdData *cmdData = target->Data()[0];


		if (dynamic_cast<evdVariableNum*>(cmdData) != 0)
		{
			evdVariableNum *num = static_cast<evdVariableNum*>(cmdData);
			if (num->DataVariable() == 0)
				return true;
			sb->Append(num->DataVariable()->Name);
			sb->Append(" : ");
			sb->Append(num->DataVariable()->Value());
		}
		else if (dynamic_cast<evdVariableState*>(cmdData) != 0)
		{
			evdVariableState *state = static_cast<evdVariableState*>(cmdData);
			evVariableState *st = state->DataVariable;
			if (st == 0)
				return true;
			sb->Append(st->Name);
			sb->Append(" : ");
			sb->Append(st->Value());
		}
		seRootScene::Instance::IOManager::DebugString(sb->ToString());
#endif
		return true;
	}




	// Cxgߍ쐬
	evCmdEvent *evCmdList::CraeteCmd(evEvent::Type eCmd, IevCmdUI *ui)
	{
		evCmdEvent *cmd = new evCmdEvent();
		cmd->CmdType( eCmd );
	
		// ̃\bhł͓f[^͍ȂB
		// [hAҏWɍB
	
		switch (eCmd)
		{
			// Aj[VFĐ
			case evEvent::AnimPlay:
				cmd->pOnFunc = DoAnimPlay;
				break;
			// Aj[VF~
	//				case evEvent.AnimStop: inHeader = new evCmdAnimStop(); break;
			// Aj[VFꎞ~
			case evEvent::AnimPause:
				cmd->pOnFunc = DoAnimPause;
				break;
	
			// Aj[VFĊJ
			case evEvent::AnimContinue:
				cmd->pOnFunc = DoAnimContinue;
				break;
	
			// Aj[VFWv
	//				case evEvent.AnimJump: inHeader = new evCmdAnimJump(); break;
	
			// Aj[VĐ
			case evEvent::SceneAnim:
				cmd->pOnFunc = DoSceneAnim;
				break;
	
			// ۗ
	
			// ́Fꎞ~
			case evEvent::InputPause:
				break;
			// ́FĊJ
			case evEvent::InputContinue:
				break;
	
	
			// Xe[^XϐFݒ
			case evEvent::SetStatus:
				cmd->pOnFunc = DoSetStatus;
				break;
			// Xe[^XϐF폜
			case evEvent::RemoveStatus:
				cmd->pOnFunc = DoRemoveStatus;
				break;
	
			// Xe[^XϐFNA
			case evEvent::ClearStatus:
				cmd->pOnFunc = DoClearStatus;
				break;
	
			// lZ
			case evEvent::OperatorValue:
				cmd->pOnFunc = DoOperatorValue;
				break;
	
			// fobOFo
			case evEvent::DebugString:
				cmd->pOnFunc = DoDebugString;
				break;
	
			// fobOFϐo
			case evEvent::DbWriteNum:
				cmd->pOnFunc = DoDbWriteNum;
				break;
	
			// Rg
			case evEvent::Comment:
				break;
	
			// T|[g(Ȃ)
			default:
				break;
		}
	
		// ߕ
		return cmd;
	}


	// evCmd
	// 

	evCmd::evCmd()
	{

	}


	evCmd::~evCmd()
	{
		glb::DeleteVector( this->lisData );
	}


	void evCmd::AddData(evCmdData *data)
	{
		this->lisData.push_back(data);
	}

	void evCmd::SetData(evCmdData *data, int iIndex)
	{
		delete this->lisData[ iIndex ];
		this->lisData[ iIndex ] = data;
	}


	evCmd::evCmd(evCmd *mst) : evCmdHeader(mst)
	{
		this->pOnFunc = mst->pOnFunc;

		this->lisData.clear();
		est::vector<evCmdData*>::iterator it = mst->lisData.begin();
		for(;it != mst->lisData.end(); it++)
		{
			this->lisData.push_back( static_cast<evCmdData*>((*it)->Clone()));
		}
	}

	bool evCmd::DoFunc(IevCmdUI *cmd)
	{
		if (this->pOnFunc == null) return true;
		return (*this->pOnFunc)(cmd, this);
	}

	void evCmd::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdHeader::Load(pReader, pCmdUI);
	
		int dataCount = pReader->ReadInt();
		for (int i = 0; i < dataCount; i++)
		{
			evDataType::Type eType = pReader->ReadEnum<evDataType,evDataType::Type>();
			evCmdData *cmdData = evData::CreateData(eType, pCmdUI);
			cmdData->Load(pReader, pCmdUI);
	
			this->lisData.push_back(cmdData);
		}
	}

	// evCmdEvent
	// 

	evCmdEvent::evCmdEvent()
	{
	}

	evCmdEvent::evCmdEvent(evCmdEvent *mst) : evCmd(mst)
	{
		this->CmdType( mst->CmdType() );
	}

	evCmdHeader *evCmdEvent::Clone()
	{
		return new evCmdEvent(this);
	}




	// evCmdCond
	// 

	evCmdCond::evCmdCond()
	{
	}

	evCmdCond::evCmdCond(evCmdCond *mst) : evCmd(mst)
	{
		this->CmdType( mst->CmdType() );
	}

	evCmdHeader *evCmdCond::Clone()
	{
		return new evCmdCond(this);
	}
}
