#include "glbPch.h"
#include "glb/glbDebug.h"
#include "glb/glbString.h"
#include "evc/common/seDefine.h"
#include "evc/file/IseStateIO.h"
#include "evCmd.h"
#include "evData.h"
#include "evCmdFlowIf.h"

using namespace evc;
using namespace glb;
namespace evc
{
	// RXgN^(N[p)
	evCmdFlowCondManager::evCmdFlowCondManager( evCmdFlowCondManager *pCondManager )
	{
		if ( pCondManager == null) return;
		est::vector<evCmdCond*>::iterator it = pCondManager->lisCondCmd.begin();
		for(; it != pCondManager->lisCondCmd.end(); it ++)
		{
			this->Add( evc_new evCmdCond( *it ) );
		}
	}



	evCmdFlowCondManager::~evCmdFlowCondManager()
	{
		glb::DeleteVector( this->lisCondCmd );
	}

	void evCmdFlowCondManager::Add(evCmdCond *pCmd)
	{
		this->lisCondCmd.push_back(pCmd);
	}

	void evCmdFlowCondManager::Remove(evCmdCond *pCmd)
	{
		est::vector<evCmdCond*>::iterator it;
		it = est::find( this->lisCondCmd.begin(), this->lisCondCmd.end(), pCmd );
		this->lisCondCmd.erase( it );
	}


	bool evCmdFlowCondManager::IsCondAll(IevCmdUI *pCmdUI)
	{
		// ꍇ͖ōi
		if (this->lisCondCmd.size() <= 0)
			return true;
	
		for (est::vector<evCmdCond*>::const_iterator pCond = this->lisCondCmd.begin();
				pCond != this->lisCondCmd.end(); pCond++)
		{
			// LŖꍇ̓X[Ȃ̂ōi
			if (!(*pCond)->Enable()) continue;
			if (!(*pCond)->DoFunc(pCmdUI)) return false;
		}
		return true;
	}

	void evCmdFlowCondManager::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->lisCondCmd.clear();
		int iCount = static_cast<int>(pReader->ReadShort());
		seDefine::DebugTrace("CondCount", iCount);
		for (int i = 0; i < iCount; i++)
		{
			evCond::Type eCond = static_cast<evCond::Type>(pReader->ReadByte());
//			seDefine::DebugTrace(eCond->ToString(), 0);
			evCmdCond *pCmdCond = evCmdList::CraeteCmd(eCond,pCmdUI);
			pCmdCond->Load(pReader, pCmdUI);
			this->Add(pCmdCond);
		}
	}


	evCmdFlowCondManager *evCmdFlowCondManager::Clone()
	{
		evCmdFlowCondManager *pCondManager = evc_new evCmdFlowCondManager();

		const est::vector<evCmdCond*> &cmds = *this->Cmds();
		est::vector<evCmdCond*>::const_iterator it = cmds.begin();
		while( it != cmds.end() )
		{
			pCondManager->Add(static_cast<evCmdCond*>((*it)->Clone()));
			it ++;
		}

		return pCondManager;
	}

	//==========================================
	// evCmdFlowIf
	// 


	void evCmdFlowIf::NextFlow(evCmdFlowIf *value)
	{
		if (value != null)
		{
			value->Parent( this->Parent() );
			value->pBefoFlow = this;
		}
		this->pNextFlow = value;
	}

	evCmdFlowIf::evCmdFlowIf()
	{
		this->NextFlow(null);
		this->pCondManager = evc_new evCmdFlowCondManager();
	}

	evCmdFlowIf::evCmdFlowIf( evCmdFlowIf *pFlowIf )
		: evCmdHeader( pFlowIf )
	{
		evCmdFlowIf *pNextIf = pFlowIf->NextFlow();
		if ( pNextIf != null)
		{
			pNextIf = evc_new evCmdFlowIf( pNextIf );
		}
		this->NextFlow( pNextIf );

		this->CondManager( evc_new evCmdFlowCondManager(pFlowIf->CondManager()) );

		est::vector<evCmdHeader*>::iterator it = pFlowIf->lisCmd.begin();
		for(;it != pFlowIf->lisCmd.end(); it++)
		{
			this->Add( (*it)->Clone() );
		}
	}


	evCmdFlowIf::~evCmdFlowIf()
	{
		glb::DeleteVector( this->lisCmd );
		GLB_SAFE_DELETE( this->pCondManager );
		GLB_SAFE_DELETE( this->pNextFlow );
	}

	void evCmdFlowIf::Add(evCmdHeader *pCmd)
	{
		this->lisCmd.push_back(pCmd);
		// t[Ȃeݒ
		pCmd->Parent( this );
	}

	void evCmdFlowIf::Remove(evCmdHeader *pCmd)
	{
		est::vector<evCmdHeader*>::iterator it;
		it = est::find( this->lisCmd.begin(), this->lisCmd.end(), pCmd );
		this->lisCmd.erase( it );
		// e폜
		pCmd->Parent( null );
	}


	bool evCmdFlowIf::DoAll(deFunc1<evCmdHeader*, bool> *pFunc)
	{
		if (!(*pFunc)(this)) return false;
	
		// if Ȃs
		{
			const est::vector<evCmdCond*> &cmds = *this->CondManager()->Cmds();
			est::vector<evCmdCond*>::const_iterator it = cmds.begin();
			while( it != cmds.end() )
			{
				if (!(*pFunc)(*it) ) return false;
				it ++;
			}
		}

		// Cxgɑ΂ďs
		{
			const est::vector<evCmdHeader*> &headers = *this->Cmds();
			est::vector<evCmdHeader*>::const_iterator it = headers.begin();
			while( it != headers.end() )
			{
				evCmdHeader *pHeader = (*it);
				if (pHeader->Type() == evCmdType::FlowIf) 
				{ 
					evCmdFlowIf *f = static_cast<evCmdFlowIf*>(pHeader); 
					if (!f->DoAll(pFunc)) return false;
				} 
				else
				{
					if (!(*pFunc)(pHeader)) return false;
				}

				it ++;
			}
		}


		// ̘At[Ȃ炻s
		if (this->NextFlow() != null)
		{
			return this->NextFlow()->DoAll(pFunc);
		}
	
		return true;
	}
	
	
	// Cxg̎s
	// true:s@false:sȂ
	void evCmdFlowIf::DoEvent(IevCmdUI *pCmdUI)
	{
		// s
		est::vector<evCmdHeader*>::iterator it = this->lisCmd.begin();
		while( it != this->lisCmd.end() )
		{
			if ((*it)->Enable())
			{
				(*it)->DoFunc( pCmdUI );
			}
			it ++;
		}
	}
	
	
	// N[FĐpłB
	// ҏWŎg̃f[^̓Rs[Ă܂B
	evCmdHeader* evCmdFlowIf::Clone()
	{
		return evc_new evCmdFlowIf( this );
	}
	
	
	// 
	bool evCmdFlowIf::DoFunc(IevCmdUI *pCmdUI)
	{
		// t[LłA𖞂Ȃ
		if (this->Enable() && this->pCondManager->IsCondAll(pCmdUI))
		{
			this->DoEvent(pCmdUI);
		}
		else if (this->NextFlow() != null)
		{
			this->NextFlow()->DoFunc(pCmdUI);
		}
		return true;
	}
	
	// f[^[h
	void evCmdFlowIf::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		int iCount = static_cast<int>(pReader->ReadShort());
		seDefine::DebugTrace("CmdCount", iCount);
		for (int i = 0; i < iCount; i++)
		{
			// ^Cvʏ
			evCmdHeader *pHeader = null;
			LoadCmdType::Type eCmdType = static_cast<LoadCmdType::Type>(pReader->ReadByte());

//			seDefine::DebugTrace(eCmdType.ToString(), 0);
			switch (eCmdType)
			{
				case LoadCmdType::FlowIf:
					pHeader = evc_new evCmdFlowIf();
					break;
				case LoadCmdType::Event:
					{
						evEvent::Type eEvent = static_cast<evEvent::Type>(pReader->ReadByte());
						pHeader = evCmdList::CraeteCmd(eEvent,pCmdUI);
					}
					break;
				default:
					GLB_ASSERT(false);
					break;
			}
	
			pHeader->Load(pReader, pCmdUI);
			this->Add(pHeader);
		}
	
		this->CondManager()->Load(pReader, pCmdUI);
	
		LoadCmdType::Type eNextType = static_cast<LoadCmdType::Type>(pReader->ReadByte());
		evCmdFlowIf *pFlow = null;
		switch (eNextType)
		{
			case LoadCmdType::FlowIf:
				pFlow = evc_new evCmdFlowIf();
				break;
			case LoadCmdType::FlowNon:
				break;
			default:
				GLB_ASSERT(false);
				break;
		}
	
		// eRs[
		this->NextFlow(pFlow);
		if (pFlow != null)
		{
			pFlow->Load(pReader, pCmdUI);
		}
	}
}

