#include "glbPch.h"
#include "evData.h"
#include "evCmd.h"
#include "evCmdBase.h"
#include "evManager.h"
#include "evInputControl.h"
#include "evc/IO/siEventIO.h"
#include "evc/IO/listener/siEventListener.h"
#include "evc/file/IseStateIO.h"
#include "evc/common/seMath.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seKeyControlManager.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemAnim/seStateKeyInfo.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seObject/ObjectParts/seoScene.h"

using namespace glb;
namespace evc
{
	// glbVector2̗vfItZbgŕԂ
	static float GetVectorElement( const glbVector2 *v, int offset )
	{
		switch (offset)
		{
			case 0: return v->X;
			case 1: return v->Y;
			default: break;
		}
		return 0.0f;
	}

	// glbVector2̔Cӂ̃ItZbg̗vfݒ
	static glbVector2 SetVectorElement( const glbVector2 *v, int offset, float value )
	{
		glbVector2 ret = *v;
		switch (offset)
		{
			case 0: ret.X = value; break;
			case 1: ret.Y = value; break;
			default: break;
		}
		return ret;
	}


	int evData::GetValue(IseStateType *stype, emElemState::Type stateType, int offset)
	{
		// o͐ʏ
		switch (stateType)
		{
			case emElemState::Pos:
				return static_cast<int>( GetVectorElement( 
						(static_cast<seStatePos*>(stype))->Value(),
						offset));

			case emElemState::Scale:
			case emElemState::UV:
			case emElemState::Center:
				return static_cast<int>( GetVectorElement( 
						(static_cast<seStateFloat2*>(stype))->Value(),
						offset));
	
			case emElemState::Rot:
				return static_cast<int>((static_cast<seStateRot*>(stype))->Rot() * 180.0f / glb::PI);
	
			case emElemState::Color:
				{
					const glbColor *c = (static_cast<seStateColor*>(stype))->Color();
					switch (offset)
					{
						case 0: return static_cast<int>(c->R);
						case 1: return static_cast<int>(c->G);
						case 2: return static_cast<int>(c->B);
						case 3: return static_cast<int>(c->A);
						default: break;
					}
				}
				break;
	
			case emElemState::Volume:
			case emElemState::Pan:
			case emElemState::Num:
				return (static_cast<seStateInt1*>(stype))->Value();
	
			// T|[gO̐ݒ
			default:
				break;
		}
	
		GLB_ASSERT(false);
		return 0;
	}

	void evData::SetValue(IseStateType *stype, emElemState::Type stateType, int offset, int value)
	{
		// o͐ʏ
		switch (stateType)
		{
			case emElemState::Pos:
				{
					seStatePos *p2 = static_cast<seStatePos*>(stype);
					glbVector2 w = SetVectorElement( p2->Value(),
									offset, static_cast<float>(value));
					p2->Value( &w );
				}
				break;
			case emElemState::Scale:
			case emElemState::UV:
			case emElemState::Center:
				{
					seStateFloat2 *f2 = static_cast<seStateFloat2*>(stype);
					glbVector2 w = SetVectorElement(f2->Value(),
								offset, static_cast<float>(value));
					f2->Value(&w);
				}
				break;
	
			case emElemState::Rot:
				(static_cast<seStateRot*>(stype))->Rot(
						static_cast<float>(value) * 
						glb::PI / 180.0f );
				break;
	
			case emElemState::Color:
				{
					seStateColor *cs = static_cast<seStateColor*>(stype);
					glbColor c = *cs->Color();
					switch (offset)
					{
						case 0: c.R = static_cast<unsigned char>(value); break;
						case 1: c.G = static_cast<unsigned char>(value); break;
						case 2: c.B = static_cast<unsigned char>(value); break;
						case 3: c.A = static_cast<unsigned char>(value); break;
						default: break;
					}
					cs->Color( &c );
				}
				break;
	
			case emElemState::Volume:
			case emElemState::Pan:
			case emElemState::Num:
				(static_cast<seStateInt1*>(stype))->Value(value);
				break;
			// T|[gO̐ݒ
			default:
				GLB_ASSERT(false);
				break;
		}
	}

	evCmdData *evData::CreateData(evDataType::Type eType, IevCmdUI *cmdUI)
	{
		switch (eType)
		{
			case evDataType::AnimID:
				return new evdInt(eType, seAnimManager::InvalidID);
			case evDataType::Button:
				return new evdInt(eType, (int)ButtonType::Non);
			case evDataType::Elem:
				return new evdInt(eType, seElemManager::InvalidID);
			case evDataType::Int:
				return new evdInt(eType, 0);
	
			case evDataType::UserCall:
				return new evdUserCall(cmdUI->Target());
	
			case evDataType::String:
			case evDataType::DebugMsg:
				return new evdString(eType, 0);
	
			case evDataType::ElemState:
				return new evdElemState(cmdUI->Target());
	
			case evDataType::KeyName:
				return new evdKeyValue(cmdUI->Target(),
						cmdUI->SceneManager()->KeyControlManager());
	
			case evDataType::VariableNum:
				return new evdVariableNum();
	
			case evDataType::VariableState:
				return new evdVariableState();
	
			case evDataType::VariableStateItem:
				return new evdVariableStateItem();
	
			default:
				break;
		}
	
		return null;
	}



	evCmdData::evCmdData()
	{
	}

	evCmdData::evCmdData(evCmdData *mst)
	{
		this->Operator( mst->Operator() );
	}

	void evCmdData::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->Operator( pReader->ReadEnum<evOperator,evOperator::Type>() );
	}


	evdInt::evdInt(evDataType::Type eType, int iValue)
	{
		this->DataType(eType);
		this->Value(iValue);
	}

	evdInt::evdInt(evdInt *mst) : evCmdData(mst)
	{
		this->DataType(mst->DataType());
		this->Value(mst->Value());
	}

	ICmdBase *evdInt::Clone()
	{
		return new evdInt(this);
	}

	void evdInt::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
		this->Value(pReader->ReadInt());
	}

	evdString::evdString(evDataType::Type eType, int iValue)
	{
		this->DataType(eType);
		this->Value(iValue);
		glbString s = glbString();
		this->Text( &s );
	}

	evdString::evdString(evdString *mst) : evCmdData(mst)
	{
		this->DataType(mst->DataType());
		this->Text( mst->Text() );
		this->Value(mst->Value());
	}

	ICmdBase *evdString::Clone()
	{
		return new evdString(this);
	}

	void evdString::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
		this->Value(pReader->ReadInt());
		glbString s = pReader->ReadString();
		this->Text( &s );
	}

	evDataType::Type evdVariableNum::DataType()
	{
		return evDataType::VariableNum;
	}

	void evdVariableNum::DataType(evDataType::Type value)
	{
	}

	int evdVariableNum::Value()
	{
		if (this->DataVariable() == null) return 0;
		return this->DataVariable()->Value();
	}

	void evdVariableNum::Value(int value)
	{
		if (this->DataVariable() == null) return;
		this->DataVariable()->Value(value);
	}

	evdVariableNum::evdVariableNum()
	{
		this->DataVariable(null);
	}

	evdVariableNum::evdVariableNum(evdVariableNum *mst) : evCmdData(mst)
	{
		this->DataVariable( mst->DataVariable() );
	}

	ICmdBase *evdVariableNum::Clone()
	{
		return new evdVariableNum(this);
	}

	void evdVariableNum::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
		this->DataVariable( evCmdUtil::Load<evVariableNum>(pReader, pCmdUI) );
	}

	evDataType::Type evdVariableState::DataType()
	{
		return evDataType::VariableState;
	}

	void evdVariableState::DataType(evDataType::Type value)
	{
	}

	int evdVariableState::Value()
	{
		return this->DataVariable()->Value();
	}

	void evdVariableState::Value(int value)
	{
		this->DataVariable()->Value(value);
	}

	evdVariableState::evdVariableState()
	{
		this->DataVariable( null );
	}

	evdVariableState::evdVariableState(evdVariableState *mst) : evCmdData(mst)
	{
		this->DataVariable( mst->DataVariable() );
	}

	ICmdBase *evdVariableState::Clone()
	{
		return new evdVariableState(this);
	}

	void evdVariableState::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
		this->DataVariable( evCmdUtil::Load<evVariableState>(pReader, pCmdUI) );
	}

	evDataType::Type evdVariableStateItem::DataType()
	{
		return evDataType::VariableStateItem;
	}

	void evdVariableStateItem::DataType(evDataType::Type value)
	{
	}


	int evdVariableStateItem::Value()
	{
		return this->ItemID();
	}

	void evdVariableStateItem::Value(int value)
	{
		GLB_ASSERT(false);
	}

	evdVariableStateItem::evdVariableStateItem()
	{
		this->DataVariable( null );
		this->ItemID( 0 );
	}

	evdVariableStateItem::evdVariableStateItem(evdVariableStateItem *mst) : evCmdData(mst)
	{
		this->DataVariable( mst->DataVariable() );
		this->ItemID( mst->ItemID() );
	}

	ICmdBase *evdVariableStateItem::Clone()
	{
		return new evdVariableStateItem(this);
	}

	void evdVariableStateItem::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
		this->DataVariable( evCmdUtil::Load<evVariableState>(pReader, pCmdUI) );
		this->ItemID( pReader->ReadInt() );
	}

	evDataType::Type evdElemState::DataType()
	{
		return evDataType::ElemState;
	}

	void evdElemState::DataType(evDataType::Type value)
	{
	}

	evdElemState::evdElemState(seElem *elem)
	{
		this->Target(elem);
		this->StateType( emElemState::Pos );
		this->Offset( 0 );
	}

	evdElemState::evdElemState(evdElemState *mst) : evCmdData(mst)
	{
		this->Target( mst->Target() );
		this->StateType( mst->StateType() );
		this->Offset( mst->Offset() );
	}

	int evdElemState::Value()
	{
		if (this->Target() == null) return 0;
		IseStateType *stype = this->Target()->State()->At(this->StateType());
		return evData::GetValue(stype, this->StateType(), this->Offset());
	}

	void evdElemState::Value(int value)
	{
		if (this->Target() == null) return;
	
		IseStateType *stype = this->Target()->State()->At(this->StateType());
		evData::SetValue(stype, this->StateType(), this->Offset(), value);
	
		// ̃Xe[^XXVAȉ̌pĎs
		this->Target()->State()->Apply();

		struct func : public deFunc1<seElem *,bool>
		{
			bool operator()(seElem *pElem )
			{
				pElem->Succession(seRootScene::Instance()->Camera());
				return true;
			}
		} f;
		this->Target()->DoAll(&f);
	}


	ICmdBase *evdElemState::Clone()
	{
		return new evdElemState(this);
	}

	void evdElemState::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
	
		this->Target( pCmdUI->Target() );
		this->StateType( pReader->ReadEnum<emElemState,emElemState::Type>());
		this->Offset( pReader->ReadInt() );
	}

	evDataType::Type evdKeyValue::DataType()
	{
		return evDataType::KeyName;
	}

	void evdKeyValue::DataType(evDataType::Type value)
	{
	}

	evdKeyValue::evdKeyValue(seElem *elem, seKeyControlManager *keyManager)
	{
		this->Target( elem );
		this->KeyManger( keyManager );
		this->Index( evdKeyValue::NoKey );
	}

	evdKeyValue::evdKeyValue(evdKeyValue *mst) : evCmdData(mst)
	{
		this->Target( mst->Target() );
		this->KeyManger( mst->KeyManger() );
		this->KeyID( mst->KeyID() );
		this->Index( mst->Index() );
	}

	int evdKeyValue::Value()
	{
		if (this->KeyManger() == null) return 0;
		// GgXe[^XɒlZorݒ
		seStateKeyInfo *keyInfo = this->KeyManger()->seIDManager<seStateKeyInfo*>::Find(this->KeyID());
		if (keyInfo == null) return 0;
	
		return evData::GetValue(keyInfo->Data(), keyInfo->StateType(), this->Index());
	}

	void evdKeyValue::Value(int value)
	{
		if (this->KeyManger() == null) return;
		seStateKeyInfo *keyInfo = this->KeyManger()->seIDManager<seStateKeyInfo*>::Find(this->KeyID());
		if (keyInfo == null) return;
		evData::SetValue(keyInfo->Data(), keyInfo->StateType(), this->Index(), value);
	
		// ̃Xe[^XXVAȉ̌pĎs
		this->Target()->State()->Apply();

		struct func : public deFunc1<seElem *,bool>
		{
			bool operator()(seElem *pElem )
			{
				pElem->Succession(seRootScene::Instance()->Camera());
				return true;
			}
		} f;
		this->Target()->DoAll(&f);
	}

	ICmdBase *evdKeyValue::Clone()
	{
		return new evdKeyValue(this);
	}

	void evdKeyValue::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->evCmdData::Load(pReader, pCmdUI);
	
		this->Target( pCmdUI->Target() );
		this->KeyManger( pCmdUI->SceneManager()->KeyControlManager() );
	
		this->KeyID( pReader->ReadInt() );
		this->Index( pReader->ReadInt() );
	}

	evDataType::Type evdUserCall::DataType()
	{
		return evDataType::UserCall;
	}

	void evdUserCall::DataType(evDataType::Type value)
	{
	}

	int evdUserCall::Value()
	{
		siEventListener *el = seRootScene::Instance()->EventListener();
		if (el == null) return 0;
		return el->GetValue(this->Name(), this->Target()->ElemID());
	}

	void evdUserCall::Value(int value)
	{
		siEventListener *el = seRootScene::Instance()->EventListener();
		if (el == null) return;
		el->CallCommand(this->Name(), this->Target()->ElemID(), value);
	}

	evdUserCall::evdUserCall(seElem *target)
	{
		this->Target( target );
		glbString s = glbString();
		this->Name( &s );
	}

	evdUserCall::evdUserCall(evdUserCall *mst) : evCmdData(mst)
	{
		this->Target( mst->Target() );
		this->Name( mst->Name() );
	}

	ICmdBase *evdUserCall::Clone()
	{
		return new evdUserCall(this);
	}

	void evdUserCall::Load(IseStateInput *pReader, IevCmdUI *pCmdUI)
	{
		this->Target( pCmdUI->Target() );
		this->evCmdData::Load(pReader, pCmdUI);
		glbString s = pReader->ReadString();
		this->Name( &s );
	}
}
