#include "glbPch.h"
#include "glb/glbDebug.h"
#include "evc/common/seDefine.h"
#include "evc/scene/seRootScene.h"
#include "evc/IO/siIOManager.h"
#include "evc/file/IseStateIO.h"
#include "evVariable.h"

using namespace glb;
namespace evc
{
	evVariableState::evVariableState()
		: iDefaultValue( 0 )
		, bMultiSelect( false )
		, iValue( 0 )
	{
	}

	void evVariableState::Init()
	{
		this->Value(this->DefaultValue());
	}

	bool evVariableState::IsOn(int iStateID) const
	{
		return ((this->iValue & iStateID) == iStateID);
	}

	void evVariableState::Add(int iStateID)
	{
		// IłȂȂx폜
		if (!this->MultiSelect())
		{
			this->iValue = 0;
		}
		this->iValue |= iStateID;
	}

	void evVariableState::Remove(int iStateID)
	{
		this->iValue &= ~iStateID;
	}

	void evVariableState::Clear()
	{
		this->iValue = 0;
	}

	void evVariableState::Load(IseStateInput *pInput)
	{
		this->bMultiSelect = pInput->ReadBool();
		this->iDefaultValue = pInput->ReadInt();
		this->iValue = pInput->ReadInt();
	}

	evVariable *evVariableState::Clone()
	{
		evVariableState *pNewState = evc_new evVariableState();
		pNewState->Name(this->Name());
		pNewState->bMultiSelect = this->bMultiSelect;
		pNewState->iDefaultValue = this->iDefaultValue;
		pNewState->iValue = this->iValue;
		return pNewState;
	}

#if defined(_DEBUG)
	// fobOp\
	void evVariableState::DbString()
	{
		est::stringstream ss;
		const est::vector<unsigned char>& rBuffer = evVariable::Name()->Buffer();
		ss << &rBuffer[ 0 ] << ":" << this->iValue;
		glbString s( ss.str().c_str() );
		seRootScene::Instance()->IOManager()->DebugString( &s );

//		GLB_DB_STRING( "%s:%d\n", evVariable::Name()->Buffer(), this->iValue );
	}
#endif




	void evVariableNum::Value(int value)
	{
		this->iValue = value;
		if (this->Loop() && this->EnableMax() && this->EnableMin())
		{
			int iLen = this->Max() - this->Min();
			this->iValue = (iLen == 0) ? this->Min() : (this->iValue % iLen) + this->Min();
		}
		else
		{
			// `FbN
			if (this->EnableMin())
			{
				this->iValue = glb::MaxValue(this->iValue, this->Min());
			}
			if (this->EnableMax())
			{
				this->iValue = glb::MinValue(this->iValue, this->Max());
			}
		}
	
	}

	evVariableNum::evVariableNum()
		: iMin( 0 )
		, iMax( 0 )
		, bLoop( false )
		, bEnableMin( false )
		, bEnableMax( false )
		, iDefaultValue( 0 )
		, iValue( 0 )
	{
	}

	void evVariableNum::Init()
	{
		this->Value(this->DefaultValue());
	}

	void evVariableNum::Load(IseStateInput *pInput)
	{
		this->iMin = pInput->ReadInt();
		seDefine::DebugTrace( "Min", this->iMin);
		this->iMax = pInput->ReadInt();
		seDefine::DebugTrace("Max", this->iMax);
		this->bEnableMin = pInput->ReadBool();
		seDefine::DebugTrace("EnableMin", this->bEnableMin ? 1 : 0);
		this->bEnableMax = pInput->ReadBool();
		seDefine::DebugTrace("EnableMax", this->bEnableMax ? 1 : 0);
		this->iDefaultValue = pInput->ReadInt();
		seDefine::DebugTrace("DefaultValue", this->iDefaultValue);
		this->bLoop = pInput->ReadBool();
		seDefine::DebugTrace("Loop", this->bLoop ? 1 : 0);
	//			this.iValue = pInput.ReadInt();
	}

	evVariable *evVariableNum::Clone()
	{
		evVariableNum *pNum = evc_new evVariableNum();
		pNum->Name(this->Name());
		pNum->iMin = this->iMin;
		pNum->iMax = this->iMax;
		pNum->bLoop = this->bLoop;
		pNum->bEnableMin = this->bEnableMin;
		pNum->bEnableMax = this->bEnableMax;
		pNum->iDefaultValue = this->iDefaultValue;
		pNum->iValue = this->iValue;
	
		return pNum;
	}

#if defined(_DEBUG)
	void evVariableNum::DbString()
	{
		est::stringstream ss;
		const est::vector<unsigned char>& rBuffer = evVariable::Name()->Buffer();
		ss << &rBuffer[ 0 ] << ":" << this->iValue;
		glbString s( ss.str().c_str() );
		seRootScene::Instance()->IOManager()->DebugString( &s );

//		GLB_DB_STRING( "%s:%d", this->evVariable::Name()->Buffer(), this->iValue );
	}
#endif

	int evVariableRandomNum::Value()
	{
		this->evVariableNum::Value( GLB_RAND() );
		return this->evVariableNum::Value();
	}

	void evVariableRandomNum::Value(int value)
	{
		this->evVariableNum::Value(value);
	}



}
