#include "glbPch.h"
#include "evc/common/seDefine.h"
#include "evc/common/seIDManager.h"
#include "evc/file/IseStateIO.h"
#include "evVariableManager.h"
#include "evVariableSp.h"

namespace evc
{
	evVariableGroup::evVariableGroup(int iBaseID)
		: seIDManager<evVariable*>(iBaseID)
	{
	}

	evVariableGroup::evVariableGroup()
		: seIDManager<evVariable*>( evVariableGroup::LocalBaseID )
	{
	}

	evVariableGroup::~evVariableGroup()
	{
		this->DeleteAllInstance();
	}

	evVariableGroup *evVariableGroup::Clone()
	{
		evVariableGroup *pClone = evc_new evVariableGroup(this->seIDManager<evVariable*>::BaseID());

		const est::map<int,evVariable*> &varis = *this->Piars();
		est::map<int,evVariable*>::const_iterator it = varis.begin();

		while(it != varis.end())
		{
			pClone->Add(it->first, it->second->Clone());
			it ++;
		}

		return pClone;
	}

	void evVariableGroup::Load(IseStateInput *pReader)
	{
		int iCount = static_cast<int>(pReader->ReadShort());
		seDefine::DebugTrace( "VariableCount", iCount);
		for (int i = 0; i < iCount; i++)
		{
			evVariable *pVariable = pReader->ReadBool() ? static_cast<evVariable*>(evc_new evVariableState())
														  : static_cast<evVariable*>(evc_new evVariableNum());
			int iID = static_cast<int>(pReader->ReadShort());
			seDefine::DebugTrace( "Variable ID", iID);
			pVariable->Load(pReader);
			// lݒ
			pVariable->Init();
			this->Add(iID, pVariable);
		}
	}

	evVariableGroup *evVariableManager::SpGroup()
	{
		return &evVariableSp::Instance();
	}

	evVariableManager::evVariableManager()
		: pGlobalGroup( null )
		, bClone( false )
	{
		this->pGlobalGroup = evc_new evVariableGroup(evVariableManager::GlobalBaseID);
	}

	evVariableManager::evVariableManager(evVariableGroup *pGlobal)
		: pGlobalGroup( pGlobal )
		, bClone( true )
	{}

	// fXgN^
	evVariableManager::~evVariableManager()
	{
		// IWiȂ폜
		if ( !this->bClone )
		{
			GLB_SAFE_DELETE( this->pGlobalGroup );
		}
	}


	VariableGroup::Type evVariableManager::GetGroupType(evVariable *pVariable)
	{
		if (this->pGlobalGroup->Find(pVariable) != null) return VariableGroup::Global;
		if (this->SpGroup()->Find(pVariable) != null) return VariableGroup::Sp;
		return VariableGroup::Local;
	}

	void evVariableManager::Clear()
	{
		// S̗̈̕ϐNA
		this->pGlobalGroup->Clear();
	}

	evVariableManager *evVariableManager::Clone()
	{
		// ϐ͊ȈՃRs[
		return evc_new evVariableManager(this->pGlobalGroup->Clone());
	}

	void evVariableManager::Load(IseStateInput *pReader)
	{
		// S̕ϐO[v̂݁B
		// [Jϐ̓GgƈꏏɁAϐ͓Iɍ쐬
		this->pGlobalGroup->Load(pReader);
	}
}
