#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seDefine.h"
#include "evc/file/IseStateIO.h"
#include "seElemManager.h"
#include "seAnimManager.h"
#include "seOption.h"
#include "seRootScene.h"
#include "seSceneManager.h"

#include "evc/IO/listener/siAnimListener.h"

#include "evc/seObject/seObject.h"
#include "evc/seObject/seObjectWork.h"
#include "evc/seObject/ObjectParts/seoScene.h"
#include "evc/seObject/ObjectParts/seoMesh.h"
#include "evc/Event/evManager.h"


#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemAnim/seStateKeyManager.h"
#include "evc/seElem/seElemAnim/seElemAnimManager.h"
#include "evc/seElem/seElemAnim/seElemAnim.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/IO/siMaterial.h"
#include "evc/IO/siIOManager.h"

using namespace glb;
namespace evc
{
	seElemManager::~seElemManager()
	{
		if ( this->pRootElem != null )
		{
			seStateFrameCtrl *f = static_cast<seStateFrameCtrl*>(
						this->pRootElem->State()->At( emElemState::FrameCtrl ) );
			GLB_SAFE_DELETE( f );
		}
		GLB_SAFE_DELETE( this->pRootElem );
		this->Clear();
	}

	void seElemManager::RootElem(seElem *value)
	{
		this->pRootElem = value;
		this->pRootElem->State()->ClearStateData();
		seStateFrameCtrl *pFrameControl = static_cast<seStateFrameCtrl*>(
							seStateUtil::CreateStateType(emElemState::FrameCtrl));

		pFrameControl->SceneManager( this->pSceneManager );
		this->pRootElem->State()->LinkStateData(emElemState::FrameCtrl, pFrameControl);
	}



	void DrawPriority::Add(seElem *pElem)
	{
		if (this->pFirstElem == 0)
		{
			this->pFirstElem = pElem;
			this->pLastElem = pElem;
			this->pFirstElem->BeforDraw( null );
		}
		else
		{
			this->pLastElem->NextDraw( pElem );
			pElem->BeforDraw( this->pLastElem );
			this->pLastElem = pElem;
		}
		this->pLastElem->NextDraw(null);
	}

	bool seElemManager::DoDrawAll( deFunc1<seElem*, bool> *pFunc)
	{
		seElem *pElem = this->pFirstDrawElem;
		while (pElem != 0)
		{
			if (!(*pFunc)(pElem))
				return false;
			pElem = pElem->NextDraw();
		}
		return true;
	}

	// `Ggo^ċA֐
	void seElemManager::RegisterElem( seElem *pElem )
	{
		// ҏWOFFtOĂȂ炻̃Ggȉ͖
		if (pElem->DisableEditParent()) return;

		// `\Ȃ
		if (pElem->IsVisible())
		{
			int iPriority = pElem->State()->NowPriority();
			
			est::map<int,DrawPriority>::iterator it = this->dicDrawList.find( iPriority );
			// ꍇVKo^
			if ( it == this->dicDrawList.end() )
			{
				DrawPriority d;
				d.Clear();
				this->dicDrawList[ iPriority ] = d;
				it = this->dicDrawList.find( iPriority );
			}
			it->second.Add( pElem );
		}

		const est::vector<seElem*> &child = *pElem->Child();
		est::vector<seElem*>::const_iterator it = child.begin();
		// qɑ΂ďs
		while(it != child.end())
		{
			this->RegisterElem(*it);
			it++;
		}
	}


	void seElemManager::RegisterDrawList()
	{
		// Ōɕ`悵vf
		this->dicDrawList.clear();
		this->pFirstDrawElem = null;
		this->pLastDrawElem = null;
	
		if (this->pRootElem != null)
		{
			// o^Jn
			this->RegisterElem( this->pRootElem );
	
			// vCIeB̃GgA
			// Pz̕`惊Xg쐬
			DrawPriority *pBfoWork = null;

			est::map<int,DrawPriority>::iterator it = this->dicDrawList.begin();
			while( it != this->dicDrawList.end() )
			{
				if (pBfoWork != null)
				{
					pBfoWork->pLastElem->NextDraw( it->second.pFirstElem );
					it->second.pFirstElem->BeforDraw( pBfoWork->pLastElem );
				}
				else
				{
					this->pFirstDrawElem = it->second.pFirstElem;
					this->pFirstDrawElem->BeforDraw( null );
				}
	
				this->pLastDrawElem = it->second.pLastElem;
				pBfoWork = &it->second;

				it ++;
			}
	
		}
	}

	const glbColor *seElemManager::Color() const
	{
		if (this->pRootElem == null) return null;
		return this->pRootElem->State()->NowColor();
	}

	void seElemManager::Color(const glbColor *value)
	{
		if (this->pRootElem == null) return;
		this->pRootElem->State()->NowColor( value );
	}


	float seElemManager::PlayTime()
	{
		return this->fPlayTime;
	}

	void seElemManager::PlayTime(float value)
	{
		this->fPlayTime = value;
		float fFrameTime = this->pSceneManager->Option()->FrameTime();
		this->iFrame = static_cast<int>(this->fPlayTime / fFrameTime);
	}

	int seElemManager::Frame()
	{
		return this->iFrame;
	}

	void seElemManager::Frame(int value)
	{
		this->iFrame = value;
		float fFrameTime = this->pSceneManager->Option()->FrameTime();
		this->fPlayTime = fFrameTime * static_cast<float>(this->iFrame);
	}

	void seElemManager::Clear()
	{
		this->seIDManager<seElem*>::Clear();

		// vfmۂĂ郊\[XSĉ
		GLB_SAFE_DELETE(this->pRootElem);
		this->fPlayTime = 0.0f;
		this->iFrame = 0;
	}


	bool seElemManager::DoAll(deFunc1<seElem*, bool> *pFunc)
	{
		if (this->pRootElem == null) return false;
		return this->pRootElem->DoAll(pFunc);
	}

	void seElemManager::SetAnim(int iAnimID)
	{
		this->iLastAnim = iAnimID;
	
		// ꏈFFrameCtrl̓Aj[Vɏ(ԂpȂj
		// FrameCtrlroot̂ݑ݂
		this->pRootElem->State()->GetData(emElemState::FrameCtrl)->ChangeAnim(null);
	
		// ҏW[h̎̓Aj[VxNA
		// Aj[Vđ݂̂ݏs
		
		struct func : deFunc1<seElem *, bool>
		{
			int iAnimID;
			bool operator()(seElem *pElem)
			{
				seElemAnim *pAnim = pElem->Anim()->Find( iAnimID );
				if ( pAnim == null ) return true;

				seElemState *pState = pElem->State();
				const est::map<emElemState::Type, seStateKeyManager*> &keyManagers = *pAnim->KeyManagers();
				est::map<emElemState::Type, seStateKeyManager*>::const_iterator it = keyManagers.begin();
				while( it != keyManagers.end() )
				{
					seStateKeyManager *pKeyManager = it->second;
					seStateData *d = pState->GetData(pKeyManager->StateType());
					d->ChangeAnim( pKeyManager );
					it ++;
				}
				return true;
			}
			func( int iInAnimID )
				: iAnimID( iInAnimID ) {}
		} f( iAnimID );

		this->DoAll( &f );


		// Aj[VCxgʒm
		seRootScene *rs = seRootScene::Instance();
		siAnimListener *al = rs->AnimListener();
		if ( al != null )
		{
			seAnimInfo *ai = rs->RootScene()->AnimManager()->Find( iAnimID );
			al->OnAnimStart(ai->Name());
		}

	}

	void seElemManager::ResetAnim()
	{
		if (this->pRootElem == null) return;
		float fFrameTime = this->pSceneManager->Option()->FrameTime();
	
		this->ResetAnim(this->Frame() * fFrameTime, fFrameTime, 
			seRootScene::Instance()->Camera());
	}

	void seElemManager::ResetAnim(float fTime, float fFrameTime, const IGLMatrix *pCamera)
	{
		struct func : deFunc1<seElem *,bool>
		{
			float fTime;
			float fFrameTime;
			const IGLMatrix *pCamera;
			bool operator() ( seElem *pElem )
			{
				pElem->ResetAnim(pCamera, fFrameTime, fTime);
				return true;
			}
			func( float time, float frame, const IGLMatrix *cam )
				: fTime( time )
				, fFrameTime( frame )
				, pCamera ( cam ){}

		} f( fTime, fFrameTime, pCamera );
		this->DoAll( &f );

		// `惊XgXV
		this->RegisterDrawList();
	}

	void seElemManager::Draw()
	{
		siMaterial *pMaterial = seRootScene::Instance()->Material();
		if ( pMaterial != null )
		{
			pMaterial->Blend( Blend::NORMAL );
			pMaterial->Doubly( Doubly::Normal );
			pMaterial->MaskAlpha( 255 );
			pMaterial->Apply();
		}

		// o^ς݂̃Ggɑ΂ĕ`
		struct func : deFunc1<seElem *,bool>
		{
			siMaterial *pMaterial;
			bool operator() ( seElem *pElem )
			{
				pElem->Draw( pMaterial );
				return true;
			}
			func( siMaterial *pInMaterial ) 
				: pMaterial( pInMaterial ){}

		} f( pMaterial );
		this->DoDrawAll( &f );

		// ŌɃ}XÑNAĂB
		seRootScene::Instance()->IOManager()->ClearMask();
	}

	void seElemManager::Update(float fAddTime)
	{
		this->Update( seRootScene::Instance()->Camera(), fAddTime );
	}

	// ċAp
	void seElemManager::UpdateElem( seElem *pElem,
									const IGLMatrix *mParent,
									float fFrameTime,
									float fAddTime)
	{
		// falseȂ牺ʂ̃Gg̏͂Ȃ
		if (!pElem->Update(mParent, fFrameTime, fAddTime)) return;

		const est::vector<seElem*> &child = *pElem->Child();
		est::vector<seElem*>::const_iterator it = child.begin();
		while(it != child.end())
		{
			this->UpdateElem(*it, mParent, fFrameTime, fAddTime);
			it ++;
		}
	}


	void seElemManager::Update( const IGLMatrix *mParent, float fAddTime)
	{
		// vfAj[Vs
		if (this->pRootElem != null)
		{
			this->PlayTime(this->PlayTime() + fAddTime);
			float fFrameTime = this->pSceneManager->Option()->FrameTime();

			this->UpdateElem( this->pRootElem,
					mParent, fFrameTime, fAddTime );
		}
		// `惊XgXV
		this->RegisterDrawList();
	}

	void seElemManager::Succession( const IGLMatrix *pCamera)
	{
		struct func : deFunc1<seElem *,bool>
		{
			const IGLMatrix *pCamera;
			bool operator()( seElem *pElem )
			{
				pElem->Succession( pCamera );
				return true;
			}
			
			func( const IGLMatrix *cam )
				: pCamera( cam ){}
		} f( pCamera );


		this->DoAll( &f );
	}


	void seElemManager::ResetObjectWork( seObject *pNewObject, 
										 seObject *pOldObject)
	{
		// `IuWFNg̍Ċ蓖āB
		struct func : deFunc1<seElem *,bool>
		{
			seObject *pNewObject;
			seObject *pOldObject;

			bool operator()( seElem *pElem )
			{
				if (pElem->ObjectWork() == null) return true;
				if (pElem->ObjectWork()->Object() == pOldObject)
				{
					pElem->ObjectWork( (pNewObject == 0) ? null 
														  : pNewObject->CreateDrawWork(pElem) );
				}
				return true;
			}
			func( seObject *n, seObject *o )
				: pNewObject( n )
				, pOldObject( o ) {}
		} f( pNewObject, pOldObject );
		this->DoAll( &f );
	}

	bool seElemManager::UpdateHit( const glbVector2 *sPoint, bool bFirstHit)
	{
		// `悵vfO珇ɏĂ
		seElem *pElem = this->pLastDrawElem;
		for (; pElem != 0; pElem = pElem->BeforDraw())
		{
			// CxgptONA
			pElem->Hit( seHit::Non );
	
			// V[vfȂV[̓蔻
			seObjectWork *pObjWork = pElem->ObjectWork();
			if ( pObjWork == null ) continue;

			if ( pObjWork->Object()->ElemType() == emObject::Scene )
			{
				seoSceneWork *pSceneWork = static_cast<seoSceneWork*>(pObjWork);
				// V[[hĂȂꍇnull
				seSceneManager *pLocalScene = pSceneWork->LocalScene();
				if (pLocalScene != null)
				{
					bFirstHit = pLocalScene->ElemManager()->UpdateHit(sPoint, bFirstHit);
				}
			}
			// ȊO͒ʏ̓蔻
			else if (pElem->IsInRect(sPoint))
			{
				bool bHit = true;
				// eNX`ŃsNZPʂ蔻Ȃ
				seStateTextureControl *pTexControl = static_cast<seStateTextureControl*>(pElem->State()->At(emElemState::TextureControl));
				if ( (pTexControl != null) && 
					 (pTexControl->Hit() == seTexHit::Pixel))
				{
					// sNZPʂœ蔻
					bHit = pElem->IsHitPixel(sPoint);
				}
	
				if (bHit)
				{
					pElem->Hit( (bFirstHit) ? seHit::TopHit
											: seHit::Hit );
					bFirstHit = false;
				}
			}
		}
		return bFirstHit;
	}

	void seElemManager::DoEvent()
	{
		// CxgSĂɑ΂ďs
		// \łs
		// 蔻͔\̃Gg͎s悤ɂȂĂ܂B
		// 
		if (this->pRootElem == null) return;
	
		// `IuWFNg̍Ċ蓖āB
		struct func : deFunc1<seElem *,bool>
		{
			seSceneManager *pScene;

			bool operator()( seElem *pElem )
			{
				if (pElem->Event() != null)
				{
					pElem->Event()->DoFunc( pScene );
				}
				// V[Ȃ璆̃Cxgs
				seObjectWork *pObjWork = pElem->ObjectWork();
				if ( pObjWork == null ) return true;

				if ( pObjWork->Object()->ElemType() == emObject::Scene )
				{
					seoSceneWork *sw = static_cast<seoSceneWork*>(pObjWork);
					sw->DoEvent();
				}
				return true;
			}
			func ( seSceneManager *sc ) 
				: pScene( sc ) {}
		} f( this->pSceneManager );
		this->pRootElem->DoAll(&f);
	}

	// [h
	void seElemManager::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		this->RootElem(evc_new seElem(null, 0, seElemManager::InvalidID));
		this->RootElem()->Load(pInput, pManager);

		const est::map<int,seObject*> &objs = *pManager->ObjectManager()->Piars();
		est::map<int,seObject*>::const_iterator it = objs.begin();
		while(it != objs.end())
		{
			seObject *pObj = it->second;
			if ( pObj->ElemType() == emObject::Mesh )
			{
				// ɂANSẴGgXV܂B
				seoMesh *mesh = static_cast<seoMesh*>(pObj);
				mesh->RegisterMesh();
				mesh->UpdateMeshElem();
			}
			it++;
		}
	}
}

