#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbString.h"
#include "evc/common/seDefine.h"
#include "evc/file/IseStateIO.h"
#include "evc/Event/evInputControl.h"
#include "evc/IO/siIOManager.h"
#include "evc/IO/listener/siAnimListener.h"
#include "evc/IO/listener/siEventListener.h"

#include "evc/seResource/seResourceScene.h"
#include "seObjectManager.h"
#include "seSceneManager.h"
#include "seOption.h"
#include "seElemManager.h"
#include "seRootScene.h"

using namespace glb;
namespace evc
{
	seRootScene * seRootScene::instance = null;

	// V[[hO̔wiF
	glbColor seRootScene::DefaultBgColor = glbColor(0,0,0,255);


	bool seRootScene::IsRootLoading() const
	{
		return (this->pSceneResource != null);
	}

	void seRootScene::RootCancel()
	{
		if (this->pSceneResource == null) return;
		this->pSceneResource->Cancel();
		delete this->pSceneResource;
		this->pSceneResource = null;
	}

	const glbColor *seRootScene::BgColor()
	{
		if (this->pSceneResource == null) return &seRootScene::DefaultBgColor;
		if (!this->pSceneResource->IsEffective()) return &seRootScene::DefaultBgColor;
		return this->pSceneResource->Scene()->Option()->BgColor();
	}

	void seRootScene::Load( const glbString *pSceneName, const glbString *pInAnimName)
	{
		// [hɃ[hvꍇB
		if (this->pSceneResource != null)
		{
			delete this->pSceneResource;
		}
	
		this->pSceneResource = evc_new seResourceScene(pSceneName);
		this->strAnimName = *pInAnimName;
		this->pSceneResource->Load(0);
	}

	void seRootScene::UpdateInput()
	{
		if (this->pRootScene == null) return;
		seElemManager *em = this->pRootScene->ElemManager();
		if (em == null) return;
	
		// qbgvf̍XV
		em->UpdateHit(evInputControl::Instance().Pointer(), true);
	
		// ̓CxgJn
		evInputControl::Instance().Update();
	
		// Cxgs
		em->DoEvent();
	}

	void seRootScene::DoAnim(float fAddTime)
	{
		// [g̃V[[hȂ
		if (this->pSceneResource != null)
		{
			this->pSceneResource->Update();
			if (this->pSceneResource->IsEffective())
			{
				// ANeBuV[؂ւ
				GLB_SAFE_DELETE( this->pRootScene );

				seSceneManager *pSceneManager = this->pSceneResource->PassesScene();
				this->pRootScene = pSceneManager;
				this->pRootScene->RequestAnim( &this->strAnimName );
				// \[X̍폜
				GLB_SAFE_DELETE(this->pSceneResource);
	
				if (this->OnInit != null)
				{
					(*this->OnInit)(this->pRootScene);
				}
				this->pRootScene->ElemManager()->ResetAnim();
				return;
			}
		}
	
		// Aj[V̍XV
		if (this->pRootScene != null)
		{
			this->pRootScene->OnProc(fAddTime);
		}
	}

	void seRootScene::Draw()
	{
		if (this->pRootScene == null) return;
		this->pRootScene->ElemManager()->Draw();
	}

	void seRootScene::Clear()
	{
		if (this->pRootScene == null) return;
		// V[NA
		this->pRootScene->Clear();
	}

	seRootScene::seRootScene( siIOManager *pIOManager, 
							  siEventIO *pInEventIO, 
							  siMaterial *pInMaterial )
		: pSceneResource(null)	// V[\[X
		, pRootMatrix(null)		// ŏʃ}gNX
		, pRootScene(null)		// ŜǗV[
		, bEditMode( false )	// ҏWĐ(ĐftHg)
		, pEventIO(null)
		, pIOManager(null)		// o̓NX
		, pMaterial(null)
		, pEventListener(null)
		, pAnimListener(null)
		, OnInit( null )
	{
		GLB_ASSERT( seRootScene::instance == null );
	
		seRootScene::instance = this;
	
		this->pIOManager = pIOManager;
		this->pRootMatrix = this->pIOManager->CreateMatrix();
		this->pRootMatrix->Identity();
		this->pEventIO = pInEventIO;
		this->pMaterial = pInMaterial;
	}

	seRootScene::~seRootScene()
	{
		GLB_SAFE_DELETE( this->pRootScene );
		GLB_SAFE_DELETE( this->pSceneResource );
		GLB_SAFE_DELETE( this->pRootMatrix );
	}
}
