#include "glbPch.h"

#include "seElem.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/Event/evManager.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemAnim/seElemAnim.h"
#include "evc/seElem/seElemAnim/seElemAnimManager.h"
#include "evc/seElem/seElemAnim/seStateKeyManager.h"
#include "evc/seElem/seElemState/seElemState.h"	
#include "evc/seElem/seElemState/seStateData.h"	
#include "evc/seObject/seObject.h"
#include "evc/seObject/seObjectWork.h"
#include "evc/IO/siMaterial.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"


using namespace glb;
namespace evc
{


	bool seElem::UpdateAnim()
	{
		return bUpdateAnim;
	}


	const IGLMatrix *seElem::NowMatrix() const
	{
		return this->State()->NowMatrix();
	}

	bool seElem::IsVisible() const
	{
		return (this->State()->Visible() != seAttrVisible::Hide);
	}


	void seElem::Init()
	{
		this->iElemID = seElemManager::InvalidID;
		this->bSelectWork = false ;
		this->bStopAnim = false ;
		this->bStopAnimAll = false ;
		this->eHit = seHit::Non ;
		this->bDisableEdit = false ;
		this->bDisableEditParent = false ;
		this->pParent = null ;
		this->pBeforDraw = null ; // O̕`vf
		this->pNextDraw = null ; // ̕`vf
		this->bUpdateAnim = false ; // Aj[VXVۂ
		this->pEvent = null ; // Cxg
		this->pAnimManager = null ; // Aj[VǗ
		this->pObjectWork = null ;
		this->pState = null ; // f[^Ǘ
		this->bClone = false ;
	}

	seElem::seElem(const glbString *pName, seElem *pInParent, int iIndex)
	{
		this->Init();
		this->bClone = false;
		this->pParent = pInParent;
		this->iElemID = iIndex;
		if ( pName != null )
		{
			this->strName = *pName;
		}
	
		if (this->pParent != null)
		{
			this->pParent->AddChild(this);
		}
	
		this->pState = evc_new seElemState(); // f[^Ǘ
		this->pAnimManager = evc_new seElemAnimManager(); // Aj[VǗ
	}

	seElem::seElem(seElem *pMstElem, seElem *pInParent, seSceneManager *pManager)
	{
		this->Init();
		this->bClone = true;
		this->pParent = pInParent;
		this->iElemID = pMstElem->ElemID();
		this->strName = *pMstElem->Name();
	
		if (this->pParent != null)
		{
			this->pParent->AddChild(this);
		}
	
		// Xe[^X쐬
		this->pState = evc_new seElemState();
	
		// `IuWFNg̍Ċ蓖āB
		seObjectWork *w = pMstElem->ObjectWork();
		this->pObjectWork = (w == null) ? null : w->Object()->CreateDrawWork(this);
	
		// Xe[^XRs[
		this->State()->SetState(pMstElem->State());
		// Aj[VRs[
		this->pAnimManager = pMstElem->Anim();
	
		// CxgRs[
		if (pMstElem->Event() != null)
		{
			this->pEvent = pMstElem->Event()->Clone(this, pManager);
		}
	}

	seElem::~seElem()
	{
		// ɉʂ̎q폜
		glb::DeleteVector( this->lisChildList );

		// 폜ɐe̎qXg폜Ɖ̂
		// eȂB
#if 0
		// eȂ
		if ( this->pParent != null )
		{
			// eƂ̊֌W؂
			this->pParent->RemoveChild( this );
		}
#endif
		GLB_SAFE_DELETE( this->pObjectWork );
		GLB_SAFE_DELETE( this->pState );
		// N[̏ꍇ̓|C^̂ݏ
		if ( this->bClone )
		{
			this->pAnimManager = null;
		}
		// IWȉꍇ̂݃Aj[V폜
		else
		{
			GLB_SAFE_DELETE( this->pAnimManager );
		}
		GLB_SAFE_DELETE( this->pEvent );
	}


	bool seElem::Update( const IGLMatrix *mCamera, float fFrameTime, float fTime)
	{
		// ȉ̃Aj[VSĒ~ȂI
		if (this->StopAnimAll()) return false;
		// OFFȂI
		if (this->DisableEdit()) return false;
	
		// ̃Aj[V~Ȃ
		if (!this->StopAnim())
		{
			this->UpdateOne(mCamera, fFrameTime, fTime);
		}
	
		return true;
	}

	void seElem::ResetAnim(const IGLMatrix *mCamera, float fFrameTime, float fTime)
	{
		this->DisableEditParent( this->DisableEdit() );
		if ( this->Parent() != null )
		{
			if ( this->Parent()->DisableEditParent() == true )
			{
				this->DisableEditParent( true );
			}
		}
		if ( this->DisableEditParent() ) return;
	
		// Xe[^XAj[VJnOԂɂB
		this->State()->RestartAnim();
		// `IuWFNg̓[NZbg
		if (this->pObjectWork != null)
		{
			this->pObjectWork->Reset();
		}
	
		// Aj[VXV(eXg)
		this->UpdateOne(mCamera, fFrameTime, fTime);
	}

	void seElem::UpdateOne(const IGLMatrix *mCamera, float fFrameTime, float fTime)
	{
		// Aj[V̒lXe[^XɓK
		this->bUpdateAnim = this->State()->UpdateAnim(fFrameTime, fTime);
	
		if (this->Parent() == null)
		{
			// rootAj[V̎Wv
			seStateData *pData = this->State()->GetData(emElemState::FrameCtrl);
			// L[ׂȂ
			if (pData->MoveKey())
			{
				// t[Rg[`FbN
				seStateFrameCtrl *pFrameCtrl = static_cast<seStateFrameCtrl*>(pData->Data());
				pFrameCtrl->DoCheck();
			}
		}
		else
		{
			// eAj[VĂꍇ̓Aj[V
			if ( this->Parent()->UpdateAnim() )
			{
				this->bUpdateAnim = true;
			}
		}
	
		// Ɛěp֌WXV
		// {AXVƉʂ̃Ggɂe^̂
		// ȉ̑SĂ̌pXVKv܂B
		// AXVĂɍsȂꍇAʂȌpĂ܂
		// ʂ̍XV͂Ă܂B
		this->Succession(mCamera);
	
		// ΏۃIuWFNgȂԂXV
		if (this->pObjectWork != null)
		{
			this->pObjectWork->Update(fTime);
		}
	}

	void seElem::Succession(const IGLMatrix *mCamera)
	{
		// eȂ(root)ȂJ}gNX|
		seElem *pParent = this->Parent();
		if (pParent == null)
		{
			// }gNX̂݌p
			if (mCamera != null)
			{
				this->State()->Succession(mCamera);
			}
		}
		// eȂp
		else
		{
			// Aj[VSČpB
			this->State()->Succession(pParent->State());
		}
	}

	seElem *seElem::Clone(seElem *pElemParent, seSceneManager *pManager)
	{
		seElem *pElem = evc_new seElem(this, pElemParent, pManager);
	
		const est::vector<seElem*> &child = this->lisChildList;
		est::vector<seElem*>::const_iterator it = child.begin();
		while(it != child.end())
		{
			(*it)->Clone(pElem, pManager);
			it ++;
		}

		return pElem;
	}

	bool seElem::DoAll(deFunc1<seElem*, bool> *pFunc)
	{
		if (!(*pFunc)(this)) return false;
	
		// vectorSĂɑ΂ď:iteratorƂitgp
		VECTOR_FOR_EACH( seElem *, this->lisChildList,
		{
			if ( !(*it)->DoAll( pFunc ) ) return false;
		});
		return true;
	}

	void seElem::RemoveChild(seElem *pElem)
	{
		est::vector<seElem *>::iterator it = est::find( 
							this->lisChildList.begin(), 
							this->lisChildList.end(), pElem );
		this->lisChildList.erase( it );
		pElem->pParent = null;
	}

	void seElem::AddChild(seElem *pElem)
	{
		this->lisChildList.push_back(pElem);
		pElem->pParent = this;
	}

	void seElem::Load(IseStateInput *pInput, seSceneManager *pSceneManager)
	{
		this->iElemID = static_cast<int>(pInput->ReadShort());
		seDefine::DebugTrace("ElemID", this->iElemID);
	
		// Ȃvf
		if ( pSceneManager->Option()->IncludeElemName() )
		{
			this->strName = pInput->ReadString();
		}
	
		int iObjectID = static_cast<int>(pInput->ReadShort());
		seObject *pObject = pSceneManager->ObjectManager()->Find(iObjectID);
		this->pObjectWork = (pObject==null) ? null : pObject->CreateDrawWork(this);
	
		// ͕ۑ܂B
		// Cxg̓rԂۑ铙AKvɔꂽl܂B
		// 

		// ҏWOFFtO
		this->bDisableEdit = pInput->ReadBool();
	
		// CxgĂȂ烍[h
		bool bHasEvent = pInput->ReadBool();
	
		// Aj[V[h
		this->Anim()->Load(pInput, pSceneManager);
	
		if (bHasEvent)
		{
			this->Event(evc_new evManager(this, pSceneManager));
			this->Event()->Load(pInput);
		}
		else
		{
			this->Event(null);
		}
	
		int iChildCount = static_cast<int>(pInput->ReadShort());
		seDefine::DebugTrace("ChildCount", iChildCount);
		for (int i = 0; i < iChildCount; i++)
		{
			seElem *pChildElem = evc_new seElem( null, this, seElemManager::InvalidID );
			pChildElem->Load(pInput, pSceneManager);
		}
	}
}
