#include "glbPch.h"
#include "seElem.h"

#include "evc/common/seDefine.h"
#include "evc/common/seMatrixWork.h"
#include "evc/common/seMath.h"
#include "evc/scene/seRootScene.h"
#include "evc/IO/siMaterial.h"
#include "evc/IO/siIOManager.h"
#include "evc/IO/siResource.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "seStateType.h"
#include "seStateTypeEx.h"


using namespace glb;
namespace evc
{
	// ŎgvZp
	static glbVector2 ainPosBase[] = 
	{
		glbVector2(0.0f, 0.0f), 
		glbVector2(1.0f, 0.0f), 
		glbVector2(1.0f, 1.0f), 
		glbVector2(0.0f, 1.0f), 
	};


	bool seElem::IsInRect( const glbVector2 *pVector )
	{
		if (this->pObjectWork == null) return false;

		seUseWorkMatrix gw;

		IGLMatrix *m = gw.GetMatrix();
		this->GetRectMatrix(m);
		const glbVector2 *pObjectSize = this->pObjectWork->GetSize();

		return seMath::IsInRect( pObjectSize, m, pVector);
	}

	bool seElem::IsHitPixel(const glbVector2 *sPos)
	{
		if (this->pObjectWork == null) return false;
		seObject *pObject = this->pObjectWork->Object();
	
		// `ŃeNX`̏ꍇAhbgPʂŔ肷
		if (pObject->ElemType() != emObject::Texture) return false;
	
		// eNX`ݒ肳ĂȂꍇ̓qbgȂ
		siResourceTexture *pTexture = (static_cast<seoTextureSprite*>(pObject))->ResourceTexture();
		if (pTexture == null) return false;
	
		const glbVector2 *sObjectSize = this->pObjectWork->GetSize();
	
		seUseWorkMatrix gw;

		IGLMatrix *mInv = gw.GetMatrix();
		mInv->Copy(this->NowMatrix());
		mInv->Invert();
		glbVector2 s = mInv->TransformCoordinate(sPos);
		// ڂ̃[vɂ邩
		int iLoopX = static_cast<int>(s.X) / static_cast<int>(sObjectSize->X);
		int iLoopY = static_cast<int>(s.Y) / static_cast<int>(sObjectSize->Y);
		// \IuWFNĝǂ̕ӂɂ邩Zo
		s.X -= static_cast<float>(iLoopX) * sObjectSize->X;
		s.Y -= static_cast<float>(iLoopY) * sObjectSize->Y;

		// IuWFNgUVItZbg
		const glbVector2 *sObjUV = (static_cast<seoTextureSprite*>(pObject))->UV();
		seStateFloat2 *pUV = static_cast<seStateFloat2*>(this->State()->At(emElemState::UV));

		// UVړĂȂ
		glbVector2 sUVOffset = *pUV->Value();
		sUVOffset.Add(*sObjUV);
		// B(1Ŋ])
		sUVOffset.X = sUVOffset.X - static_cast<float>(static_cast<int>(sUVOffset.X));
		sUVOffset.Y = sUVOffset.Y - static_cast<float>(static_cast<int>(sUVOffset.Y));

		// }CiXȂ烋[v
		if (sUVOffset.X < 0.0f)
		{
			sUVOffset.X += 1.0f;
		}
		if (sUVOffset.Y < 0.0f)
		{
			sUVOffset.Y += 1.0f;
		}

		glbVector2I sTexSize = *(pTexture->TextureSize());

		sUVOffset.X *= (float)sTexSize.X;
		sUVOffset.Y *= (float)sTexSize.Y;
		glbVector2I sHitPos = glbVector2I(
							static_cast<int>(s.X + sUVOffset.X),
							static_cast<int>(s.Y + sUVOffset.Y));

		// [vȂ炻̕XP[Z
		seStateLoop *pLoop = static_cast<seStateLoop*>(this->State()->At(emElemState::Loop));
		if (pLoop != null)
		{
			// ]Ŋ̃[vɂꍇ
			if (pLoop->RevW() && ((iLoopX & 1) == 1))
			{
				// Eʒutɂ
				sHitPos.X = static_cast<int>(sUVOffset.X + sObjectSize->X - s.X);
			}
			// ]Ŋ̃[vɂꍇ
			if (pLoop->RevH() && ((iLoopY & 1) == 1))
			{
				// ㉺ʒutɂ
				sHitPos.Y = static_cast<int>(sUVOffset.Y + sObjectSize->Y - s.Y);
			}
		}
		sHitPos.X %= sTexSize.X;
		sHitPos.Y %= sTexSize.Y;

		return pTexture->IsHit(&sHitPos);
	}

	void seElem::GetRectMatrix(IGLMatrix *pMatrix)
	{
		// IuWFNg[Nݒ肳ĂȂȂ牽Ȃ
		if (this->ObjectWork() == null) return;
	
		pMatrix->Copy(this->State()->NowMatrix());
	
		// [vȂ炻̕XP[Z
		seStateLoop *pLoop = static_cast<seStateLoop*>(this->State()->At(emElemState::Loop));
		if (pLoop != null)
		{
			seUseWorkMatrix gw;

			IGLMatrix *ml = gw.GetMatrix(); 
			IGLMatrix *mPosOffset = gw.GetMatrix();
			glbVector2 sOffset = *this->ObjectWork()->GetSize(); 
			if (pLoop->LoopCountW() >= 0 ) sOffset.X = glbVector2::Zero.X;
			if (pLoop->LoopCountH() >= 0 ) sOffset.Y = glbVector2::Zero.Y; 
			mPosOffset->SetPos( &sOffset); 
			ml->SetScale((float)pLoop->LoopCountW(), 
						 (float)pLoop->LoopCountH());

			pMatrix->Mul( mPosOffset );
			pMatrix->Mul( ml );
		}
	}

	bool seElem::GetRect( est::vector<glbVector2> &rPosBuffer )
	{
		if (this->pObjectWork == null) return false;
	
		glbVector2 vSize = *this->pObjectWork->GetSize();

		seUseWorkMatrix gw;
		IGLMatrix *m = gw.GetMatrix();
		this->GetRectMatrix(m);

		rPosBuffer.resize( GLB_ARRAY_COUNT( ainPosBase ) );
		for ( int i = 0; i < (int)GLB_ARRAY_COUNT( ainPosBase ); i++ )
		{
			glbVector2 vWork = ainPosBase[ i ];
			vWork.Mul( vSize );
			rPosBuffer[ i ] = m->TransformCoordinate( &vWork );
		}
	
		return true;
	}

	void seElem::Draw(siMaterial *pMaterial)
	{
		// Cxgŗvf蒆ɕ`͓Ă邱ƂB
		if (!this->IsVisible()) return;
	
		// ΏۃIuWFNgȂȂ疳
		if (this->pObjectWork != null)
		{
			this->State()->SetMaterial(pMaterial);
	
			// }XNK
			switch (this->State()->Attrbute()->Mask())
			{
				// }XN̈ʒm
				case seAttrMask::Data:
					{
						seUseWorkMatrix gw;

						IGLMatrix *m = gw.GetMatrix();
						glbVector2 vSize = *this->pObjectWork->GetSize();
						this->GetRectMatrix(m);
						seRootScene::Instance()->IOManager()->RegisterMask(m, &vSize);
					}
					break;
				default:
					break;
			}
	
			this->pObjectWork->Draw();
	
			// }XNIȂ
			if (this->State()->Attrbute()->Mask() == seAttrMask::End)
			{
				// }XNIʒm
				seRootScene::Instance()->IOManager()->ClearMask();
			}
	
		}
	}
}
