#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seDefine.h"
#include "evc/common/seMath.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siMaterial.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "seElemState.h"
#include "seStateData.h"

using namespace glb;
namespace evc
{

	void seElemState::RestartAnim()
	{
		VECTOR_FOR_EACH( seStateData *, this->asStateList, 
		{
			if ((*it) == null) continue;
			(*it)->RestartAnim();
		});
		this->_bfoPos = *this->Pos()->Value();
	}

	bool seElemState::UpdateAnim(float fFrameTime, float fTime)
	{
		this->_bfoPos = *this->Pos()->Value();

		bool bUpdate = false;
		VECTOR_FOR_EACH( seStateData*, this->asStateList, 
		{
			if ((*it) == null) continue;
			bUpdate |= (*it)->Update(fFrameTime, fTime);
		});
	
		// Aj[V̍XVȂ}gNXč쐬
		if (bUpdate)
		{
			this->Apply();
		}
		return bUpdate;
	}

	void seElemState::ClearAnim()
	{
		VECTOR_FOR_EACH( seStateData*, this->asStateList, 
		{
			if ((*it) == null) continue;
			(*it)->ChangeAnim(0);
		});
		this->_bfoPos = *this->Pos()->Value();
	}


	IseStateType *seElemState::At(emElemState::Type eType)
	{
		seStateData *sd = this->asStateList[static_cast<int>(eType)];
		return (sd == null) ? null : sd->Data();
	}

	seStateData *seElemState::GetData(emElemState::Type eType)
	{
		return this->asStateList[static_cast<int>(eType)];
	}

	void seElemState::SetState(seElemState *pState)
	{
		if (pState == null) return;
	
		// SXe[^Xz쐬
		int iCount = 0;
		VECTOR_FOR_EACH( seStateData*, this->asStateList, 
		{
			if ((*it) != null)
			{
				(*it)->Data()->Set(pState->At( (emElemState::Type)iCount ));
			}
			iCount ++;
		});
	}

	void seElemState::Apply()
	{
		float fRot = (this->Rot()->SpinType() == seSpinType::Direction)
					? seMath::GetRot(&this->_bfoPos, this->Pos()->Value())
					: this->Rot()->Rot();

		// ݂̃f[^}gNX𐶐
		seMath::Transformation2D(this->sAnimMatrix, 
									this->Center()->Value(), 
									this->Scale()->Value(), 
									fRot, 
									this->Pos()->Value());
	}

	void seElemState::Succession( const IGLMatrix *mParent)
	{
		// e}gNXۑ
		this->sParentMatrix = mParent;
		this->sNowMatrix->Copy(mParent);
		this->sNowMatrix->Mul(this->sAnimMatrix);
	}

	// Xe[^XLۂ
	#define	IsSuccession( _Type ) this->GetData(_Type)->EnableSuccession()

	void seElemState::Succession(seElemState *pParentState)
	{
		// pĂ邩

	
		// `揇ʂ͂OȂep
		int iNowPriority = this->Priority()->Value();
		this->iPriority = (iNowPriority == 0) ? pParentState->NowPriority()
											  : iNowPriority;
	
		// ̃J[擾
		this->sColor = this->Color()->Color();
		// pȂẽJ[ƍ
		if (IsSuccession(emElemState::Color))
		{
			this->sColor.Mul(pParentState->NowColor());
		}
	
		// \̌p
		this->eVisible = this->Attrbute()->Visible();
		if (this->Visible() == seAttrVisible::Succes)
		{
			this->eVisible = pParentState->Visible();
		}
	
		bool bSuccesCenter = IsSuccession(emElemState::Center);
		bool bSuccesPos = IsSuccession(emElemState::Pos);
		bool bSuccesScale = IsSuccession(emElemState::Scale);
		bool bSuccesRot = IsSuccession(emElemState::Rot);
	
		// }gNXvfSČpȂ
		if (bSuccesCenter && bSuccesPos && bSuccesScale && bSuccesRot)
		{
			// ẽ}gNX̂܂܎g
			this->Succession(pParentState->NowMatrix());
		}
		// }gNXvf̂ǂꂩpȂ
		else
		{
			// pvf݂̂Ń}gNX
			seMath::Transformation2D(
				this->sParentMatrixWork, 
				bSuccesCenter ? pParentState->Center()->Value() : &glbVector2::Zero,
				bSuccesScale ? pParentState->Scale()->Value() : &glbVector2::One,
				bSuccesRot ? pParentState->Rot()->Rot() : 0.0f, 
				bSuccesPos ? pParentState->Pos()->Value() : &glbVector2::Zero );

			this->sParentMatrixWork->MulR(pParentState->ParentMatrix());
			// gpe}gNXɐݒ
			this->Succession( this->sParentMatrixWork );
		}
	
	}

	void seElemState::SetMaterial(siMaterial *pMaterial)
	{
		if ( pMaterial == null ) return;

		// uh[hݒ:l͒ʏ\
		Blend::Type eBlendType = Blend::NORMAL;
		switch (this->Attrbute()->Blend())
		{
			case seAttrBlend::Add:	  eBlendType = Blend::ADD; break;
			// Z͖̈
			case seAttrBlend::Sub:	  eBlendType = Blend::NORMAL; break;
			case seAttrBlend::Normal:
			default:
				eBlendType = Blend::NORMAL;
				break;
		}
	
		// }XN^Cv̐ݒ
		Doubly::Type eMaskType = Doubly::Normal;
		bool bUpdateMaskAlpah = false;
	
		switch (this->Attrbute()->Mask())
		{
			case seAttrMask::NoUse:
				eMaskType = Doubly::Normal;
				break;
			case seAttrMask::Data:
				{
					eMaskType = Doubly::Mask;
					int iMaskAlpha = this->Mask()->Value();
					if (pMaterial->MaskAlpha() != iMaskAlpha)
					{
						pMaterial->MaskAlpha( iMaskAlpha );
						bUpdateMaskAlpah = true;
					}
				}
				break;
			case seAttrMask::Reflection:
				eMaskType = Doubly::MaskReflection;
				break;
			case seAttrMask::End:
				eMaskType = Doubly::MaskReflection;
				break;
			default:
				break;
		}
	
		if ((pMaterial->Blend() != eBlendType) || 
			(pMaterial->Doubly() != eMaskType) || bUpdateMaskAlpah )
		{
			pMaterial->Blend( eBlendType );
			pMaterial->Doubly( eMaskType );
			pMaterial->Apply();
		}
	}

	void seElemState::LinkStateData( emElemState::Type eState,
									 IseStateType *pStateType)
	{
		this->asStateList[static_cast<int>(eState)] = evc_new seStateData(pStateType, true);
	}

	void seElemState::ClearStateData()
	{
		for (int i = (int)seDefine::LastCommonState + 1; i < (int)this->asStateList.size(); i++)
		{
			GLB_SAFE_DELETE( this->asStateList[ i ] );
		}
	}

	seElemState::~seElemState()
	{
		glb::DeleteVector( this->asStateList );
		GLB_SAFE_DELETE( this->sAnimMatrix );
		GLB_SAFE_DELETE( this->sNowMatrix );
		GLB_SAFE_DELETE( this->sParentMatrixWork );
		GLB_SAFE_DELETE( this->pPriority );
		GLB_SAFE_DELETE( this->pAttribute ); 
		GLB_SAFE_DELETE( this->pColor );
		GLB_SAFE_DELETE( this->pPos );
		GLB_SAFE_DELETE( this->pScale );
		GLB_SAFE_DELETE( this->pCenter );
		GLB_SAFE_DELETE( this->pRot );
		GLB_SAFE_DELETE( this->pMask );
	}

	seElemState::seElemState()
		: sAnimMatrix( null )		// Aj[V}gNX
		, sParentMatrixWork( null )	// ptO鎞ɍŏIIɐe}gNX
		, sParentMatrix( null )		// ŏIIɗpe}gNX(N̂)
		, sNowMatrix( null )
		, sColor( glbColor::White )
		, iPriority( 0 )
		, pPriority( null )
		, pAttribute( null )
		, pColor( null )
		, pPos( null )
		, pScale( null )
		, pCenter( null )
		, pRot( null )
		, pMask( null )
		, eVisible( seAttrVisible::Succes )
	{
		// SXe[^Xz쐬
		this->asStateList.resize(emElemState::EnumMax() + 1, null);
	
		this->pPriority = static_cast<seStateInt1*>( seStateUtil::CreateStateType( emElemState::Priority ));
		this->pAttribute = static_cast<seStateAttribute*>( seStateUtil::CreateStateType( emElemState::Attribute ));
		this->pColor = static_cast<seStateColor*>( seStateUtil::CreateStateType( emElemState::Color ));
		this->pPos = static_cast<seStatePos*>( seStateUtil::CreateStateType( emElemState::Pos ));
		this->pScale = static_cast<seStateFloat2*>( seStateUtil::CreateStateType( emElemState::Scale ));
		this->pCenter = static_cast<seStateFloat2*>( seStateUtil::CreateStateType( emElemState::Center ));
		this->pRot = static_cast<seStateRot*>( seStateUtil::CreateStateType( emElemState::Rot ));
		this->pMask = static_cast<seStateInt1*>( seStateUtil::CreateStateType( emElemState::MaskAlpha ));


		// ʃXe[^X
		this->asStateList[static_cast<int>(emElemState::Attribute)] = evc_new seStateData(this->Attrbute(), true);
		this->asStateList[static_cast<int>(emElemState::Priority)] = evc_new seStateData(this->Priority(), true);
		this->asStateList[static_cast<int>(emElemState::Pos)] = evc_new seStateData(this->Pos(), true);
		this->asStateList[static_cast<int>(emElemState::Center)] = evc_new seStateData(this->Center(), true);
		this->asStateList[static_cast<int>(emElemState::Scale)] = evc_new seStateData(this->Scale(), true);
		this->asStateList[static_cast<int>(emElemState::Rot)] = evc_new seStateData(this->Rot(), true);
		this->asStateList[static_cast<int>(emElemState::MaskAlpha)] = evc_new seStateData(this->Mask(), true);
		this->asStateList[static_cast<int>(emElemState::Color)] = evc_new seStateData(this->Color(), true);
	
		this->pAttribute->Mask( seAttrMask::NoUse );
		this->pAttribute->Visible( seAttrVisible::Succes );
		this->pAttribute->Blend( seAttrBlend::Normal );
	
		siIOManager *pIOManager = seRootScene::Instance()->IOManager();
		this->sAnimMatrix = pIOManager->CreateMatrix();
		this->sAnimMatrix->Identity();
		this->sNowMatrix = pIOManager->CreateMatrix();
		this->sNowMatrix->Identity();
		this->sParentMatrixWork= pIOManager->CreateMatrix();
		this->sParentMatrixWork->Identity();
	}
}
