#include "glbPch.h"
#include "glb/glbDebug.h"
#include "seStateType.h"
#include "seStateTypeEx.h"
#include "evc/file/IseStateIO.h"

using namespace glb;
namespace evc
{

	IseStateType *seStateUtil::CreateStateType(emElemState::Type eStateType)
	{
		IseStateType *pStateType = null;
		switch (eStateType)
		{
			// 擪WvB͊Ď\[X
			case emElemState::FrameCtrl:
				pStateType = evc_new seStateFrameCtrl();
				break;
			case emElemState::Attribute:
				pStateType = evc_new seStateAttribute() ;
				break;
			// ]
			case emElemState::Rot:
				pStateType = evc_new seStateRot(0.0f, 0, seSpinType::Non);
				break;
	
			// XP[
			case emElemState::Scale:
				pStateType = evc_new seStateFloat2(&glbVector2::One);
				break;
	
			case emElemState::Pos: // ʒu
				pStateType = evc_new seStatePos();
				break;
			case emElemState::UV: // UV
			case emElemState::Center: // Sʒu
				pStateType = evc_new seStateFloat2(&glbVector2::Zero);
				break;
	
			case emElemState::Texture:
				pStateType = evc_new seStateSwitchInt1(0);
				break;
			case emElemState::TextureControl:
				pStateType = evc_new seStateTextureControl();
				break;
			case emElemState::Priority:
				pStateType = evc_new seStateInt1(0);
				break;
	
			// [v
			case emElemState::Loop:
				pStateType = evc_new seStateLoop(false, 1, false, 1);
				break;
			// J[
			case emElemState::Color:
				pStateType = evc_new seStateColor();
				break;
	
			// eLXgRg[
			case emElemState::TextControl :
				pStateType = evc_new seStateTextControl();
				break;
	
			// {[
			case emElemState::Volume:
				pStateType = evc_new seStateInt1(1000);
				break;
			// }XNAt@
			case emElemState::MaskAlpha:
				pStateType = evc_new seStateInt1(255);
				break;
			// V[Aj[Vݒ
			case emElemState::SceneAnim:
				pStateType = evc_new seStateString();
				break;
			// V[\[X[hݒ
			case emElemState::Load:
				pStateType = evc_new seStateLoadControl();
				break;
			// p
			case emElemState::Pan:
				pStateType = evc_new seStateInt1(0);
				break;
			// l
			case emElemState::Num:
				pStateType = evc_new seStateInt1(0);
				break;
			// lXe[^X
			case emElemState::NumState :
				{
					seStateNumInfo *l = evc_new seStateNumInfo();
					l->MaxDisit( 5 );
					pStateType = l;
				}
				break;
			case emElemState::SoundControl :
				pStateType = evc_new seStateSoundControl();
				break;
			// Obh_
			case emElemState::GridPos1:
			case emElemState::GridPos2:
			case emElemState::GridPos3:
			case emElemState::GridPos4:
				pStateType = evc_new seStateGridPoint();
				break;

			// p[eBN
			case emElemState::ParticleControl:
				pStateType = evc_new seStateParticles();
				break;

			default:
				break;
		}
	
		GLB_ASSERT(pStateType != null);
		return pStateType;
	}

	// seStateFloat2
	seStateFloat2::seStateFloat2()
		: sValue( glbVector2::Zero )
	{
	}

	seStateFloat2::seStateFloat2( const glbVector2 *sInVector)
	{
		this->sValue = *sInVector;
	}

	IseStateType *seStateFloat2::Clone()
	{
		return evc_new seStateFloat2(this->Value());
	}

	void seStateFloat2::Set(IseStateType *pInState)
	{
		this->sValue.Set(*(static_cast<seStateFloat2*>(pInState))->Value());
	}

	void seStateFloat2::Load(IseStateInput *pInput)
	{
		this->sValue = seIOUtil::ReadVector2(pInput);
	}

	void seStateFloat2::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		const glbVector2 *v = (static_cast<seStateFloat2*>(pBefore))->Value();
		this->sValue = (static_cast<seStateFloat2*>(pNext))->sValue;
		this->sValue.Sub(*v);
		this->sValue.Scale(fRate);
		this->sValue.Add(*v);
	}


	// seStateFloat1
	seStateFloat1::seStateFloat1()
		: fValue( 0.0f )
	{
	}

	seStateFloat1::seStateFloat1(float fInValue)
	{
		this->fValue = fInValue;
	}

	IseStateType *seStateFloat1::Clone()
	{
		return evc_new seStateFloat1(this->Value());
	}

	void seStateFloat1::Set(IseStateType *pInState)
	{
		this->fValue = (static_cast<seStateFloat1*>(pInState))->Value();
	}

	void seStateFloat1::Load(IseStateInput *pInput)
	{
		this->fValue = pInput->ReadFloat();
	}

	void seStateFloat1::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		float b = (static_cast<seStateFloat1*>(pBefore))->Value();
		float n = (static_cast<seStateFloat1*>(pNext))->Value();
		this->fValue = (n - b) * fRate + b;
	}

	// seStateInt2
	seStateInt2::seStateInt2()
		: sValue( glbVector2I::Zero )
	{
	}

	seStateInt2::seStateInt2( const glbVector2I *pVector )
	{
		this->sValue = *pVector;
	}

	IseStateType *seStateInt2::Clone()
	{
		return evc_new seStateInt2( &this->sValue );
	}

	void seStateInt2::Set(IseStateType *pInState)
	{
		seStateInt2 *si2 = static_cast<seStateInt2*>(pInState);
		this->sValue = *si2->Value();
	}

	void seStateInt2::Load(IseStateInput *pInput)
	{
		this->sValue.X = pInput->ReadInt();
		this->sValue.Y = pInput->ReadInt();
	}

	void seStateInt2::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		const glbVector2I *b = static_cast<seStateInt2*>(pBefore)->Value();
		const glbVector2I *n = static_cast<seStateInt2*>(pNext)->Value();
		this->sValue.X = static_cast<int>(static_cast<float>(n->X - b->X) * fRate) + b->X;
		this->sValue.Y = static_cast<int>(static_cast<float>(n->Y - b->Y) * fRate) + b->Y;
	}


	// seStateInt1
	seStateInt1::seStateInt1()
		: iValue( 0 )
	{
	}

	seStateInt1::seStateInt1(int iNum)
	{
		this->iValue = iNum;
	}

	IseStateType *seStateInt1::Clone()
	{
		return evc_new seStateInt1(this->Value());
	}

	void seStateInt1::Set(IseStateType *pInState)
	{
		this->iValue = (static_cast<seStateInt1*>(pInState))->Value();
	}

	void seStateInt1::Load(IseStateInput *pInput)
	{
		this->iValue = pInput->ReadInt();
	}

	void seStateInt1::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		int b = (static_cast<seStateInt1*>(pBefore))->iValue;
		int n = (static_cast<seStateInt1*>(pNext))->iValue;
		this->iValue = static_cast<int>(static_cast<float>(n - b) * fRate) + b;
	}

	// seStateSwitchInt1
	seStateSwitchInt1::seStateSwitchInt1()
	{
	}

	seStateSwitchInt1::seStateSwitchInt1(int iNum)
		:iValue(iNum)
	{
	}

	IseStateType *seStateSwitchInt1::Clone()
	{
		return evc_new seStateSwitchInt1(this->Value());
	}

	void seStateSwitchInt1::Set(IseStateType *pState)
	{
		this->iValue = (static_cast<seStateSwitchInt1*>(pState))->iValue;
	}

	void seStateSwitchInt1::Load(IseStateInput *pInput)
	{
		this->iValue = pInput->ReadInt();
	}


	// seStateString
	seStateString::seStateString()
	{
	}

	seStateString::seStateString( const glbString *pString )
	{
		this->strValue = *pString;
	}

	IseStateType *seStateString::Clone()
	{
		return evc_new seStateString(this->Value());
	}

	void seStateString::Set(IseStateType *pInState)
	{
		this->strValue = *(static_cast<seStateString*>(pInState))->Value();
	}

	void seStateString::Load(IseStateInput *pInput)
	{
		this->strValue = pInput->ReadString();
	}
}
