/**
 * @file seStateType.h
 * @brief 
 *
 * @author seri
 */
#ifndef SESTATETYPE_H
#define SESTATETYPE_H
#pragma once
#include "glb/glbDefine.h"
#include "glb/glbString.h"
#include "glb/glbColor.h"
#include "evc/common/seDefine.h"

using namespace glb;
namespace evc
{
	class IseStateInput;


	/// <summary>
	/// Description of seStateType.
	/// Xe[^X^CvC^tF[X
	/// lZEeqpC^tF[X
	/// </summary>
	class IseStateType
	{
	public:
		// f[^ݒ
		virtual void Set(IseStateType*) = 0;

		// f[^Z
		// Opf[^ȊO㏑܂B
		virtual void Add(IseStateType*) = 0;

		// Aj[V
		// @param pNext :̃f[^
		// @param pBefore :PÕf[^
		// @param fRate     :(0.0f=pBefore ` 1.0f=pNext) 
		// ȃAj[V͂I[o[ChĉB
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate) = 0;

		// 
		virtual void Load(IseStateInput*) = 0;

		// 
		virtual IseStateType *Clone() = 0;
	};

	// Xe[^X쐬
	class seStateUtil
	{
		// ŏIf[^ۑpXe[^X쐬
	public:
		static IseStateType *CreateStateType(emElemState::Type eStateType);
	};



	// float2ėpNX
	class seStateFloat2 : public IseStateType
	{
	private:
		glbVector2 sValue;
	public:
		const glbVector2 *Value() const { return &this->sValue; }
		void Value( const glbVector2 *pvalue ) { this->sValue = *pvalue; }

		seStateFloat2();
		seStateFloat2( const glbVector2 *pvalue );

		// N[
		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pState);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Load(IseStateInput *pInput);
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate);
	};

	// ėpfloat1
	class seStateFloat1 : public IseStateType
	{
	private:
		float fValue;

	public:
		float Value() const		{ return this->fValue; }
		void Value(float value) { this->fValue = value; }

		seStateFloat1();
		seStateFloat1(float fInValue);

		// N[
		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pInState);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Load(IseStateInput *pInput);
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate);
	};

	// int2ėpNX
	class seStateInt2 : public IseStateType
	{
	private:
		glbVector2I sValue;

	public:
		const glbVector2I *Value() const	   { return &this->sValue; }
		void Value( const glbVector2I *value ) { this->sValue = *value; }

		seStateInt2();
		seStateInt2( const glbVector2I *value );

		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pInState);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Load(IseStateInput *pInput);
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate);

	};

	// int1ėpNX
	class seStateInt1 : public IseStateType
	{
	private:
		int iValue;
	public:
		int Value() const	  { return this->iValue; }
		void Value(int value) { this->iValue = value; }

		seStateInt1();
		seStateInt1(int iNum);
		// N[
		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pInState);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Load(IseStateInput *pInput);
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate);
	};

	// Aj[VȂint1ėpNX
	class seStateSwitchInt1 : public IseStateType
	{
	private:
		int iValue;

	public:
		int Value() const	  { return this->iValue; }
		void Value(int value) { this->iValue = value; }

		// RXgN^
		seStateSwitchInt1();
		seStateSwitchInt1(int iNum);

		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pInState);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Load(IseStateInput *pInput);
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate){}
	};


	// NX
	class seStateString : public IseStateType
	{
	private:
		glbString strValue;
	public:
		const glbString *Value() const		 { return &this->strValue; }
		void Value( const glbString *pvalue) { this->strValue = *pvalue; }

		seStateString();
		seStateString( const glbString *str );
		virtual ~seStateString(){}

		// N[
		virtual IseStateType *Clone();
		virtual void Set(IseStateType *pInState);
		virtual void Load(IseStateInput *pInput);
		virtual void Add(IseStateType *pState) { this->Set(pState); }
		virtual void Anim(IseStateType *pNext, IseStateType *pBefore, float fRate){}
	};
}
#endif //SESTATETYPE_H

