#include "glbPch.h"
#include "glb/glbDebug.h"
#include "evc/common/seDefine.h"
#include "evc/common/seMath.h"
#include "seStateTypeEx.h"
#include "evc/file/IseStateIO.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/IO/siResource.h"
#include "evc/seResource/seResourceScene.h"



using namespace glb;
namespace evc
{
	// p[W
	const float seStatePos::Power = 3.0f;

	static const float fCircle = 2.0f * glb::PI; // PT̊px
	static const float fCircleH = fCircle * 0.5f; // 

	// ============================================================
	// seStateTextureControl
	seStateTextureControl::seStateTextureControl()
		: eHit(seTexHit::Non)
		, eAddress( seTexAddress::Non ){}

	void seStateTextureControl::Hit(seTexHit::Type value)
	{ 
		this->eHit = value; 
	}

	void seStateTextureControl::Set(IseStateType *pInState)
	{
		seStateTextureControl *t = static_cast<seStateTextureControl*>(pInState);
		this->eHit = t->eHit;
		this->eAddress = t->eAddress;
	}

	void seStateTextureControl::Add(IseStateType *pState)
	{
		seStateTextureControl *t = static_cast<seStateTextureControl*>(pState);
		if(t->eHit != seTexHit::Non) this->eHit = t->eHit;
		if(t->eAddress != seTexAddress::Non) this->eAddress = t->eAddress;
	}


	void seStateTextureControl::Load(IseStateInput *pInput)
	{
		this->eHit = pInput->ReadEnum<seTexHit,seTexHit::Type>();
		this->eAddress = pInput->ReadEnum<seTexAddress,seTexAddress::Type>();
	}

	IseStateType *seStateTextureControl::Clone()
	{
		seStateTextureControl *c = evc_new seStateTextureControl();
		c->Set(this);
		return c;
	}

	// ============================================================
	// seStateLoadControl
	bool seStateLoadControl::IsEnable(seLoadControl::Type eCheckType) const
	{
		return ((this->eState & eCheckType) == eCheckType);
	}

	seStateLoadControl::seStateLoadControl()
		: eState( seLoadControl::Load ){}

	seStateLoadControl::seStateLoadControl(seLoadControl::Type eInState)
	{
		this->eState = eInState;
	}

	IseStateType *seStateLoadControl::Clone()
	{
		return evc_new seStateLoadControl(this->eState);
	}

	void seStateLoadControl::Set(IseStateType *pInState)
	{
		this->eState = (static_cast<seStateLoadControl*>(pInState))->eState;
	}


	void seStateLoadControl::Load(IseStateInput *pInput)
	{
		this->eState = pInput->ReadEnum<seLoadControl,seLoadControl::Type>();
	}

	// ============================================================
	// seStateSoundControl
	seStateSoundControl::seStateSoundControl()
		: eState( seSoundControl::Non ){}

	seStateSoundControl::seStateSoundControl( seSoundControl::Type eInSoundControl )
	{
		this->eState = eInSoundControl;
	}

	IseStateType *seStateSoundControl::Clone()
	{
		return evc_new seStateSoundControl(this->eState);
	}

	void seStateSoundControl::Set(IseStateType *pInState)
	{
		this->eState = (static_cast<seStateSoundControl*>(pInState))->eState;
	}

	void seStateSoundControl::Add(IseStateType *pState)
	{
		seSoundControl::Type eType = (static_cast<seStateSoundControl*>(pState))->eState;
		if (eType != seSoundControl::Non) this->eState = eType;

	}
	


	void seStateSoundControl::Load(IseStateInput *pInput)
	{
		this->eState = pInput->ReadEnum<seSoundControl,seSoundControl::Type>();
	}

	// ============================================================
	// seStateAttribute
	seStateAttribute::seStateAttribute()
		: eBlend( seAttrBlend::Non )
		, eMask( seAttrMask::Non )
		, eVisible( seAttrVisible::Non ){}

	IseStateType *seStateAttribute::Clone()
	{
		seStateAttribute *a = evc_new seStateAttribute();
		a->Set(this);
		return a;
	}

	void seStateAttribute::Set(IseStateType *s)
	{
		seStateAttribute *a = static_cast<seStateAttribute*>(s);
		this->Blend(a->Blend());
		this->Mask(a->Mask());
		this->Visible(a->Visible());
	}

	void seStateAttribute::Add(IseStateType *s)
	{
		seStateAttribute *a = static_cast<seStateAttribute*>(s);
		if (a->eBlend != seAttrBlend::Non) this->eBlend = a->eBlend;
		if (a->eMask != seAttrMask::Non) this->eMask = a->eMask;
		if (a->eVisible != seAttrVisible::Non) this->eVisible = a->eVisible;
	}


	void seStateAttribute::Load(IseStateInput *pInput)
	{
		this->Blend(pInput->ReadEnum<seAttrBlend,seAttrBlend::Type>());
		this->Mask(pInput->ReadEnum<seAttrMask,seAttrMask::Type>());
		this->Visible(pInput->ReadEnum<seAttrVisible,seAttrVisible::Type>());
	}

	// ============================================================
	// seStateFrameCtrl
	seStateFrameCtrl::seStateFrameCtrl()
		: pSceneManager( null )
		, iFinishLoadAnimID( seAnimManager::InvalidID )
		, iLoadingAnimID( seAnimManager::InvalidID )
		, bWatchAll( false ){}


	bool seStateFrameCtrl::IsWatch(int iResourceID) const
	{
		return (est::find(this->lisWatchID.begin(), this->lisWatchID.end(), iResourceID )
					!= this->lisWatchID.end() );
	}

	void seStateFrameCtrl::AddWatch(int iResourceID)
	{
		if (!this->IsWatch( iResourceID ))
		{
			this->lisWatchID.push_back(iResourceID);
		}
	}

	void seStateFrameCtrl::RemoveWatch(int iResourceID)
	{
		est::vector<int>::iterator it = est::find(
						this->lisWatchID.begin(), 
						this->lisWatchID.end(), iResourceID );

		this->lisWatchID.erase( it );
	}

	void seStateFrameCtrl::ClearWatch()
	{
		this->lisWatchID.clear();
	}

	void seStateFrameCtrl::DoCheck()
	{
		seResourceManager *pResourceManager = this->pSceneManager->ResourceManager();
		bool bIsLoadEnd = true;
	
		// SẴV[\[X[hIĂȂ
		if (this->IsWatchAll())
		{
			const est::map<int,siResourceBase*> &resources = *pResourceManager->Piars();
			est::map<int,siResourceBase*>::const_iterator it = resources.begin();
			while(it != resources.end())
			{
				// [hĂȂV[\[XȂ
				if ( (it->second->Type() == emResourceState::Scene) &&
					 !it->second->IsEffective() )
				{
					bIsLoadEnd = false;
					break;
				}
				it ++;
			}

		}
		// ̃\[X̃[hIĂȂ
		else
		{
			for (est::vector<int>::const_iterator it = this->lisWatchID.begin();
						it != this->lisWatchID.end(); it++)
			{
				siResourceBase *pResource = pResourceManager->seIDManager<siResourceBase*>::Find(*it);
				pResource->Update();
				// PłgpłȂ\[XȂ
				if (!pResource->IsEffective())
				{
					bIsLoadEnd = false;
					break;
				}
			}
		}
	
		// [hIĂȂ
		int iNextAnimID = (bIsLoadEnd) ? this->iFinishLoadAnimID
									   : this->iLoadingAnimID;
	
		// ̃t[ŎsAj[V
		if (iNextAnimID != seAnimManager::InvalidID)
		{
			this->pSceneManager->RequestAnim(iNextAnimID);
		}
	}

	void seStateFrameCtrl::Set(IseStateType *pInState)
	{
		seStateFrameCtrl *m = static_cast<seStateFrameCtrl*>(pInState);
	
		this->iFinishLoadAnimID = m->iFinishLoadAnimID;
		this->iLoadingAnimID = m->iLoadingAnimID;
		this->bWatchAll = m->bWatchAll;
	
		this->lisWatchID.clear();
		this->lisWatchID = m->lisWatchID;
	}


	void seStateFrameCtrl::Load(IseStateInput *pInput)
	{
		this->lisWatchID.clear();
	
		this->iFinishLoadAnimID = static_cast<int>(pInput->ReadShort());
		this->iLoadingAnimID = static_cast<int>(pInput->ReadShort());
		this->bWatchAll = pInput->ReadBool();
	
		int iWatchCount = static_cast<int>(pInput->ReadShort());
		for (int i = 0; i < iWatchCount; i++)
		{
			this->lisWatchID.push_back(static_cast<int>(pInput->ReadShort()));
		}
	}

	IseStateType *seStateFrameCtrl::Clone()
	{
		seStateFrameCtrl *c = evc_new seStateFrameCtrl();
		c->Set(this);
		return c;
	}


	// ============================================================
	// seStateTextControl
	seStateTextControl::seStateTextControl()
		: iPage( 0 )
		, eState( seTextControl::TextSpeed ){}

	IseStateType *seStateTextControl::Clone()
	{
		seStateTextControl *c = evc_new seStateTextControl();
		c->Set(this);
		return c;
	}

	void seStateTextControl::Set(IseStateType *pInState)
	{
		seStateTextControl *t = static_cast<seStateTextControl*>(pInState);
		this->iPage = t->iPage;
		this->eState = t->eState;
	}


	void seStateTextControl::Load(IseStateInput *pInput)
	{
		this->iPage = static_cast<int>(pInput->ReadShort());
		this->eState = pInput->ReadEnum<seTextControl,seTextControl::Type>();
	}

	// ============================================================

	// seStateNumImage
	void seStateNumImage::SetImage(seoNumImage::Type eImageType, seObject *pObject)
	{
		this->apImageList[static_cast<int>(eImageType)] = pObject;
	}

	seObject *seStateNumImage::GetImage(seoNumImage::Type eImageType)
	{
		return this->apImageList[static_cast<int>(eImageType)];
	}

	seStateNumImage::seStateNumImage()
		: pObjectManager( null )
	{
		// SĖŖ߂
		this->apImageList.resize( seoNumImage::EnumCount() );
		for (int i = 0; i < seoNumImage::EnumCount(); i++)
		{
			this->apImageList[i] = null;
		}
	}

	IseStateType *seStateNumImage::Clone()
	{
		seStateNumImage *c = evc_new seStateNumImage();
		c->Set(this);
		return c;
	}

	void seStateNumImage::Set(IseStateType *pInState)
	{
		seStateNumImage *n = static_cast<seStateNumImage*>(pInState);
		this->pObjectManager = n->pObjectManager;
		this->apImageList.clear();
		this->apImageList = n->apImageList;
	}

	void seStateNumImage::Load(IseStateInput *pInput)
	{
		for (int i = 0; i < seoNumImage::EnumCount(); i++)
		{
			this->apImageList[i] = this->pObjectManager->Find(static_cast<int>(pInput->ReadShort()));
		}
	}

	// ============================================================
	// seStateNumInfo
	IseStateType *seStateNumInfo::Clone()
	{
		seStateNumInfo *c = evc_new seStateNumInfo();
		c->Set(this);
		return c;
	}


	void seStateNumInfo::Set(IseStateType *pInState)
	{
		seStateNumInfo *n = static_cast<seStateNumInfo*>(pInState);
		this->eArrange = n->eArrange;
		this->CharLen(n->CharLen());
		this->VisibleComma(n->VisibleComma());
		this->VisibleMinus(n->VisibleMinus());
		this->VisibleZeroBuries(n->VisibleZeroBuries());
		this->iMaxDisit = n->iMaxDisit;
		this->iPointDraw = n->iPointDraw;
		this->iNumLen = n->iNumLen;
		this->iCommaLen = n->iCommaLen;
	}

	void seStateNumInfo::Add(IseStateType *pState)
	{
		seStateNumInfo *n = static_cast<seStateNumInfo*>(pState);
		if (n->eArrange != seNumArrange::Non) this->eArrange = n->eArrange;
		if (n->eCharLen != seNumCharLen::Non) this->eCharLen = n->eCharLen;
		if (n->eVisibleComma != seNumVisible::Non) this->eVisibleComma = n->eVisibleComma;
		if (n->eVisibleMinus != seNumVisible::Non) this->eVisibleMinus = n->eVisibleMinus;
		if (n->eZeroBuries != seNumVisible::Non) this->eZeroBuries= n->eZeroBuries;
		this->iMaxDisit = n->iMaxDisit;
		this->iPointDraw = n->iPointDraw;
		this->iNumLen = n->iNumLen;
		this->iCommaLen = n->iCommaLen;
	}



	void seStateNumInfo::Load(IseStateInput *pInput)
	{
		this->eArrange = pInput->ReadEnum<seNumArrange, seNumArrange::Type>();
		this->CharLen(pInput->ReadEnum<seNumCharLen, seNumCharLen::Type>());
		this->VisibleComma(pInput->ReadEnum<seNumVisible,seNumVisible::Type>());
		this->VisibleMinus(pInput->ReadEnum<seNumVisible,seNumVisible::Type>());
		this->VisibleZeroBuries(pInput->ReadEnum<seNumVisible,seNumVisible::Type>());
		this->iMaxDisit = static_cast<int>(pInput->ReadShort());
		this->iPointDraw = static_cast<int>(pInput->ReadShort());
		this->iNumLen = static_cast<int>(pInput->ReadShort());
		this->iCommaLen = static_cast<int>(pInput->ReadShort());
	}

	// ============================================================

	IseStateType *seStateRot::Clone()
	{
		seStateRot *r = evc_new seStateRot();
		r->Set(this);
		return r;
	}

	void seStateRot::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		seStateRot *r = static_cast<seStateRot*>(pBefore);
		seStateRot *nr = static_cast<seStateRot*>(pNext);
	
		float fRot = nr->Rot(); // ڕWpx
		float fBefoRot = r->Rot(); // ̊px
	
		switch (this->SpinType())
		{
			case seSpinType::Non:
				{
					// ŏIl̕傫ƂčlĂ݂
					float fLRot = (fRot < fBefoRot) ? fRot + evc::fCircle : fRot;
					// ̕Z
					if ((fLRot - fBefoRot) < evc::fCircleH)
					{
						fRot = fLRot;
						fRot += evc::fCircle * static_cast<float>(this->RotNum());
					}
					// E̕Z
					else
					{
						// Ȅꍇ͌̊px̕傫̂Ƃčl
						if (fBefoRot < fRot)
						{
							fBefoRot += evc::fCircle;
						}
						fBefoRot += evc::fCircle * static_cast<float>(this->RotNum());
					}
				}
				break;
	
			// E
			case seSpinType::Right:
				// ŏIpxOxׂŃAj[VȂ
				if (fRot < fBefoRot)
				{
					fRot += evc::fCircle;
				}
				fRot += evc::fCircle * static_cast<float>(this->RotNum());
				break;

			// 
			case seSpinType::Left:
				// Ȅꍇ͌̊px̕傫̂Ƃčl
				if (fBefoRot < fRot)
				{
					fBefoRot += evc::fCircle;
				}
				fBefoRot += evc::fCircle * static_cast<float>(this->RotNum());
				break;
			// ړ̏ꍇ͂ł͉Ȃ
			case seSpinType::Direction:
				break;
			default:
				break;
		}
	
		this->Rot((fRot - fBefoRot) * fRate + fBefoRot);
	}

	void seStateRot::Set(IseStateType *pState)
	{
		seStateRot *r = static_cast<seStateRot*>(pState);
		this->fRot = r->fRot;
		this->eSpinType = r->eSpinType;
		this->iRotNum = r->iRotNum;
	}


	void seStateRot::Load(IseStateInput *pInput)
	{
		this->fRot = pInput->ReadFloat();
		this->eSpinType = pInput->ReadEnum<seSpinType,seSpinType::Type>();
		this->iRotNum = pInput->ReadShort();
	}

	// ============================================================

	IseStateType *seStateLoop::Clone()
	{
		return evc_new seStateLoop(this->bRevW, this->iLoopCountW, this->bRevH, this->iLoopCountH);
	}

	void seStateLoop::Set(IseStateType *pInState)
	{
		seStateLoop *l = static_cast<seStateLoop*>(pInState);
		this->bRevH = l->bRevH;
		this->iLoopCountH = l->iLoopCountH;
		this->bRevW = l->bRevW;
		this->iLoopCountW = l->iLoopCountW;
	}

	void seStateLoop::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		// [v̂݃Aj[V
		seStateLoop *l = static_cast<seStateLoop*>(pBefore);
		seStateLoop *nl = static_cast<seStateLoop*>(pNext);
		this->iLoopCountW = static_cast<int>(static_cast<float>(nl->iLoopCountW - l->iLoopCountW) * fRate) + l->iLoopCountW;
		this->iLoopCountH = static_cast<int>(static_cast<float>(nl->iLoopCountH - l->iLoopCountH) * fRate) + l->iLoopCountH;
	}

	void seStateLoop::Load(IseStateInput *pInput)
	{
		this->bRevH = pInput->ReadBool();
		this->iLoopCountH = static_cast<int>(pInput->ReadShort());
		this->bRevW = pInput->ReadBool();
		this->iLoopCountW = static_cast<int>(pInput->ReadShort());
	}


	// ============================================================



	seStatePos::seStatePos()
		: _value(0.0f, 0.0f)		// ʒu
		, _type(sePosType::Line)	// ړ^Cv

	{
		this->_exData[0] = 0.0f;
		this->_exData[1] = 0.0f;
	}

	seStatePos::seStatePos( seStatePos *mst )
	{
		this->Set(mst);
	}

	IseStateType *seStatePos::Clone()
	{
		return evc_new seStatePos( this );
	}


	void seStatePos::Set(IseStateType *mst)
	{
		seStatePos *pos = static_cast<seStatePos *>(mst);
		this->Value( pos->Value() );
		this->Type( pos->Type() );
		this->_exData[0] = pos->ExData()[0];
		this->_exData[1] = pos->ExData()[1];
	}

	void seStatePos::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		seStatePos *b = static_cast<seStatePos *>(pBefore);
		seStatePos *n = static_cast<seStatePos *>(pNext);
		glbVector2 v = n->_value;
		glbVector2 bPower = glbVector2::Zero;


		switch (this->Type())
		{
			// 
			case sePosType::Line:
				v.Sub( *b->Value() );
				v.Scale( fRate );
				v.Add( *b->Value() );
				break;

			// pX
			case sePosType::Path:
				{
					bPower = glbVector2::Zero;
					// PÕ^Cv
					switch (b->Type())
					{
						// 
						case sePosType::Line:
							break;
						// pX
						case sePosType::Path:
							bPower = glbVector2(b->ExData()[0],
												b->ExData()[1]);
							break;
						default:
							break;
					}

					glbVector2 np = glbVector2(n->_exData[0], n->_exData[1]);
					v = seMath::GetHermiteCurvePoint(
							b->Value(), &bPower,
							&v, &np,
							fRate, seStatePos::Power );
				}
				break;
			default:
				break;
		}

		this->_value = v;

	}

	void seStatePos::Load(IseStateInput *pInput)
	{
		glbVector2 v = seIOUtil::ReadVector2(pInput); 
		this->Value( &v );
		this->Type(pInput->ReadEnum<sePosType,sePosType::Type>());
		int iCount = (int)pInput->ReadByte();

		// gf[^[h
		for (int i = 0; i < iCount; i++)
		{
			this->_exData[ i ] = pInput->ReadFloat();
		}
	}


	// ============================================================
	// J[

	void seStateColor::Load(IseStateInput *pInput)
	{
		this->sColor = seIOUtil::ReadColor(pInput);
	}

	IseStateType *seStateColor::Clone()
	{
		return evc_new seStateColor(&this->sColor);
	}

	void seStateColor::Set(IseStateType *pInState)
	{
		this->sColor = (static_cast<seStateColor*>(pInState))->sColor;
	}

	static unsigned char ColorParam(unsigned char bNext, unsigned char bBefor, float fRate )
	{
		int iColor = static_cast<int>(static_cast<float>(bNext - bBefor) * fRate) + bBefor;
		if (iColor < 0) return 0;
		if (iColor > 255) return 255;
		return static_cast<unsigned char>(iColor);
	}

	void seStateColor::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		glbColor *c = &(static_cast<seStateColor*>(pBefore))->sColor;
		glbColor *nc = &(static_cast<seStateColor*>(pNext))->sColor;
		this->sColor.R = evc::ColorParam(nc->R, c->R, fRate);
		this->sColor.G = evc::ColorParam(nc->G, c->G, fRate);
		this->sColor.B = evc::ColorParam(nc->B, c->B, fRate);
		this->sColor.A = evc::ColorParam(nc->A, c->A, fRate);
	}

	// ============================================================
	// ObhIuWFNgNX

	seStateGridPoint::seStateGridPoint()
		: _value(0.0f, 0.0f)
		, _powerH(0.0f, 0.0f)
		, _powerV(0.0f, 0.0f)
	{
	}

#if 1
	seStateGridPoint::seStateGridPoint(seStateGridPoint *pMst)
	{
		this->Set(pMst);
	}
	// N[
	IseStateType *seStateGridPoint::Clone()
	{
		return evc_new seStateGridPoint(this);
	}
#endif

	void seStateGridPoint::Set(IseStateType *pState)
	{
		const seStateGridPoint *pMst = static_cast<const seStateGridPoint*>(pState);
		this->_value.Set(pMst->_value);
		this->_powerH.Set(pMst->_powerH);
		this->_powerV.Set(pMst->_powerV);
	}
	void seStateGridPoint::Load(IseStateInput *pInput)
	{
		glbVector2 v = seIOUtil::ReadVector2(pInput);
		this->_value.Set(v);
		v = seIOUtil::ReadVector2(pInput);
		this->_powerH.Set(v);
		v = seIOUtil::ReadVector2(pInput);
		this->_powerV.Set(v);
	}


	// Q_Ɣ䗦猻݂̈ʒuZo
	static glbVector2 vecAnim( const glbVector2 *pNext, const glbVector2 *pBfo, float fRate)
	{
		glbVector2 v = *pNext;
		v.Sub(*pBfo);
		v.Scale(fRate);
		v.Add(*pBfo);
		return v;
	}


	void seStateGridPoint::Anim(IseStateType *pNext, IseStateType *pBefore, float fRate)
	{
		const seStateGridPoint *b = static_cast<const seStateGridPoint*>(pBefore);
		const seStateGridPoint *n = static_cast<const seStateGridPoint*>(pNext);
		this->_value = vecAnim(&n->_value, &b->_value, fRate);
		this->_powerH = vecAnim(&n->_powerH, &b->_powerH, fRate);
		this->_powerV = vecAnim(&n->_powerV, &b->_powerV, fRate);
	}


	// ============================================================
	// p[eBN

	seStateParticles::seStateParticles()
		: _outCount(1.0f)		// Pt[̏o͐
		, _startFrame(0)		// X^[gt[
		, _play(ParticlesPlay::Play)	// ˊJnE~
		, _priority(ParticlesDrawPriority::Top)	// p[eBN̔zuʒu
		, _successionScale(ParticlesSuccession::NoUse)	// ẽXP[p邩ۂ
		, _successionRot(ParticlesRot::Normal)	// ẻ]p邩ۂ
		, _outType(ParticlesOutType::Plane)	// o̓^Cv
		, _outVector(0.0f, 0.0f)	// o͕yуp[
		, _outScaleLenge(1.0f) // o͔͈(p[XP[)
		, _outLenge(0.0f)	// o͔͈(px)
		, _influenceVector(0.0f, 0.0f) // eyуp[
		, _influenceScaleLenge(1.0f)	// e͈(p[XP[)
		, _influenceLenge(0.0f) // e͈
	{

	}

	seStateParticles::seStateParticles(seStateParticles *pMst)
	{
		this->Set(pMst);
	}

	void seStateParticles::Set(IseStateType *pState)
	{
		seStateParticles *pMst = static_cast<seStateParticles *>(pState);
		this->_outCount = pMst->_outCount;
		this->_startFrame = pMst->_startFrame;
		this->_play = pMst->_play;
		this->_priority = pMst->_priority;
		this->_successionRot = pMst->_successionRot;
		this->_successionScale = pMst->_successionScale;
		this->_outType = pMst->_outType;
		this->_outVector = pMst->_outVector;
		this->_outScaleLenge = pMst->_outScaleLenge;
		this->_outLenge = pMst->_outLenge;
		this->_influenceVector = pMst->_influenceVector;
		this->_influenceScaleLenge = pMst->_influenceScaleLenge;
		this->_influenceLenge = pMst->_influenceLenge;
	}

	void seStateParticles::Add(IseStateType *pState)
	{
		seStateParticles *pMst = static_cast<seStateParticles *>(pState);
		this->_outCount = pMst->_outCount;
		this->_startFrame = pMst->_startFrame;
		if (pMst->_play != ParticlesPlay::Non) this->_play = pMst->_play;
		if (pMst->_priority != ParticlesDrawPriority::Non) this->_priority = pMst->_priority;
		if (pMst->_successionRot != ParticlesRot::Non) this->_successionRot = pMst->_successionRot;
		if (pMst->_successionScale != ParticlesSuccession::Non) this->_successionScale = pMst->_successionScale;
		if (pMst->_outType != ParticlesOutType::Non) this->_outType = pMst->_outType;
		this->_outVector = pMst->_outVector;
		this->_outScaleLenge = pMst->_outScaleLenge;
		this->_outLenge = pMst->_outLenge;
		this->_influenceVector = pMst->_influenceVector;
		this->_influenceScaleLenge = pMst->_influenceScaleLenge;
		this->_influenceLenge = pMst->_influenceLenge;
	}

	void seStateParticles::Load(IseStateInput *pInput)
	{
		this->_outCount = pInput->ReadFloat();
		this->_startFrame = pInput->ReadInt();
		this->_play = pInput->ReadEnum<ParticlesPlay,ParticlesPlay::Type>();
		this->_priority = pInput->ReadEnum<ParticlesDrawPriority,ParticlesDrawPriority::Type>();
		this->_successionRot = pInput->ReadEnum<ParticlesRot,ParticlesRot::Type>();
		this->_successionScale = pInput->ReadEnum<ParticlesSuccession,ParticlesSuccession::Type>();
		this->_outType = pInput->ReadEnum<ParticlesOutType,ParticlesOutType::Type>();
		this->_outVector = seIOUtil::ReadVector2(pInput);
		this->_outScaleLenge = pInput->ReadFloat();
		this->_outLenge = pInput->ReadFloat();
		this->_influenceVector = seIOUtil::ReadVector2(pInput);
		this->_influenceScaleLenge = pInput->ReadFloat();
		this->_influenceLenge = pInput->ReadFloat();
	}

	IseStateType *seStateParticles::Clone()
	{
		return evc_new seStateParticles(this);
	}


	// ============================================================
	// p[eBNIuWFNgXe[^X
	// IuWFNgł̂ݎgp
	seStateParticlesInfo::seStateParticlesInfo()
		: _rotLenge(0.0f)	// px
		, _scaleLenge(1.0f) // XP[
		, _lifeMin(10) 	// ŒZ
		, _lifeMax(13)	// Œ
		, _maxNum(100)	// ő`搔
	{
	}

	seStateParticlesInfo::seStateParticlesInfo(seStateParticlesInfo *pMst)
	{
		this->Set(pMst);
	}


	void seStateParticlesInfo::Load(IseStateInput *pInput)
	{
		glbColor c = seIOUtil::ReadColor(pInput);
		this->_color.Start(c);
		c = seIOUtil::ReadColor(pInput);
		this->_color.End(c);

		this->_rot.Start(pInput->ReadFloat());
		this->_rot.End(pInput->ReadFloat());

		this->_rotLenge = pInput->ReadFloat();

		glbVector2 v = seIOUtil::ReadVector2(pInput);
		this->_scale.Start(v);
		v = seIOUtil::ReadVector2(pInput);
		this->_scale.End(v);

		this->_scaleLenge = pInput->ReadFloat();

		v = seIOUtil::ReadVector2(pInput);
		this->_center.Start(v);
		v = seIOUtil::ReadVector2(pInput);
		this->_center.End(v);

		this->_lifeMin = pInput->ReadInt();
		this->_lifeMax = pInput->ReadInt();
		// o͎ݒ
		this->_maxNum = pInput->ReadInt();

		this->_color.LoadAnim(pInput);
		this->_rot.LoadAnim(pInput);
		this->_scale.LoadAnim(pInput);
		this->_center.LoadAnim(pInput);
	}

}
