#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"

#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"

#include "seoColorSprite.h"

using namespace glb;
namespace evc
{

	seObject *seoColorSpriteWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}

	const glbVector2 *seoColorSpriteWork::GetSize()
	{
		return this->pObject->Size();
	}

	seoColorSpriteWork::seoColorSpriteWork( seElem *pInElem, seoColorSprite *pInObjct)
		: pObject( pInObjct )
		, pElem( pInElem )
	{
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
	}

	void seoColorSpriteWork::Draw()
	{
		this->pObject->Draw( this->pElem->ElemID(),
							 this->pElem->NowMatrix(),
							 this->pElem->State()->NowColor() );
	}

	void seoColorSpriteWork::Update( float fAddTime ){}



	// seoColorSprite
	seoColorSprite::seoColorSprite()
		: pColorSprite( null )
		, sSize( 10.0f, 10.0f )
		, sColor( &glbColor::White )
	{

	}


	seoColorSprite::~seoColorSprite()
	{
		GLB_SAFE_DELETE( this->pColorSprite );
	}

	void seoColorSprite::Size( const glbVector2 *value )
	{
		this->sSize = *value;
		this->pColorSprite->UpdateSize( &this->sSize);
	}


	void seoColorSprite::Init(int iInObjectID, seSceneManager *pInScene)
	{
		seObject::Init(iInObjectID, pInScene);
		// VID蓖Ăꂽ̂ōȂ
		if (this->pColorSprite != null)
		{
			delete this->pColorSprite;
		}
		this->pColorSprite = seRootScene::Instance()->IOManager()->CreateColorSprite(this->ObjectID());
		this->pColorSprite->UpdateSize(&this->sSize);
	}

	seObjectWork *seoColorSprite::CreateDrawWork(seElem *pElem)
	{
		return static_cast<seObjectWork *>(evc_new seoColorSpriteWork(pElem, this));
	}

	void seoColorSprite::Draw(int iElemID, const IGLMatrix *m, const glbColor *sMulColor )
	{
		glbColor c = *sMulColor;
		c.Mul( &this->sColor );
		this->pColorSprite->Draw( iElemID, m, &c );
	}

	void seoColorSprite::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->sSize = seIOUtil::ReadVector2(pInput);
		this->sColor = seIOUtil::ReadColor(pInput);
	}
}
