#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/IGLMatrix.h"
// #include "glb/glbDebug.h"
#include "seFontData.h"
#include "seFontChar.h"
#include "seFontLine.h"
#include "seFontPage.h"
#include "seoFont.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seAnimManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seKeyControlManager.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/file/IseStateIO.h"


using namespace glb;
namespace evc
{
	// =======================================================

	void seFontTextState::CreateImage(seFontData *pFontData)
	{
		this->iFontStyleID = pFontData->FontIO()->CreateFontStyle(
						&this->strFontName, this->fHeight, this->Style());
		pFontData->Style(this->iFontStyleID);
		pFontData->Height(this->fHeight);
	}

	void seFontTextState::Load(IseStateInput *pInput)
	{
		this->strFontName = pInput->ReadString();
		this->Height(pInput->ReadFloat());
		this->Style(static_cast<seFontStyle::Type>(pInput->ReadByte()));
	}

	// =======================================================

	void seFontTextColor::CreateImage(seFontData *pFontData)
	{
		// x[XJ[
	#if defined(false)
		glbColor *c = this->sColor;
		c->Mul(pFontData->BaseColor());
		pFontData->Color(c);
	#else
		// x[XJ[͕`掞Ɋ|
		pFontData->Color(&this->sColor);
	#endif    
	}

	void seFontTextColor::Load(IseStateInput *pInput)
	{
		this->sColor = seIOUtil::ReadColor(pInput);
	}

	// =======================================================

	void seFontTextString::CreateImage(seFontData *pFontData)
	{
		// Psf[^ȂȂ쐬܂łɐݒ肳ꂽ
		// sɔfׁAC͂ō쐬Kv܂B
		seFontPage *pPage = (pFontData->Page()->size() == 0)
								? pFontData->CreateNewPage()
								: pFontData->GetLastPage();
	
		if (pPage->Lines()->size() == 0)
		{
			pPage->Add(evc_new seFontLine(pFontData->Height()));
		}
	
		// sR[h̐ݒ
		short sChangingCode = (short)'\n';
		short sEndCode = (short)'\0';
		switch( this->strText.Format() )
		{
			// jR[h̏ꍇ
			case seTextFormat::Utf16:
				sChangingCode = L'\n';
				sEndCode = L'\0';
				break;
			// Shift-jis̏ꍇ
			case seTextFormat::Shift_Jis:
				sChangingCode = (short)'\n';
				sEndCode = (short)'\0';
				break;
		}

		int iAddCounter = 1;
		const est::vector<unsigned char> &rBuffer = this->strText.Buffer();
		for( int i = 0; i < (int)rBuffer.size(); i+= iAddCounter )
		{
			short sCode = 0;
			switch( this->strText.Format() )
			{
				// jR[h̏ꍇ
				case seTextFormat::Utf16:
					{
						unsigned short s1 = (unsigned short)(unsigned char)rBuffer[ i ];
						unsigned short s2 = (unsigned short)(unsigned char)rBuffer[ i + 1 ] * 256;
						sCode = (short)(s1 + s2);
						iAddCounter = 2;
					}
					break;
				// Shift-jis̏ꍇ
				case seTextFormat::Shift_Jis:
					unsigned char c = rBuffer[ i ];
					// 2byteȂ
					if ( (c>=0x81 && c<=0x9f) || (c>=0xe0 && c<=0xfc))
					{
						sCode = (short)((unsigned short)c + 
										(unsigned short)(rBuffer[ i + 1 ] * 256));
						iAddCounter = 2;
					}
					// 1byteȂ
					else
					{
						sCode = (short)c;
						iAddCounter = 1;
					}
					break;
			}

			// IR[hȂ
			if ( sCode == sEndCode )
			{
				break;
			}
			// sR[hȂ
			else if ( sCode == sChangingCode )
			{
				pPage->Add( evc_new seFontLine(pFontData->Height()));
			}
			// ȊO͕Ƃēo^
			else
			{
				siObjectFontChar *pCharInfo = pFontData->FontIO()->CreateChar(
							pFontData->Style(), this->strText.Format(), sCode);
				pFontData->AddChar(pCharInfo, pFontData->Color());
			}

		}
	}

	void seFontTextString::Load(IseStateInput *pInput)
	{
		this->strText = pInput->ReadString();
	}

	// =======================================================

	void seFontSpKey::CreateImage(seFontData *pFontData)
	{
		switch (this->eSpKey)
		{
			case SpKey::Non:
				break;
			case SpKey::Variable:
	
				break;
	
			case SpKey::NextPage:
				pFontData->CreateNewPage();
				break;
			// L[[h
			case SpKey::KeyWord:
				{
					seFontTextString *pFontString = evc_new seFontTextString();
					pFontString->Text(pFontData->ParentObject()->Scene()->KeyWord()->GetString( this->Text() ));
					pFontString->CreateImage(pFontData);
				}
				break;
			case SpKey::Speed:
				pFontData->AddCharDataEx(evc_new seFontCharExSpeed(this->Tag()->f));
				break;
			case SpKey::Wait:
				pFontData->AddCharDataEx(evc_new seFontCharExWait(this->Tag()->f));
				break;
			case SpKey::Anim:
				pFontData->AddCharDataEx(evc_new seFontCharExAnim(this->Text()));
				break;
			default:
				break;
		}
	
	}

	void seFontSpKey::Load(IseStateInput *pInput)
	{
		this->SpKey(static_cast<SpKey::Type>(pInput->ReadByte()));
		SpKeyInput::Type eSpKeyInput = static_cast<SpKeyInput::Type>(pInput->ReadByte());
	
		switch (eSpKeyInput)
		{
			case SpKeyInput::Float:
				this->Tag(pInput->ReadFloat());
				break;
			case SpKeyInput::String:
				this->strText = pInput->ReadString();
				break;
			default:
				break;
		}
	}

	// =======================================================


	seFontPage *seFontData::GetLastPage()
	{
		GLB_ASSERT( !this->lisPage.empty() );
		return this->lisPage[this->lisPage.size() - 1];
	}

	void seFontData::AddChar(siObjectFontChar *pCharInfo, const glbColor *sColor)
	{
		seFontChar *pChar = evc_new seFontChar(pCharInfo, sColor);

		// gf[^o^
		est::vector<IseFontCharEx*>::iterator it = this->lisCharEx.begin();
		for( ; it != this->lisCharEx.end(); it ++)
		{
			pChar->AddData( *it );
		}
		this->lisCharEx.clear();
		this->GetLastPage()->GetLastLine()->Add(pChar);
	}

	void seFontData::AddCharDataEx(IseFontCharEx *pDataEx)
	{
		this->lisCharEx.push_back(pDataEx);
	}

	seFontPage *seFontData::CreateNewPage()
	{
		seFontPage *pPage = evc_new seFontPage();
		this->lisPage.push_back(pPage);
		return pPage;
	}

	seFontData::seFontData(seObject *pParent)
	{
		this->pParentObject = pParent;
		this->pObjectFont = seRootScene::Instance()->IOManager()->CreateObjectFont(0);
	}


	void seFontData::Clear()
	{
		glb::DeleteVector( this->lisFontData );
		glb::DeleteVector( this->lisPage );
		// f[^B
		this->pObjectFont->Clear();
	}

	void seFontData::Add(seFontTextBase *pData)
	{
		this->lisFontData.push_back(pData);
	}

	void seFontData::CreateImageText()
	{
		this->pObjectFont->Clear();
		glb::DeleteVector( this->lisCharEx );
		glb::DeleteVector( this->lisPage );


		// Sf[^o^
		this->pObjectFont->BeginCharCreating();
	
		VECTOR_FOR_EACH( seFontTextBase *, this->lisFontData,
		{
			(*it)->CreateImage(this);
		});


		// XVʒm
		this->pObjectFont->EndCharCreating();
	
		// TCYĐݒ
		this->sSize = glbVector2::Zero;
	
		VECTOR_FOR_EACH( seFontPage *, this->lisPage,
		{
			(*it)->UpdateSize();
			const glbVector2 *pSize = (*it)->Size();
			this->sSize.X = glb::MaxValue(this->sSize.X, pSize->X);
			this->sSize.Y = glb::MaxValue(this->sSize.Y, pSize->Y);
		});
	}


	seFontChar *seFontData::GetCharInfo(int iPage, int iTextNum)
	{
		if (this->lisPage.empty()) return null;
	
		// ~bg`FbN
		iPage = glb::MinValue(glb::MaxValue(iPage, 0), (int)(this->lisPage.size() - 1));
		return this->lisPage[iPage]->GetCharInfo(iTextNum);
	}

	void seFontData::Draw(int iElemID, const IGLMatrix *mParent, 
						const glbColor *sColor, int iPage, int iDrawNum)
	{
		if (this->lisPage.empty()) return;
	
		// ~bg`FbN
		iPage = glb::MinValue(glb::MaxValue(iPage, 0), (int)(this->lisPage.size() - 1));
	
		// Cӂ̃y[W`
		this->lisPage[iPage]->Draw(iElemID, mParent, sColor, iDrawNum);
	}

	seFontData::~seFontData()
	{
		this->Clear();
		glb::DeleteVector( this->lisCharEx );
		GLB_SAFE_DELETE( this->pObjectFont );
	}

	void seFontData::Load(IseStateInput *pInput)
	{
		this->Clear();
	
		this->sBaseColor = seIOUtil::ReadColor(pInput);
		this->sSize = seIOUtil::ReadVector2(pInput);
	
	
		int iDataCount = static_cast<int>(pInput->ReadShort());
		for (int i = 0; i < iDataCount; i++)
		{
			SaveFontInfo::Type eFontInfoType = static_cast<SaveFontInfo::Type>(pInput->ReadByte());
			seFontTextBase *pFontText = null;
			switch (eFontInfoType)
			{
				case SaveFontInfo::State:
					pFontText = evc_new seFontTextState();
					break;
				case SaveFontInfo::Color:
					pFontText = evc_new seFontTextColor();
					break;
				case SaveFontInfo::Text:
					pFontText = evc_new seFontTextString();
					break;
				case SaveFontInfo::SpKey:
					pFontText = evc_new seFontSpKey();
					break;
				default:
					GLB_ASSERT(false);
					break;
			}
			pFontText->Load(pInput);
			this->Add(pFontText);
		}
		this->CreateImageText();
	}
}
