/**
 * @file seFontData.h
 * @brief 
 *
 * @author seri
 */
#ifndef SEFONTDATA_H
#define SEFONTDATA_H
#pragma once
#include "glb/glbDefine.h"
#include "glb/glbString.h"
#include "glb/glbVector.h"
#include "glb/glbColor.h"
#include "glb/IGLMatrix.h"
#include "evc/IO/siObjectFont.h"

using namespace glb;
namespace evc
{
	class siObjectFont;
	class siObjectFontChar;
	class IseStateInput;
	class IseFontCharEx;
	class seFontPage;
	class seFontData;
	class seFontChar;
	class seFontTextBase;
	class seObject;

	// L[^Cv
	struct SpKey
	{
		enum Type
		{
			Non = 0, // L[ł͂Ȃ
			Variable, // "\\v[", // ϐ
			NextPage, // "\\n", // y[W
			KeyWord, // "\\k[", // L[[h
			Speed, // "\\s[", // Xs[h
			Wait, // ҋ@
			Anim, // Aj[VĂяo
		};
	};

	// ^Cv
	struct SpKeyInput
	{
		enum Type
		{
			Non = 0, // Ȃ
			Float, // float
			String, // string
		};
	};

	// Z[uptHg
	struct SaveFontInfo
	{
		enum Type
		{
			State = 0, // X^C
			Color, // J[
			Text, // eLXg
			SpKey, // L[
		};
	};


	// x[X
	class seFontTextBase
	{
	public:

		virtual ~seFontTextBase(){}

		// `C[W쐬
		virtual void CreateImage(seFontData*) = 0;

		// oCi[[h
		virtual void Load(IseStateInput*) = 0;
	};

	// Xe[^X
	class seFontTextState : public seFontTextBase
	{
	private:
		int iFontStyleID;
		seFontStyle::Type eStyle;
		float fHeight;
		glbString strFontName;

	public:
		virtual ~seFontTextState(){}

		seFontStyle::Type Style() const		{ return this->eStyle; }
		void Style(seFontStyle::Type value)	{ this->eStyle = value; }

		// tHg̍擾Eݒ
		float Height() const		{ return this->fHeight; }
		void Height(float value)	{ this->fHeight = value; }

		// tHg̎擾Eݒ
		const glbString *FontName() const	 { return &this->strFontName; }
		void FontName(const glbString *value){ this->strFontName = *value; }


		// `C[W쐬
		virtual void CreateImage(seFontData *pFontData);

		// [h
		virtual void Load(IseStateInput *pInput);
	};

	// J[f[^
	class seFontTextColor : public seFontTextBase
	{
	private:
		glbColor sColor;

	public:
		// J[
		const glbColor *Color() const		{ return &this->sColor; }
		void Color( const glbColor *value)	{ this->sColor = *value; }


		// `C[W쐬
		virtual void CreateImage(seFontData *pFontData);

		// [h
		virtual void Load(IseStateInput *pInput);

	};

	// f[^
	class seFontTextString : public seFontTextBase
	{
	private:
		glbString strText;

	public:
		virtual ~seFontTextString(){}

		// 
		const glbString *Text() const	 { return &this->strText; }
		void Text(const glbString *value){ this->strText = *value; }

		// `C[W쐬
		virtual void CreateImage(seFontData *pFontData);

		// [h
		virtual void Load(IseStateInput *pInput);
	};


	class seFontSpKey : public seFontTextBase
	{
	private:
		SpKey::Type eSpKey;
		ArbValue oTag;		// L[ʃf[^
		glbString strText;	// L[ʃf[^(p)

	public:

		virtual ~seFontSpKey(){}

		// L[ʃf[^
		const ArbValue *Tag() const	{ return &this->oTag; }
		void Tag( int iTag )		{ this->oTag.i = iTag; }
		void Tag( float fTag )		{ this->oTag.f = fTag; }
		void Tag( void *pTag )		{ this->oTag.pv = pTag; }

		const glbString *Text() const		{ return &this->strText; }
		void Text( const glbString *value ) { this->strText = *value; }

		SpKey::Type SpKey() const	  { return this->eSpKey; }
		void SpKey(SpKey::Type value) { this->eSpKey = value; }

		// `C[W쐬
		virtual void CreateImage(seFontData *pFontData);

		// [h
		virtual void Load(IseStateInput *pInput);

	};



	// bZ[W
	class seFontData
	{
		// `惏[N
	private:
		est::vector<IseFontCharEx*>	lisCharEx; // ŏ̃f[^
		est::vector<seFontPage*>	lisPage;
		est::vector<seFontTextBase*>lisFontData;

		float fHeight; // Ōɐݒ肳ꂽ
		glbColor sColor; // x[XJ[
		siObjectFont *pObjectFont;
		glbVector2 sSize; // TCY
		glbColor sBaseColor; // x[XJ[
		seObject *pParentObject;
		int iStyle; // tHgXe[^X

	public:

		// y[WXg
		const est::vector<seFontPage*> *Page() const { return &this->lisPage; }

		// tHgXg
		const est::vector<seFontTextBase*> *Font() const { return &this->lisFontData; }

		// 擾Eݒ
		float Height() const		{ return this->fHeight; }
		void Height(float value)	{ this->fHeight = value; }

		// tHgX^C擾Eݒ
		int Style() const		{ return this->iStyle; }
		void Style(int value)	{ this->iStyle = value; }

		// J[擾Eݒ
		const glbColor *Color() const		{ return &this->sColor; }
		void Color( const glbColor *value ) { this->sColor = *value; }

		// tHg`擾
		siObjectFont *FontIO() { return this->pObjectFont; }

		// `̈TCY擾
		const glbVector2 *Size() const  { return &this->sSize; }

		// x[XJ[̎擾Eݒ
		const glbColor *BaseColor() const		{ return &this->sBaseColor; }
		void BaseColor( const glbColor *value)	{ this->sBaseColor = *value; }

		// eIuWFNg
		seObject *ParentObject() { return this->pParentObject; }


		// o^֌W

		// Ō̍s擾
		seFontPage *GetLastPage();
		// Ō̃y[W̍Ō̍sɃf[^ǉ
		void AddChar( siObjectFontChar *pCharInfo, const glbColor *sColor);
		// gf[^𒙂߂
		void AddCharDataEx(IseFontCharEx *pDataEx);


		// Vs쐬
		seFontPage *CreateNewPage();

		// RXgN^
		seFontData(seObject *pParent);
		virtual ~seFontData();



		// f[^NA
		void Clear();

		// Xe[^Xǉ
		void Add(seFontTextBase *pData);


		// `NXf[^쐬
		void CreateImageText();

		// Cӂ̃y[W̔Cӂ̕ڂ̏擾
		seFontChar *GetCharInfo(int iPage, int iTextNum);

		// `
		void Draw( int iElemID, 
				const IGLMatrix *mParent, 
				const glbColor *sColor, 
				int iPage, 
				int iDrawNum );



		// [h
		void Load(IseStateInput *pInput);
	};



}
#endif //SEFONTDATA_H

