#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "seoFrame.h"


using namespace glb;
namespace evc
{

	seObject *seoFrameWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}

	const glbVector2 *seoFrameWork::GetSize()
	{
		return this->pObject->Size();
	}

	seoFrameWork::seoFrameWork(seElem *pInElem, seoFrame *pInObjct)
	{
		this->pUV = static_cast<seStateFloat2*>(seStateUtil::CreateStateType(emElemState::UV));
		this->pElem = pInElem;
		this->pObject = pInObjct;
	
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
		this->pElem->State()->LinkStateData(emElemState::UV, this->pUV);
	}

	seoFrameWork::~seoFrameWork()
	{
		GLB_SAFE_DELETE( this->pUV );
	}


	void seoFrameWork::Draw()
	{
		this->pObject->Draw( this->pElem->ElemID(),
							 this->pElem->NowMatrix(),
							 this->pElem->State()->NowColor(),
							 this->pElem->State()->Scale()->Value(),
							 this->pUV->Value() );
	}

	void seoFrameWork::Update(float fAddTime){}


	seoFrame::FrameParts::FrameParts()
		: bVisible( true )
		, pSprite( null ) {}

	// f[^쐬
	void seoFrame::FrameParts::CreateSprite( int iObjectID )
	{
		this->Clear();
		siIOManager *pIOManager = seRootScene::Instance()->IOManager();
		this->pSprite = pIOManager->CreateTextureSprite(iObjectID);
	}

	void seoFrame::FrameParts::Clear()
	{
		GLB_SAFE_DELETE( this->pSprite );
	}

	void seoFrame::Size( const glbVector2 *value)
	{
		this->sSize = *value;
		this->bUpdateSprite = true;
	}

	void seoFrame::UV( const glbVector2 *value)
	{
		this->sUV = *value;
		this->bUpdateSprite = true;
	}

	void seoFrame::ExpendStart( const glbVector2 *value)
	{
		this->sExpansionStart = *value;
		this->bUpdateSprite = true;
	}

	void seoFrame::ExpendEnd( const glbVector2 *value)
	{
		this->sExpansionEnd = *value;
		this->bUpdateSprite = true;
	}

	seoFrame::seoFrame()
		: bUpdateSprite( false )
		, sSize( 10.0f, 10.0f )
		, sUV( 0.0f, 0.0f )
		, sExpansionStart( 0.0f, 0.0f )
		, sExpansionEnd( 0.0f, 0.0f )
		, iResourceID( seResourceManager::InvalidID )
		, pResourceTexture( null )
		, sColor( &glbColor::White )
	{
	}

	void seoFrame::Apply()
	{
		if (!this->bUpdateSprite)
			return;
	
		// L`ƃIuWFNg͕KĂ̂ƂB
		// (SɌĂȂ肢H)
	
		const glbVector2I *pTexSize = this->ResourceTexture()->TextureSize();
		glbVector2 sImageSize = glbVector2( static_cast<float>(pTexSize->X),
											static_cast<float>(pTexSize->Y) );
	
		glbVector2 es = glbVector2( glb::MaxValue(0.0f, this->sExpansionStart.X),
									glb::MaxValue(0.0f, this->sExpansionStart.Y));
		glbVector2 ee = glbVector2( glb::MinValue(this->Size()->X, this->sExpansionEnd.X),
									glb::MinValue(this->Size()->Y, this->sExpansionEnd.Y) );
	
	
		// UVXg
		glbVector2 asPartsUV[3];
		asPartsUV[0] = this->sUV;
		asPartsUV[1] = es;
		asPartsUV[1].Div(sImageSize);
		asPartsUV[1].Add(this->sUV);
		asPartsUV[2] = ee;
		asPartsUV[2].Div(sImageSize);
		asPartsUV[2].Add(this->sUV);
	
		// Xg
		glbVector2 asPartsSize[3];
		asPartsSize[0] = es;
		asPartsSize[1] = ee;
		asPartsSize[1].Sub(es);
		asPartsSize[2] = *this->Size();
		asPartsSize[2].Sub(ee);
	

		glbVector2 sUpdateUV;
		glbVector2 sUpdateSize;

		int iCount = 0;
		for (int y = 0; y < 3; y++)
		{
			for (int x = 0; x < 3; x++)
			{
				FrameParts &pParts = this->apImages[iCount];
				// ʐς0ȉ̏ꍇ͔\
				if ((asPartsSize[x].X * asPartsSize[y].Y) <= 0.0f)
				{
					pParts.Visible(false);
				}
				else
				{
					pParts.Visible(true);
					pParts.Sprite()->UpdateTexture(this->ResourceTexture());

					sUpdateSize.Set( asPartsSize[x].X, asPartsSize[y].Y );
					sUpdateUV.Set( asPartsUV[x].X, asPartsUV[y].Y );
					pParts.Sprite()->UpdateSize( &sUpdateSize );
					pParts.Sprite()->UpdateUV( &sUpdateUV );
				}
				iCount ++;
			}
		}
	
		this->bUpdateSprite = false;
	}


	void seoFrame::ResourceID(int value)
	{
		this->iResourceID = value;
		this->pResourceTexture = null;
		if (this->iResourceID == seResourceManager::InvalidID)
		{
			this->bUpdateSprite = true;
		}
	}

	siResourceTexture *seoFrame::ResourceTexture()
	{
		if ((this->pResourceTexture == null) &&
			(this->iResourceID != seResourceManager::InvalidID))
		{
			this->pResourceTexture = static_cast<siResourceTexture*>(
				this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));
	
			// ꍇ̓\[XԂɂ
			if (this->pResourceTexture != null)
			{
				// \[X͂邪łĂȂꍇ
				if (!this->pResourceTexture->IsEffective())
				{
					this->pResourceTexture = null;
				}
			}
			this->bUpdateSprite = true;
		}
	
		return this->pResourceTexture;
	}


	void seoFrame::Init(int iInObjectID, seSceneManager *pInScene)
	{
		seObject::Init(iInObjectID, pInScene);
	
		for (int i = 0; i < (int)GLB_ARRAY_COUNT( this->apImages ); i++)
		{
			this->apImages[i].CreateSprite( iInObjectID );
		}
		this->bUpdateSprite = true;
	}

	seObjectWork *seoFrame::CreateDrawWork(seElem *pElem)
	{
		return evc_new seoFrameWork(pElem, this);
	}

	void seoFrame::Draw( int iElemID, 
						 const IGLMatrix *mParent, 
						 const glbColor *sMulColor, 
						 const glbVector2 *sMstScale, 
						 const glbVector2 *sUV )
	{
		// `XV
		this->Apply();
	
		glbColor c = *sMulColor;
		c.Mul(this->Color());
			// eNX`t[obt@֕`
			// [JTCY
			// ʒu
			// XP[
					// \Ȃ玟
		seUseWorkMatrix gw;
		
		IGLMatrix *m = gw.GetMatrix();
		IGLMatrix *ms = gw.GetMatrix();
		IGLMatrix *mUV = gw.GetMatrix();
		mUV->SetUV( sUV );
		glbVector2 es( glb::MaxValue(0.0f, this->sExpansionStart.X),
					   glb::MaxValue(0.0f, this->sExpansionStart.Y) );
		glbVector2 ee( glb::MinValue(this->Size()->X, this->sExpansionEnd.X),
					   glb::MinValue(this->Size()->Y, this->sExpansionEnd.Y) );
		glbVector2 l0 = es;
		l0.Div(*sMstScale);
		glbVector2 l2 = *this->Size();
		l2.Sub(ee);
		l2.Div(*sMstScale);
		glbVector2 sLastPos = *this->Size();
		sLastPos.Sub(l2);
		glbVector2 sPutPos[3] = { glbVector2::Zero, l0, sLastPos };
		glbVector2 s = l0;
		s.Add(l2);
		glbVector2 pw1 = ee;
		pw1.Sub(es);
		glbVector2 s2 = *this->Size();
		s2.Sub(s);
		s2.Div(pw1);
		glbVector2 s3 = glbVector2::One;
		s3.Div(*sMstScale);
		glbVector2 sPutScale[3] = { s3, s2, s3 };
		int iCount = 0;
		glbVector2 sScale;
		glbVector2 sPos;
		for (int y = 0; y < 3; y++)
		{
			for (int x = 0; x < 3; x++)
			{
				FrameParts &pParts = this->apImages[iCount];
				iCount++;
				if (!pParts.Visible()) continue;
				sScale.X = sPutScale[x].X;
				sScale.Y = sPutScale[y].Y;
				ms->SetScale(&sScale);
				sPos.X = sPutPos[x].X;
				sPos.Y = sPutPos[y].Y;
				m->SetPos(&sPos);
				m->Mul(ms);
				m->MulR(mParent);
				pParts.Sprite()->Draw(iElemID, m, sMulColor, mUV);
			}
		}
	}

	seoFrame::~seoFrame()
	{
		for( int i = 0; i < (int)GLB_ARRAY_COUNT( this->apImages ); i++ )
		{
			this->apImages[ i ].Clear();
		}
	}

	void seoFrame::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->sSize = seIOUtil::ReadVector2(pInput);
		this->sUV = seIOUtil::ReadVector2(pInput);
		this->sExpansionStart = seIOUtil::ReadVector2(pInput);
		this->sExpansionEnd = seIOUtil::ReadVector2(pInput);
		this->ResourceID(static_cast<int>(pInput->ReadShort()));
		this->sColor = seIOUtil::ReadColor(pInput);
	}
}
