#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/common/seMath.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/seResource/seResourceScene.h"
#include "seoGrid.h"

using namespace glb;
namespace evc
{
	// ========================================================================
	// ObhIuWFNg[N

	const float seoGridWork::PowerScale = 3.0f;

	const glbVector2 *seoGridWork::GetSize()
	{
		return this->_obj->Size();
	}

	seObject *seoGridWork::Object()
	{
		return this->_obj;
	}



	seoGridWork::seoGridWork(seElem *elem, seoGrid *obj)
	{
		this->_elem = elem;
		this->_obj = obj;
		
		this->_mUV = seRootScene::Instance()->IOManager()->CreateMatrix();
		this->_mUV->Identity();

		this->_texture = static_cast<seStateSwitchInt1*>(seStateUtil::CreateStateType(emElemState::Texture));
		this->_textureControl = static_cast<seStateTextureControl*>(seStateUtil::CreateStateType(emElemState::TextureControl));
		this->_uv = static_cast<seStateFloat2*>(seStateUtil::CreateStateType(emElemState::UV));
		this->_gridPoint.push_back( static_cast<seStateGridPoint*>(seStateUtil::CreateStateType(emElemState::GridPos1)));
		this->_gridPoint.push_back( static_cast<seStateGridPoint*>(seStateUtil::CreateStateType(emElemState::GridPos2)));
		this->_gridPoint.push_back( static_cast<seStateGridPoint*>(seStateUtil::CreateStateType(emElemState::GridPos3)));
		this->_gridPoint.push_back( static_cast<seStateGridPoint*>(seStateUtil::CreateStateType(emElemState::GridPos4)));

		// Xe[^X̊gf[^NA
		seElemState *state = this->_elem->State();
		state->ClearStateData();
		state->LinkStateData(emElemState::Texture, this->_texture);
		state->LinkStateData(emElemState::TextureControl, this->_textureControl);
		state->LinkStateData(emElemState::UV, this->_uv);
		state->LinkStateData(emElemState::GridPos1, this->_gridPoint[0]);
		state->LinkStateData(emElemState::GridPos2, this->_gridPoint[1]);
		state->LinkStateData(emElemState::GridPos3, this->_gridPoint[2]);
		state->LinkStateData(emElemState::GridPos4, this->_gridPoint[3]);
	}

	seoGridWork::~seoGridWork()
	{
		GLB_SAFE_DELETE(this->_mUV);
		GLB_SAFE_DELETE(this->_texture);
		GLB_SAFE_DELETE(this->_textureControl);
		GLB_SAFE_DELETE(this->_uv);
		glb::DeleteVector(this->_gridPoint);
	}


	void seoGridWork::Reset()
	{
	}

	void seoGridWork::DrawSub(const IGLMatrix *mBase, const glbColor *color)
	{
		// _vZ
		siObjectMesh *mesh = this->_obj->Mesh();
		if (mesh != null)
		{
			mesh->BeginEditVertex(this->_elem->ElemID());

			est::vector<seStateGridPoint*> &mstPos = this->_gridPoint; //4
			int partX = this->_obj->Partitions()->X;
			int partY = this->_obj->Partitions()->Y;

			// _
			int index = 0;
			for (int y = 0; y <= partY; y++)
			{
				float fRate = static_cast<float>(y) / static_cast<float>(partY);

				// ẼC̍WZo
				glbVector2 posL = seMath::GetHermiteCurvePoint(
									mstPos[0]->Value(), 
									mstPos[0]->PowerV(), 
									mstPos[3]->Value(), 
									mstPos[3]->PowerV(), 
									fRate, 
									seoGridWork::PowerScale);

				glbVector2 posR = seMath::GetHermiteCurvePoint(
									mstPos[1]->Value(), 
									mstPos[1]->PowerV(), 
									mstPos[2]->Value(), 
									mstPos[2]->PowerV(), 
									fRate, 
									seoGridWork::PowerScale);


				glbVector2 power1 = *mstPos[3]->PowerH();
				const glbVector2 *pPow1End = mstPos[0]->PowerH();
				power1.Sub(*pPow1End);
				power1.Scale(fRate);
				power1.Add(*pPow1End);

				glbVector2 power2 = *mstPos[2]->PowerH();
				const glbVector2 *pPow2End = mstPos[1]->PowerH();
				power2.Sub(*pPow2End);
				power2.Scale(fRate);
				power2.Add(*pPow2End);

				for (int x = 0; x <= partX; x++)
				{
					glbVector2 v = seMath::GetHermiteCurvePoint(
							&posL, &power1, 
							&posR, &power2, 
							static_cast<float>(x) / static_cast<float>(partX),
							seoGridWork::PowerScale);
					mesh->SetPos(index, &v);
					index++;
				}
			}

			mesh->EndEditVertex();
		}

		this->_obj->Draw(this->_elem->ElemID(), mBase, color, this->_mUV);
	}

	void seoGridWork::Draw()
	{
		this->DrawSub(this->_elem->NowMatrix(),
					  this->_elem->State()->NowColor());
	}

	void seoGridWork::Update(float fAddTime)
	{
		seStateFloat2 *statUV = static_cast<seStateFloat2*>(this->_elem->State()->At(emElemState::UV));
		this->_mUV->SetUV(statUV->Value());
	}


	// ========================================================================
	// ObhIuWFNg

	emObject::Type seoGrid::ElemType() const
	{
		return emObject::Grid;
	}

	void seoGrid::ResourceID(int value)
	{
		this->iResourceID = value;
		this->_bUpdateImage = true;
	}

	siResourceTexture *seoGrid::ResourceTexture()
	{
		this->UpdateImage(seElemManager::InvalidID);
		return this->_pTexture;
	}

	bool seoGrid::UpdateImage(int elemID)
	{
		if (!this->_bUpdateImage)
			return false;

		// Ȃ
		if (this->iResourceID == seResourceManager::InvalidID)
		{
			this->_pTexture = null;
		}
		else
		{
			this->_pTexture = static_cast<siResourceTexture*>(this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));

			// \[X͂邪łĂȂꍇ
			if ((this->_pTexture != null) && !this->_pTexture->IsEffective())
			{
				// XVIȂ
				this->_pTexture = 0;
				return false;
			}
		}

		// XVꂽ
		if (this->_pMesh != null)
		{
			this->Mesh()->UpdateTexture(this->_pTexture);
			// C[WXV̂UVĐݒ
			this->ResetUV(elemID);
		}

		this->_bUpdateImage = false;
		return true;
	}

	seoGrid::seoGrid()
		: _pMesh(null)
		, _uv(0.0f, 0.0f)
		, _partitions(10, 10)
		, _color(255,255,255,255)
		, _size(1.0f, 1.0f)
		, _uvRate(0.0f, 0.0f)
		, iResourceID(seResourceManager::InvalidID)
		, _pTexture(null)
		, _bUpdateImage(false)
	{
	}

	seoGrid::~seoGrid()
	{
		GLB_SAFE_DELETE(this->_pMesh);
	}

	void seoGrid::ResetTriIndex()
	{
		int itriNum = (this->_partitions.X *
					   this->_partitions.Y * 2) * 3;

		// Op`index쐬
		if ((int)this->_triIndex.size() != itriNum)
		{
			this->_triIndex.resize(itriNum);
		}

		// PC̃ItZbg
		int offsetY = this->_partitions.X + 1;
		int index = 0;
		for (int y = 0; y < this->_partitions.Y; y++)
		{
			int iLineY = y * offsetY;
			for (int x = 0; x < this->_partitions.X; x++)
			{
				this->_triIndex[index] = static_cast<short>(iLineY + x);
				this->_triIndex[index + 1] = static_cast<short>(this->_triIndex[index] + 1);
				this->_triIndex[index + 2] = static_cast<short>(this->_triIndex[index] + offsetY);
				this->_triIndex[index + 3] = this->_triIndex[index + 2];
				this->_triIndex[index + 4] = this->_triIndex[index + 1];
				this->_triIndex[index + 5] = static_cast<short>(this->_triIndex[index + 2] + 1);
				index += 6;
			}
		}
	}

	void seoGrid::ResetUV(int elemID)
	{
		siObjectMesh *mv = this->_pMesh;
		mv->BeginEditVertex(elemID);

		int partX = this->_partitions.X;
		int partY = this->_partitions.Y;

		const glbVector2I *ti = this->_pTexture->TextureSize();
		glbVector2 texSize = glbVector2(static_cast<float>(ti->X),
										static_cast<float>(ti->Y));
		// _
		int index = 0;
		for (int y = 0; y <= partY; y++)
		{
			float fRate = static_cast<float>(y) / static_cast<float>(partY);

			// p␳FY
			float fY = this->_uvRate.Y + 1.0f;
			fRate *= (1.0f - fY) * fRate + fY;

			// ẼC̍WZo
			glbVector2 p1 = glbVector2(0.0f,0.0f);
			glbVector2 p2 = glbVector2(0.0f,this->Size()->Y);
			glbVector2 posL = seMath::GetHermiteCurvePoint(
							&p1, &glbVector2::Zero, 
							&p2, &glbVector2::Zero, 
							fRate,
							seoGridWork::PowerScale);

			p1 = glbVector2(this->Size()->X, 0.0f);
			glbVector2 posR = seMath::GetHermiteCurvePoint(
							&p1, &glbVector2::Zero,
							this->Size(), &glbVector2::Zero,
							fRate,
							seoGridWork::PowerScale);


			for (int x = 0; x <= partX; x++)
			{
				float fRateX = static_cast<float>(x) / static_cast<float>(partX);

				// p␳FX
				float fX = this->_uvRate.X + 1.0f;
				fRateX *= (1.0f - fX) * fRateX + fX;

				glbVector2 uv = seMath::GetHermiteCurvePoint(
								&posL, &glbVector2::Zero,
								&posR, &glbVector2::Zero, 
								fRateX, 
								seoGridWork::PowerScale);
				uv.Div(texSize);
				uv.Add(this->_uv);
				mv->SetUV(index, &uv);
				index++;
			}
		}

		mv->EndEditVertex();
	}

	void seoGrid::UpdateMesh()
	{
		GLB_SAFE_DELETE(this->_pMesh);

		// bV쐬
		this->_pMesh = seRootScene::Instance()->IOManager()->CreateMesh(
					this->ObjectID(), 
					MeshIndex::TriangleList,
					(this->_partitions.X + 1) * (this->_partitions.Y + 1),
					this->_partitions.X * this->_partitions.Y * 2 * 3);

		this->_pMesh->UpdateTexture(this->ResourceTexture());

		// Op`indexXg쐬
		this->ResetTriIndex();
		this->_pMesh->SetIndex(this->_triIndex);
	}

	void seoGrid::Init(int objID, seSceneManager *scene)
	{
		seObject::Init(objID, scene);

		// obt@č쐬
		this->UpdateMesh();
	}

	seObjectWork *seoGrid::CreateDrawWork(seElem *elem)
	{
		// UVXg쐬
		return evc_new seoGridWork(elem, this);
	}

	void seoGrid::Draw(int elemID, const IGLMatrix *m,
				const glbColor *color, const IGLMatrix *mUV)
	{
		this->UpdateImage(elemID);
		glbColor c = *color;
		c.Mul(&this->_color);
		this->_pMesh->Draw(elemID, m, &c, mUV);
	}

	void seoGrid::Load(IseStateInput *input, seSceneManager *manager)
	{
		seObject::Load(input, manager);

		this->ResourceID(static_cast<int>(input->ReadShort()));
		this->_uv = seIOUtil::ReadVector2(input);
		this->_partitions.X = input->ReadInt();
		this->_partitions.Y = input->ReadInt();
		this->_color = seIOUtil::ReadColor(input);
		this->_size = seIOUtil::ReadVector2(input); // TCY
		this->_uvRate = seIOUtil::ReadVector2(input);
	}
}
