#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "seoMesh.h"
#include "seoMeshChild.h"


using namespace glb;
namespace evc
{

	// ==============================================================
	// seoMeshPos
	// 

	seoMeshPos::seoMeshPos( const glbVector2 *p, 
							const glbVector2 *u, 
							const est::vector<unsigned char> *pweight)
		: pstvWeight( pweight )
		, sPos( p->X, p->Y )
		, sUV( u->X, u->Y )
	{
	}

	seoMeshPos *seoMeshPos::Clone()
	{
		return evc_new seoMeshPos(this->Pos(), this->UV(), this->pstvWeight);
	}


	// ==============================================================
	// seoMeshWork
	//

	void seoMeshWork::BoneList( const est::vector<seElem *> &value )
	{
		this->stvBoneList.clear();
		this->stvBoneList.assign(value.begin(), value.end());
	}


	seObject *seoMeshWork::Object()
	{
		return static_cast<seObject*>(this->pObject);
	}


	const glbVector2 *seoMeshWork::GetSize()
	{
		return this->pObject->Size();
	}

	seoMeshWork::seoMeshWork(seElem *elem, seoMesh *obj, siObjectMesh *vertex)
		: pObject( obj )
		, pElem( elem )
		, pVertex( vertex )
	{
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
	}

	void seoMeshWork::Reset()
	{
		// _XgH
		// 

	}

	void seoMeshWork::Draw()
	{
		// _vZ
		if ((this->stvBoneList.size() > 0) && (this->pVertex != null))
		{
			// 匳̋t}gNX|̂Ă
			seMatrixWork *gw = &seMatrixWork::Instance();
			int iMatrixWorkID = gw->StartMatrix();

			IGLMatrix *m = gw->GetMatrix();
			m->Copy(this->pElem->NowMatrix());
			m->Invert();

			// }gNXXg쐬
			int iBoneCount = this->stvBoneList.size();
			this->matWork.resize(iBoneCount);
			for (int i = 1; i < iBoneCount; i++)
			{
				IGLMatrix *mw = gw->GetMatrix();
				mw->Copy(m);
				mw->Mul(this->BoneList()[i]->NowMatrix());
				this->matWork[i] = mw;
			}

			// S_ɑ΂ďs
			const est::vector<seoMeshPos *> &mstPos = this->pObject->Vertex();
			int iVertextCount = (int)mstPos.size();
			this->Vertex()->BeginEditVertex(this->pElem->ElemID());
			for (int i = 0; i < iVertextCount; i++)
			{
				glbVector2 outPos = glbVector2::Zero;
				unsigned char bNonMoveWeight = 255;
				seoMeshPos *pos = mstPos[i];
				const est::vector<unsigned char> &weight = pos->Weight();

				for (int j = 1; j < (int)(weight.size()); j++)
				{
					unsigned char bWeight = weight[j];
					if (bWeight == 0) continue;
					if (bWeight > bNonMoveWeight)
					{
						bWeight = bNonMoveWeight;
					}
					glbVector2 p = this->matWork[j]->TransformCoordinate(pos->Pos());
					p.Scale(static_cast<float>(bWeight) / 255.0f);
					outPos.Add( p );
					bNonMoveWeight -= bWeight;
					if (bNonMoveWeight == 0) break;
				}
				if (bNonMoveWeight > 0)
				{
					glbVector2 nonMove = *pos->Pos();
					nonMove.Scale(static_cast<float>(bNonMoveWeight) / 255.0f);
					outPos.Add(nonMove);
				}
				this->Vertex()->SetPos(i, &outPos);
			}
			this->Vertex()->EndEditVertex();
			gw->EndMatrix( iMatrixWorkID );

		}

		this->pObject->Draw( this->pElem->ElemID(), 
							 this->pElem->NowMatrix(), 
							 this->pElem->State()->NowColor());
	}

	void seoMeshWork::Update(float fAddTime)
	{
	}

	// ==============================================================
	// {[\o^[N
	// 

	class BoneWork
	{
	private:
		int iObjID;
		est::vector<BoneWork*> stvChild;

	public:
		int ObjID() const 		{ return this->iObjID; }
		void ObjID(int value)	{ this->iObjID = value; }

		est::vector<BoneWork*> &Child() { return this->stvChild; }

		BoneWork(){}
		virtual ~BoneWork()
		{
			// ȉ̃[NSč폜
			glb::DeleteVector( this->stvChild );
		}
	};


	// ==============================================================
	// seoMesh
	// 

	const glbVector2 *seoMesh::Size()
	{
		if (this->ResourceTexture() == null)
		{
			this->sSize = glbVector2::Zero;
		}
		else
		{
			const glbVector2I *pv = this->ResourceTexture()->TextureSize();
			this->sSize.X = (float)pv->X;
			this->sSize.Y = (float)pv->Y;
		}

		return &this->sSize;
	}


	void seoMesh::ResourceID(int value)
	{
		this->iResourceID = value;
		this->bUpdateImage = true;
	}

	bool seoMesh::UpdateImage()
	{
		if (!this->bUpdateImage)
			return false;

		// Ȃ
		if (this->iResourceID == seResourceManager::InvalidID)
		{
			this->pTexture = 0;
		}
		else
		{
			this->pTexture = static_cast<siResourceTexture*>(
				this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));

			// \[X͂邪łĂȂꍇ
			if ((this->pTexture != null) &&
				!this->pTexture->IsEffective())
			{
				// XVIȂ
				this->pTexture = null;
				return false;
			}
		}

		if ( this->mesh != null )
		{
			this->mesh->UpdateTexture(this->pTexture);
		}
		this->bUpdateImage = false;
		return true;
	}

	siResourceTexture *seoMesh::ResourceTexture()
	{
		this->UpdateImage();
		return this->pTexture;
	}

	// RXgN^
	seoMesh::seoMesh()
		: mesh( null )
		, pRootBone( null )
		, iResourceID( seResourceManager::InvalidID )
		, bUpdateImage( false )
		, pTexture( null )
		, sColor( glbColor::White )
		, sSize( glbVector2::Zero )	// TCY
	{
	}

	// fXgN^
	seoMesh::~seoMesh()
	{
		glb::DeleteVector( this->stvVertex );
		GLB_SAFE_DELETE( this->mesh );
	}


	void seoMesh::Init(int objID, seSceneManager *scene)
	{
		seObject::Init(objID, scene);
		// VID蓖Ăꂽ̂ōȂ
		if (this->mesh != null)
		{
			GLB_SAFE_DELETE( this->mesh );
		}

		siIOManager *iom = seRootScene::Instance()->IOManager();
		if ((this->Vertex().size() > 0) &&
			(this->Triangle().size() > 0))
		{
			this->mesh = iom->CreateMesh( this->ObjectID(), 
										  MeshIndex::TriangleList,
										  this->Vertex().size(),
										  this->Triangle().size());

			this->mesh->UpdateTexture(this->ResourceTexture());
			this->mesh->SetIndex(this->Triangle());
		}
	}


	seObjectWork *seoMesh::CreateDrawWork(seElem *pElem)
	{
		// [N쐬ɂPxUVݒ
		// GgƖɃobt@ꍇz肵܂B
		siObjectMesh *om = this->mesh;
		om->BeginEditVertex(pElem->ElemID());
		for (int i = 0; i < (int)this->Vertex().size(); i++)
		{
			om->SetUV(i, this->Vertex()[i]->UV());
		}
		om->EndEditVertex();
		return evc_new seoMeshWork(pElem, this, om);
	}


	void seoMesh::Draw( int elemID, 
						const IGLMatrix *m, 
						const glbColor *color)
	{
		this->UpdateImage();
		glbColor c = *color;
		c.Mul( this->Color() );
		this->mesh->Draw(elemID, m, &c, seMatrixWork::Instance().Identity());
	}


	// _[h
	void seoMesh::LoadBone(IseStateInput *input, BoneWork *parent)
	{
		BoneWork *bw = evc_new BoneWork();
		bw->ObjID( input->ReadInt() );
		short childCount = input->ReadShort();
		if (parent != null)
		{
			parent->Child().push_back(bw);
		}
		else
		{
			this->LoadBoneWork = bw;
		}

		for (int i = 0; i < childCount; i++)
		{
			this->LoadBone(input, bw);
		}
	}



	void seoMesh::Load(IseStateInput *input, seSceneManager *inManager)
	{
		seObject::Load(input, inManager);
		this->ResourceID(static_cast<int>(input->ReadShort()));
		this->sColor = seIOUtil::ReadColor(input);

		// Kw\[h
		this->LoadBone( input, null );


		// d݃Xg[h
		this->stvWeight.clear();
		int iCount = static_cast<int>(input->ReadShort());
		if (iCount > 0)
		{
			this->stvWeight.resize( iCount );
			int weightCount = static_cast<int>(input->ReadShort());
			for (int i = 0; i < iCount; i++)
			{
				est::vector<unsigned char> &v = this->stvWeight[i];
				for (int j = 0; j < weightCount; j++)
				{
					v.push_back(input->ReadByte());
				}
			}
		}

		// _f[^
		std::vector<seoMeshPos*> &vertex = this->stvVertex;
		vertex.clear();
		int ivCount = static_cast<int>(input->ReadShort());
		if (ivCount > 0)
		{
			input->ReadShort();
			input->ReadShort();
			short sx = input->ReadShort();
			short sy = input->ReadShort();
			glbVector2 uvRate = glbVector2( static_cast<float>(sx),
											static_cast<float>(sy));
			uvRate.Sub(glbVector2::One);

			// f[^TCY擾
			unsigned char xsize = input->ReadByte();
			unsigned char ysize = input->ReadByte();
			unsigned char wsize = input->ReadByte();

			for (int i = 0; i < ivCount; i++)
			{
				int iX = this->ReadChangeability(input,xsize);
				int iY = this->ReadChangeability(input,ysize);
				int wID = this->ReadChangeability(input,wsize);

				glbVector2 vpos = glbVector2( static_cast<float>(iX),
											  static_cast<float>(iY));
				glbVector2 uv = vpos;
				uv.Div(uvRate);

				seoMeshPos *pos = evc_new seoMeshPos(&vpos, &uv, &(this->stvWeight[wID]));
				vertex.push_back(pos);
			}
		}

		// indexXg
		this->stvTriangle.clear();
		int idCount = static_cast<int>(input->ReadShort());
		if (idCount > 0)
		{
			unsigned char isize = input->ReadByte();
			MeshIndex::Type indexType = input->ReadEnum<MeshIndex,MeshIndex::Type>();

			this->stvTriangle.resize( idCount );
			switch (indexType)
			{
				case MeshIndex::TriangleList:
					for (int i = 0; i < idCount; i++)
					{
						this->stvTriangle[i] = static_cast<short>(
								this->ReadChangeability(input, isize));
					}
					break;
				default:
					GLB_ASSERT(false);
					break;
			}
		}
	}

	int seoMesh::ReadChangeability(IseStateInput *input, unsigned char size)
	{
		switch (size)
		{
			case 1: return static_cast<int>(input->ReadByte());
			case 2: return static_cast<int>(input->ReadShort());
			default: GLB_ASSERT(false); break;
		}
		return 0;
	}

	// {[NTu[`
	void seoMesh::LinkMechChildSub( seObjectManager *objManager,
									seoMeshChild *parent, 
									BoneWork *bwork)
	{
		seoMeshChild *meshChild = static_cast<seoMeshChild*>(objManager->Find(bwork->ObjID()));
		meshChild->Mesh( this );
		if (parent != null)
		{
			parent->Link(meshChild);
		}
		else
		{
			this->pRootBone = meshChild;
		}

		est::vector<BoneWork*> &bone = bwork->Child();
		est::vector<BoneWork*>::iterator it = bone.begin();
		while( it != bone.end() )
		{
			this->LinkMechChildSub( objManager, meshChild, *it );
			it ++;
		}
	}


	void seoMesh::LinkMechChild( seObjectManager *pManager)
	{
		if (this->LoadBoneWork != null)
		{
			this->LinkMechChildSub(pManager, 0, this->LoadBoneWork);
		}
		GLB_SAFE_DELETE( this->LoadBoneWork );
	}



	void seoMesh::RegisterMesh()
	{
		if (this->mesh != 0)
		{
			delete this->mesh;
		}
		this->mesh = seRootScene::Instance()->IOManager()->CreateMesh(
						this->ObjectID(),
						MeshIndex::TriangleList,
						(int)this->Vertex().size(),
						(int)this->Triangle().size());

		// ̏Kv
		// _f[^͕`Oɐݒ肳Ă
		// 

		this->mesh->UpdateTexture(this->ResourceTexture());
		this->mesh->SetIndex(this->Triangle());

		siObjectMesh *om = this->mesh;
		om->BeginEditVertex(seElemManager::InvalidID);
		for (int i = 0; i < (int)this->Vertex().size(); i++)
		{
			seoMeshPos *v = this->Vertex()[i];
			om->SetPos(i, v->Pos());
			om->SetUV(i, v->UV());
		}
		om->EndEditVertex();
	}

	// seElemseoMeshWork擾B
	// 擾łȂꍇnullԂ܂B
	static seoMeshChild *GetMeshChild( seElem *pElem )
	{
		seObjectWork *ow = pElem->ObjectWork();
		if (ow == null) return null;
		if (ow->Object()->ElemType() != emObject::MeshChild) return null;

		return static_cast<seoMeshChild*>(ow->Object());
	}

	


	void seoMesh::UpdateMeshElem(seElem *elem)
	{
		seObjectWork *pObjWork = elem->ObjectWork();
		if ( pObjWork == null ) return;
		if ( pObjWork->Object()->ElemType() != emObject::Mesh ) return;
		seoMeshWork *meshWork = static_cast<seoMeshWork*>(pObjWork);

		GLB_ASSERT(static_cast<seoMesh*>(meshWork->Object()) == this);

		// bVXV
		meshWork->Vertex( this->mesh );


		// bV艺̘AMeshChildXgB
		struct func : public deAction1<seElem *>
		{
			int iMaxLabel;
			seoMesh *mesh;
			std::vector<seElem*> stvChild;

			virtual ~func(){}
			void operator()( seElem *pElem )
			{
				seoMeshChild *meshChild = GetMeshChild( pElem );
				if ( meshChild == null ) return;

				// eႤmeshChild̏ꍇ͖
				if (meshChild->Mesh() != mesh) return;

				this->stvChild.push_back(pElem);
				this->iMaxLabel = glb::MaxValue( this->iMaxLabel, meshChild->Label());

				const est::vector<seElem*> &child = *pElem->Child();
				est::vector<seElem*>::const_iterator it = child.begin();
				// qɑ΂ďs
				while(it != child.end())
				{
					(*this)(*it);
					it ++;
				}
			}
		};

		func f;
		f.iMaxLabel = 0;
		f.mesh = this;

		const est::vector<seElem*> &child = *elem->Child();
		est::vector<seElem*>::const_iterator eit = child.begin();
		// qɑ΂ďs
		while(eit != child.end())
		{
			f(*eit);
			eit++;
		}

		meshWork->stvBoneList.clear();
		meshWork->stvBoneList.resize( f.iMaxLabel + 1 );

		std::vector<seElem*>::iterator it = f.stvChild.begin();
		while( it != f.stvChild.end() )
		{
			seoMeshChild *meshChild = GetMeshChild( *it );
			meshWork->stvBoneList[ meshChild->Label() ] = *it;
			it ++;
		}
	}

	void seoMesh::UpdateMeshElem()
	{
		// {[\ēo^
		struct sub : public deFunc1<seElem *,bool>
		{
		public:
			seoMesh *mesh;
			bool operator() ( seElem *elem )
			{
				mesh->UpdateMeshElem(elem);
				return true;
			}
		} f;
		f.mesh = this;
		this->DoUseElem(&f);
	}


	void seoMesh::DoUseElem(deFunc1<seElem*, bool> *func)
	{
		// SẴGgɑ΂ďs
		// bVɑ΂Ă̂ݎs
		// ҏWbVIuWFNgGgɂ̂ݎs
		// bVGgɑ΂ďs

		struct sub : public deFunc1<seElem *,bool>
		{
			deFunc1<seElem*, bool> *pfunc;
			seoMesh *mesh;

			bool operator() ( seElem *elem )
			{
				seObjectWork *pWork = elem->ObjectWork();
				if ( pWork == null ) return true;
				if ( pWork->Object() != this->mesh ) return true;
				return (*pfunc)(elem);
			}
		} f;
		f.pfunc = func;
		f.mesh = this;

		this->Scene()->ElemManager()->DoAll(&f);
	}
}

