#ifndef INCLUDE_SEOMESH_H
#define INCLUDE_SEOMESH_H
#pragma once

#include "glb/glbDefine.h"
#include "glb/glbVector.h"
#include "glb/glbColor.h"
#include "evc/common/seDefine.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/seObjectWork.h"

using namespace glb;

namespace glb
{
	class IGLMatrix;
}


namespace evc
{
	class siObjectMesh;
	class siObjectTextureSprite;
	class seoMeshChild;
	class seoMesh;
	class siResourceTexture;
	class seSceneManager;
	class seElem;
	class IseStateInput;
	class seObjectManager;
	class seoMesh;
	class siObjectMeshVertex;
	class BoneWork;



	class seoMeshPos
	{
	private:
		const est::vector<unsigned char> *pstvWeight;
		glbVector2 sPos;
		glbVector2 sUV;

	public:
		const est::vector<unsigned char> &Weight() const	{ return *this->pstvWeight; }

		const glbVector2 *Pos() const		{ return &this->sPos; }
		void Pos( const glbVector2 *value)	{ this->sPos = *value; }

		const glbVector2 *UV() const		{ return &this->sUV; }
		void UV( const glbVector2 *value )	{ this->sUV = *value; }

		seoMeshPos( const glbVector2 *p, 
					const glbVector2 *u,
					const est::vector<unsigned char> *pweight );

		// N[쐬
		seoMeshPos *Clone();
	};



	class seoMeshWork : public seObjectWork
	{
		friend class seoMesh;
	private:

		seoMesh *pObject;	// `IuWFNg
		seElem *pElem;		// Gg

		// _Xg(vZp)
		siObjectMesh *pVertex;
		est::vector<seElem*> stvBoneList;
		est::vector<IGLMatrix*> matWork;
	
	public:
		siObjectMesh *Vertex() const		{ return this->pVertex; }
		void Vertex(siObjectMesh *value)	{ this->pVertex = value; }

		// {[
		const est::vector<seElem*> &BoneList() const { return this->stvBoneList; }
		void BoneList(const est::vector<seElem*> &value );

		virtual seObject *Object();


		// IuWFNg̃TCY擾
		virtual const glbVector2 *GetSize();

		// RXgN^
		seoMeshWork(seElem *elem, seoMesh *obj, siObjectMesh *mesh);

		// [ÑZbg
		virtual void Reset();

		// `
		virtual void Draw();

		// XV
		virtual void Update(float fAddTime);
	};




	class seoMesh : public seObject
	{
	private:
		siObjectMesh *mesh;
		seoMeshChild *pRootBone;
		est::vector<seoMeshPos *> stvVertex;
		est::vector<short> stvTriangle;
		est::vector< est::vector<unsigned char> > stvWeight;
		int iResourceID;
		bool bUpdateImage;
		siResourceTexture *pTexture;

		glbColor sColor;
		glbVector2 sSize;	// TCY

	public:
		// {[
		seoMeshChild *RootBone() const		{ return this->pRootBone; }

		// _Xg
		const est::vector<seoMeshPos *> &Vertex() const	{ return this->stvVertex; }

		// Op`A
		const est::vector<short> &Triangle() const	{ return this->stvTriangle; }

		// d݃Xg
		const est::vector<est::vector<unsigned char> > &Weight() const	{ return this->stvWeight; }

		// TCY
		const glbVector2 *Size();

		// J[
		const glbColor *Color() const		{ return &this->sColor; }

		// \[XID
		int ResourceID() const		{ return this->iResourceID; }
		void ResourceID(int value);


		// eNX`\[X擾
		siResourceTexture *ResourceTexture();


		// RXgN^
		seoMesh();
		virtual ~seoMesh();

		// 
		virtual void Init(int objID, seSceneManager *scene);

		// `IuWFNg^Cv
		virtual emObject::Type ElemType() const { return emObject::Mesh; }


		// `掞ɕKvȃ[NNX쐬
		virtual seObjectWork *CreateDrawWork(seElem *inElem);


		void Draw(int elemID, const IGLMatrix *m, const glbColor *color);


		virtual void Load( IseStateInput *input, seSceneManager *inManager);

		// Kwꎞf[^f[^ɕϊ
		void LinkMechChild( seObjectManager *objManager);


		// bVɃf[^o^
		void RegisterMesh();

		// bV̏gpGgXV
		void UpdateMeshElem(seElem *elem);
	
		// bV̏gpGgSĂXV
		void UpdateMeshElem();
	
		// bVgGgɑ΂ďs
		void DoUseElem(deFunc1<seElem*, bool> *func);

	private:
		// σf[^擾
		int ReadChangeability(IseStateInput *input, unsigned char size);
		// ResourceIDɂ킹ăeNX`̍XV
		bool UpdateImage();

		BoneWork *LoadBoneWork;
		// _[h
		void LoadBone(IseStateInput *input, BoneWork *parent);

		// {[NTu[`
		void LinkMechChildSub( seObjectManager *objManager,
								seoMeshChild *parent, 
								BoneWork *bwork);
	};
}
#endif // INCLUDE_SEOMESH_H

