#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "seoNum.h"

using namespace glb;
namespace evc
{

	seObject *seoNumWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}

	seoNumWork::seoNumWork(seElem *pInElem, seoNum *pInObjct)
	{
		this->pElem = pInElem;
		this->pObject = pInObjct;
	
		this->pNum = static_cast<seStateInt1*>(seStateUtil::CreateStateType(emElemState::Num));
		this->pNumInfo = static_cast<seStateNumInfo*>(seStateUtil::CreateStateType(emElemState::NumState));
	
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
		this->pElem->State()->LinkStateData(emElemState::Num, this->pNum);
		this->pElem->State()->LinkStateData(emElemState::NumState, this->pNumInfo);
	}

	seoNumWork::~seoNumWork()
	{
		GLB_SAFE_DELETE( this->pNum )
		GLB_SAFE_DELETE( this->pNumInfo )
	}


	void seoNumWork::Draw()
	{
		this->pObject->Draw( this->pElem->ElemID(),
							 this->pElem->NowMatrix(),
							 this->pElem->State()->NowColor(),
							 this->pNum->Value(), 
							 this->pNumInfo );
	}

	void seoNumWork::Update(float fAddTime){}

	const glbVector2 *seoNumWork::GetSize()
	{
		float fNumSize = this->pObject->GetNumWidth(this->pNum->Value(), this->pNumInfo);
	
		// ʏ
		switch (this->pNumInfo->Arrange())
		{
			case seNumArrange::L:
			case seNumArrange::Point:
				break;
	
			case seNumArrange::R:
			default:
				// E낦̏ꍇ͕ЂԂB
				fNumSize = -fNumSize;
				break;
		}
	
		// _ɂׁAx͂ЂԂB
		this->sSize.X = fNumSize;
		this->sSize.Y = -this->pObject->Height();

		return &this->sSize;
	}


	// seoNum
	// 

	// RXgN^
	seoNum::seoNum()
		: fHeight( 0.0f )
		, sColor( glbColor::White )
	{
		this->apImageList.resize( seoNumImage::EnumCount() + 1 );
		VECTOR_FOR_EACH( seoNumTexture, this->apImageList, 
		{
			(*it).Parent( this );
		});
	}

	// seoNumTexture:RXgN^
	seoNum::seoNumTexture::seoNumTexture()
		: pParent( null )
		, pImage( null )
		, iID( seObjectManager::InvalidID ){}


	seoTextureSprite *seoNum::seoNumTexture::Image()
	{
		if ((this->pImage == null) && (this->iID != seObjectManager::InvalidID))
		{
			this->pImage = static_cast<seoTextureSprite*>(
					pParent->Scene()->ObjectManager()->seIDManager<seObject*>::Find(this->iID));
		}
		return this->pImage;
	}

	void seoNum::seoNumTexture::Image(seoTextureSprite *value)
	{
		this->pImage = value;
		this->iID = (this->pImage == null) ? seObjectManager::InvalidID
										   : this->pImage->ObjectID();
	}


	void seoNum::seoNumTexture::ID(int value)
	{
		this->iID = value;
		this->pImage = null;
	}

	void seoNum::SetImage(seoNumImage::Type eImageType, seoTextureSprite *pObject)
	{
		this->apImageList[static_cast<int>(eImageType)].Image(pObject);
	
		// ԑ傫ȍ擾
		this->fHeight = 0.0f;
		for (int i = 0; i < GLB_ARRAY_COUNT( this->apImageList ); i++)
		{
//			seoNumImage::Type eNumImage = static_cast<seoNumImage::Type>(i);
			seoTextureSprite *pCheckObje = this->apImageList[i].Image();

			if (pCheckObje == null) continue;
			this->fHeight = glb::MaxValue( this->fHeight, static_cast<float>(pCheckObje->Size()->Y));
		}

	}

	seoTextureSprite *seoNum::GetImage(seoNumImage::Type eImageType)
	{
		return this->apImageList[static_cast<int>(eImageType)].Image();
	}


	seObjectWork *seoNum::CreateDrawWork(seElem *pElem)
	{
		return evc_new seoNumWork(pElem, this);
	}

	void seoNum::Draw( int iElemID, 
					   const IGLMatrix *mParent, 
					   const glbColor *sMulColor, 
					   int iNum, 
					   seStateNumInfo *pNumInfo )
	{
		// J[̎Zo
		glbColor c = *sMulColor;
		c.Mul( this->Color() );
	
		this->vecImageWork.clear();
		this->vecImageWidth.clear();
		struct func : public deAction1<seoNumImage::Type>
		{
			seoNum *pme;
			bool bFirst;
			float fWidth;
			int iFixationWidth;
			int iCommaWidth;
			bool bFixCharLenge;
			est::vector<seoTextureSprite *> *vecTex;
			est::vector<float> *vecWidth;

			func( seoNum *me,
				  seStateNumInfo *pNum,
				  est::vector<seoTextureSprite *>* texList,
				  est::vector<float> *widthList )
				: pme( me )
				, bFirst(true)
				, fWidth(0.0f)
				, iFixationWidth( pNum->NumLen() )
				, iCommaWidth( pNum->CommaLen() )
				, bFixCharLenge(pNum->CharLen() == seNumCharLen::Fix)
				, vecTex( texList )
				, vecWidth( widthList ){}

			void operator()( seoNumImage::Type eType )
			{
				seoTextureSprite *pTex = pme->GetImage(eType);
				if (pTex == null) return;

				float fNowW = 0.0f;
				// Œ蕶łP̈ʈȏȂ
				if ( bFixCharLenge && !bFirst )
				{
					fNowW = (eType == seoNumImage::Comma) ? (float)iCommaWidth
															: (float)iFixationWidth;
				}
				else
				{
					fNowW = (float)pTex->Size()->X;
				}
				bFirst = false;

				this->fWidth += fNowW;

				vecTex->push_back( pTex );
				vecWidth->push_back( fNowW );
			}
		} f( this, pNumInfo, &this->vecImageWork, &this->vecImageWidth );
		this->SetNumInfo( iNum, pNumInfo, &f );


		// 擾
		float fXOffset = 0.0f;
		switch (pNumInfo->Arrange())
		{
			// 
			case seNumArrange::L:
				fXOffset = 0.0f;
				break;
			// |Cg
			case seNumArrange::Point:
				break;
			// ȊO͉E
			case seNumArrange::R:
			default:
				fXOffset = -f.fWidth;
				break;
		}
	
		seUseWorkMatrix gw;
	
		IGLMatrix *m = gw.GetMatrix();
	
		est::vector<float>::reverse_iterator WidthIt = f.vecWidth->rbegin();
		est::vector<seoTextureSprite *>::reverse_iterator it = f.vecTex->rbegin();
		for(; it != f.vecTex->rend(); it++ )
		{
			m->SetPos( fXOffset, -(*it)->Size()->Y );
			m->MulR( mParent );
			(*it)->Draw(iElemID, m, &c);
			fXOffset += (*WidthIt);
			WidthIt++;
		}
	}

	// \鐔lC[WP̈ʂ珇ɈɂďĂт܂B
	void seoNum::SetNumInfo(int iNum, seStateNumInfo *pNumInfo, deAction1<seoNumImage::Type> *pFunc )
	{
		// l}CiX̏ꍇ
		bool bMinus = false;
		if (iNum < 0)
		{
			iNum = -iNum;
			// }CiX\Ȃ
			bMinus = (pNumInfo->VisibleMinus() == seNumVisible::On) ?
														true : false;
		}

		// \E(ŏ͂P)
		int iMaxDisit = glb::MaxValue(pNumInfo->MaxDisit(), 1);
		for( int i = 0; i < iMaxDisit; i++ )
		{
			// J}}
			if (pNumInfo->VisibleComma() == seNumVisible::On)
			{
				if ( (i>0) && ((i%3)==0) )
				{
					(*pFunc)(seoNumImage::Comma);
				}
			}

			// lݒ
			(*pFunc)(static_cast<seoNumImage::Type>(seoNumImage::NUM_0 + (iNum % 10)));

			iNum /= 10;
			// O߂łȂāAlꍇ͏I
			if ( (iNum <= 0) && (pNumInfo->VisibleZeroBuries()!= seNumVisible::On) )
				break;
		}

		// }CiXȂ
		if (bMinus)
		{
			(*pFunc)(seoNumImage::Minus);
		}
	}


	float seoNum::GetNumWidth( int iNum, seStateNumInfo *pNumInfo )
	{
		struct func : public deAction1<seoNumImage::Type>
		{
			seoNum *pme;
			bool bFirst;
			float fWidth;
			int iFixationWidth;
			int iCommaWidth;
			bool bFixCharLenge;

			func( seoNum *me, seStateNumInfo *pNum )
				: pme( me )
				, bFirst(true)
				, fWidth(0.0f)
				, iFixationWidth( pNum->NumLen() )
				, iCommaWidth( pNum->CommaLen() )
				, bFixCharLenge(pNum->CharLen() == seNumCharLen::Fix)
			{}

			void operator()( seoNumImage::Type eType )
			{
				seoTextureSprite *pTex = pme->GetImage(eType);
				if (pTex == null) return;

				// Œ蕶łP̈ʈȏȂ
				if ( bFixCharLenge && !bFirst )
				{
					fWidth += (eType == seoNumImage::Comma) ? (float)iCommaWidth
															: (float)iFixationWidth;
				}
				else
				{
					fWidth += (float)pTex->Size()->X;
				}
				bFirst = false;
			}
		} f( this, pNumInfo );
		this->SetNumInfo( iNum, pNumInfo, &f );
		return f.fWidth;
	}

	seoNum::~seoNum()
	{
	}

	void seoNum::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->sColor = seIOUtil::ReadColor(pInput);
		this->fHeight = pInput->ReadFloat();
	
		// C[WXg
		for (int i = 0; i <= seoNumImage::EnumCount(); i++)
		{
			this->apImageList[i].ID(static_cast<int>(pInput->ReadShort()));
		}
	}
}
