#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/common/seMath.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/seResource/seResourceScene.h"
#include "seoParticles.h"

using namespace glb;

namespace evc
{
	class ParticleOne
	{
	private:
		ParticleOne *_pBefor;
		ParticleOne *_pNext;

		glbColor _color;
		IGLMatrix *_pLocalMatrix;

		float _lifeTime; // 
		float _elapsedTime; // ܂Ăo߂
		seoParticlesWork *_pObjWork; // 
		float _baseRot; // ␳]px
		glbVector2 _baseScale; // ␳XP[
		glbColor _baseColor; // x[XJ[
		bool _bRotDirection; // ړpxZo郂[hۂ

		glbVector2 _power; // ړ
		glbVector2 _power2; // e
		glbVector2 _startPos; // Jn̈ʒu

	public:
		ParticleOne *Befor()			{return this->_pBefor;}
		void Befor(ParticleOne *value)	{this->_pBefor = value;}

		ParticleOne *Next()				{return this->_pNext;}
		void Next(ParticleOne *value)	{this->_pNext = value;}

		const glbColor *Color() const		{return &this->_color;}
		void Color(const glbColor *value)	{this->_color = *value;} 

		const IGLMatrix *LocalMatrix() const	{return this->_pLocalMatrix;}

		// RXgN^
		ParticleOne(seoParticlesWork *pObjWork)
		{
			this->_pObjWork = pObjWork;
			this->_pLocalMatrix = seRootScene::Instance()->IOManager()->CreateMatrix();
		}

		// fXgN^
		virtual ~ParticleOne()
		{
			GLB_SAFE_DELETE( this->_pLocalMatrix);
		}

		// xNgƊgḱ͈Apx͈͂
		// ^̃_ʒuZo
		glbVector2 createPower( const glbVector2 *pV,
								  float rotLen, 
								  float fAddRot, 
								  float scaleLen)
		{
			glbVector2 v = *pV;
			// ړp[
			v.Scale(((scaleLen - 1.0f) * GLB_RANDF()) + 1.0f);
			
			// ͈(_px{o͂̉]p)
			float dRot = (rotLen * 2.0f * GLB_RANDF()) - rotLen;
			dRot = (dRot * glb::PI / 180.0f) + fAddRot;
			float sinR = GLB_SIN(dRot);
			float cosR = GLB_COS(dRot);
			return glbVector2(v.X * cosR - v.Y * sinR,
							  v.X * sinR + v.Y * cosR);
		}


		// Pp[eBNJn
		bool Start(float fTime, const glbVector2 *pOffset)
		{
			const seStateParticlesInfo *partInfo = this->_pObjWork->_particlesObj->Info();

			// Zo
			int iMin = partInfo->LifeMin();
			int iMax = partInfo->LifeMax();
			this->_lifeTime = static_cast<float>((iMax >= iMin)
								? glb::RandLimit(iMin, iMax)
								: glb::RandLimit(iMax, iMin))
				* seRootScene::Instance()->RootScene()->Option()->FrameTime();

			if (fTime >= this->_lifeTime) return false;


			this->_baseRot = (GLB_RANDF() * partInfo->RotLenge() * 2.0f) - partInfo->RotLenge();
			float fscale = (GLB_RANDF() * (partInfo->ScaleLenge() - 1.0f)) + 1.0f;
			this->_baseScale.Set(fscale, fscale);

			seElem *elem = this->_pObjWork->_elem;


			// ϊ̍W擾
			const IGLMatrix *mElem = elem->NowMatrix();

			glbVector2 wZero = glbVector2::Zero;
			wZero = mElem->TransformCoordinate(&wZero);
			glbVector2 wWidth = glbVector2(1.0f, 0.0f);
			wWidth = mElem->TransformCoordinate(&wWidth);
			glbVector2 wHeight = glbVector2(0.0f, 1.0f);
			wHeight = mElem->TransformCoordinate(&wHeight);
			float fElemRot = seMath::GetRot(&wZero, &wWidth);

			const glbVector2 *objSize = elem->ObjectWork()->GetSize();
			this->_baseColor = *elem->State()->NowColor();


			// XP[pȂ
			if (this->_pObjWork->_successionScale)
			{
				this->_baseScale.X *= seMath::GetDistance(&wZero, &wWidth);
				this->_baseScale.Y *= seMath::GetDistance(&wZero, &wHeight);
			}

			// ][h
			this->_bRotDirection = false;
			switch (this->_pObjWork->_successionRot)
			{
				case ParticlesRot::Succession:
					this->_baseRot += fElemRot;
					break;
				// ړpxZo
				case ParticlesRot::Direction:
					this->_bRotDirection = true;
					break;
				default:
					break;
			}


			// ANeBuȗvfɃTCY|}gNXZo
			seMatrixWork &mw = seMatrixWork::Instance();
			int imatID = mw.StartMatrix();

			IGLMatrix *mSize = mw.GetMatrix();
			IGLMatrix *mOffset = mw.GetMatrix();
			IGLMatrix *m = mw.GetMatrix();
			elem->GetRectMatrix(m);
			mSize->SetScale(objSize);

			// p[eBNXe[^XASʒu擾
			glbVector2 center = this->GetCenter(partInfo, 0.0f);
			center.Scale(-1.0f);
			mOffset->SetPos(&center);
			m->MulR(mOffset);
			m->Mul(mSize);

			// o͈ʒuAˏ
			glbVector2 startPos;
			switch (this->_pObjWork->_outType)
			{
				// 
				case ParticlesOutType::Plane:
					startPos.X = GLB_RANDF();
					startPos.Y = GLB_RANDF();
					break;
				// _
				case ParticlesOutType::Point:
					startPos.X = 0.5f;
					startPos.Y = 0.5f;
					break;
				default:
					break;
			}
			this->_startPos = m->TransformCoordinate(&startPos);
			this->_startPos.Add(*pOffset);

			mw.EndMatrix(imatID);

			// Gg̊pxZ
			seStateParticles *state = this->_pObjWork->_partState;
			this->_power = this->createPower(
									state->OutVector(),
									state->OutLenge(),
									fElemRot,
									state->OutScaleLenge());
			// pxZȂB
			this->_power2 = this->createPower(
									state->InfluenceVector(),
									state->InfluenceLenge(),
									0.0f,
									state->InfluenceScaleLenge());

			this->_elapsedTime = 0.0f;
			return this->Update(fTime);
		}


		// Ԃ̍XV
		// ͑O̒l瑫悤Ȍ`ł͂ȂAPosPowervZB
		bool Update(float fAddTime)
		{
			this->_elapsedTime += fAddTime;

			// 𒴂ĂȂ
			if (this->_elapsedTime >= this->_lifeTime)
				return false;

			// Cӂ̎Ԃ̈ʒu擾
			glbVector2 p = this->GetPos(this->_elapsedTime);
			float rate = this->_elapsedTime / this->_lifeTime;
			this->UpdateMatrixAndColor(rate, &p);
			return true;
		}


		// p[eBN̒ǉ/}
		// p[eBN͂PÂǉ̂ƂB
		void AddOne(ParticleOne *next)
		{
			next->_pNext = this->_pNext;
			next->_pBefor = this;
			if (this->_pNext != 0)
			{
				this->_pNext->_pBefor = next;
			}
			this->_pNext = next;
		}

		// p[eBN̍폜
		void Cut()
		{
			if (this->_pBefor != 0)
			{
				this->_pBefor->_pNext = this->_pNext;
			}
			if (this->_pNext != 0)
			{
				this->_pNext->_pBefor = this->_pBefor;
			}
			this->_pNext = null;
			this->_pBefor = null;
		}


	private:
		// p[eBNXe[^XASʒu擾
		glbVector2 GetCenter(const seStateParticlesInfo *partInfo, float rate)
		{
			const seStateParticlesInfo::State<glbVector2> *centerState = partInfo->Center();
			glbVector2 center = centerState->End();
			center.Sub(centerState->Start());
			center.Scale(FePointInfo::GetPos(rate, *centerState->Anim()));
			center.Add(centerState->Start());
			return center;
		}


		// }gNXƃJ[XV
		void UpdateMatrixAndColor(float rate, glbVector2 *pos)
		{
			const seStateParticlesInfo *partInfo = this->_pObjWork->_particlesObj->Info();
			// XP[
			const seStateParticlesInfo::State<glbVector2> *scaleState = partInfo->Scale();
			glbVector2 scale = scaleState->End();
			scale.Sub(scaleState->Start());
			scale.Scale(FePointInfo::GetPos(rate, *scaleState->Anim()));
			scale.Add(scaleState->Start());
			// Ԃꕝ
			scale.Mul(this->_baseScale);
			// p[eBNXe[^XASʒu擾
			glbVector2 center = this->GetCenter(partInfo, rate);

			// J[
			const seStateParticlesInfo::State<glbColor> *colorState = partInfo->Color();
			glbColor sc = colorState->Start();
			glbColor ec = colorState->End();

			float fstateRate = FePointInfo::GetPos(rate, *colorState->Anim());
			glbColor c = glbColor(
				static_cast<glb::Byte>(static_cast<int>(static_cast<float>(ec.R - sc.R) * fstateRate) + sc.R),
				static_cast<glb::Byte>(static_cast<int>(static_cast<float>(ec.G - sc.G) * fstateRate) + sc.G),
				static_cast<glb::Byte>(static_cast<int>(static_cast<float>(ec.B - sc.B) * fstateRate) + sc.B),
				static_cast<glb::Byte>(static_cast<int>(static_cast<float>(ec.A - sc.A) * fstateRate) + sc.A));
			c.Mul(&this->_baseColor);
			this->Color(&c);

			// ]
			float fRot = 0.0f;

			// ړpx
			if (this->_bRotDirection)
			{
				// Q_pxZo
				float fFrameTime = seRootScene::Instance()->RootScene()->Option()->FrameTime();
				glbVector2 np = this->GetPos(this->_elapsedTime + fFrameTime * 0.5f);
				fRot = seMath::GetRot(pos, &np);
			}
			// ȊO
			else
			{
				const seStateParticlesInfo::State<float> *rs = partInfo->Rot();
				fRot = (rs->End() - rs->Start()) * FePointInfo::GetPos(rate, *rs->Anim()) + rs->Start();
				fRot += this->_baseRot;
				fRot -= (float)(((int)fRot / 360) * 360);
				fRot = fRot / 180.0f * glb::PI;
			}
			seMath::Transformation2D(this->_pLocalMatrix,
								&center, &scale, fRot, pos);
		}



		// Cӂ̎Ԃ̈ʒu擾
		// oXVɈʒu擾ł܂B
		glbVector2 GetPos(float fTime)
		{
			// ړʉZ
			float rate = glb::MinValue(1.0f, glb::MaxValue(0.0f, fTime / this->_lifeTime));

			glbVector2 p1 = this->_power;
			p1.Scale(rate);
			glbVector2 p2 = this->_power2;
			p2.Scale(rate);
			p2.Add(this->_power);

			glbVector2 p = p2;
			p.Sub(p1);
			p.Scale(rate);
			p.Add(p1);
			p.Add(this->_startPos);
			return p;
		}
	};



	// ========================================================================


	seObject *seoParticlesWork::Object()
	{
		return this->_particlesObj;
	}

	const glbVector2 *seoParticlesWork::GetSize()
	{
		return this->_particlesObj->Size();
	}

	void seoParticlesWork::InsertUnuseTop(ParticleOne *p)
	{
		p->Next(this->_unuseList);
		if (this->_unuseList != null)
		{
			this->_unuseList->Befor(p);
		}
		this->_unuseList = p;
		p->Befor(null);
	}

	ParticleOne *seoParticlesWork::GetUnuseInstance()
	{
		ParticleOne *ret = this->_unuseList;
		if (ret != null)
		{
			this->_unuseList = ret->Next();
			if(this->_unuseList != null)
			{
				this->_unuseList->Befor(null);
			}
			ret->Next(null);
		}
		return ret;
	}

	ParticleOne *seoParticlesWork::PeekNextUnuse()
	{
		return this->_unuseList;
	}


	void seoParticlesWork::ResizeParticlesBuffer(int iPartNum)
	{
		this->ClearBuffer();
		this->_particlesList = null;
		this->_lastParticles = null;
		this->_unuseList = null;
		// p[eBN쐬ĖgpXgɓo^
		for (int i = 0; i < iPartNum; i++)
		{
			this->InsertUnuseTop(evc_new ParticleOne(this));
		}
	}

	// p[eBNSăJbg
	void seoParticlesWork::ResetParticlesBuffer()
	{
		while(this->_particlesList != null)
		{
			ParticleOne *p = this->_particlesList;
			this->_particlesList = p->Next();
			p->Next(null);
			p->Befor(null);
			this->InsertUnuseTop(p);
		}
		this->_lastParticles = null;
	}

	// p[eBN̈̍폜
	void seoParticlesWork::ClearBuffer()
	{
		// p[eBNSăJbg
		this->ResetParticlesBuffer();

		ParticleOne *p = this->GetUnuseInstance();
		while(p != null)
		{
			GLB_SAFE_DELETE(p);
			p = this->GetUnuseInstance();
		}
	}


	seoParticlesWork::seoParticlesWork(seElem *elem, seoParticles *obj)
		: _particlesObj(obj) // `IuWFNg
		, _elem(elem) // Gg
		, _particlesList(null) // p[eBNXg
		, _lastParticles(null) // p[eBNXg̍Ō
		, _unuseList(null) // gpp[eBNXg
		, _bPlay(false) // p[eBNۂ
		, _successionScale(false) // XP[pۂ
		, _successionRot(ParticlesRot::Normal)  // ]pۂ
		, _drawPriority(ParticlesDrawPriority::Top) // `惂[h
		, _outType(ParticlesOutType::Plane) // ݂̏o͏
		, _partState(null) // p[eBNXe[^X
		, _uv(null)
		, _elapsedTime(0.0f)
	{
		// p[eBNobt@̃TCY
		this->ResizeParticlesBuffer(obj->Info()->MaxNum());

		this->_partState = static_cast<seStateParticles*>(seStateUtil::CreateStateType(emElemState::ParticleControl));
		this->_uv = static_cast<seStateFloat2*>(seStateUtil::CreateStateType(emElemState::UV));

		// Xe[^X̊gf[^NA
		this->_elem->State()->ClearStateData();
		this->_elem->State()->LinkStateData(emElemState::ParticleControl, this->_partState);
		this->_elem->State()->LinkStateData(emElemState::UV, this->_uv);
	}

	seoParticlesWork::~seoParticlesWork()
	{
		this->ClearBuffer();
		GLB_SAFE_DELETE(this->_partState);
		GLB_SAFE_DELETE(this->_uv);
	}


	void seoParticlesWork::Reset()
	{
		// p[eBNobt@SĖgpɂ
		this->ResetParticlesBuffer();
		this->_elapsedTime = 0.0f;
		this->_bPlay = false;
		this->_successionRot = ParticlesRot::Normal;
		this->_successionScale = false;
		this->_drawPriority = ParticlesDrawPriority::Top;
		this->_outType = ParticlesOutType::Plane;
	}

	void seoParticlesWork::Draw()
	{
#if 0
		if (seRootScene::RootScene()->EditMode())
			return;
#endif

		// p[eBNo͂̐ẽ}gNXp
		seMatrixWork &rMatWork = seMatrixWork::Instance();
		int iMatID = rMatWork.StartMatrix();

		IGLMatrix *m = rMatWork.GetMatrix();
		ParticleOne *p = this->_particlesList;
		while(p != null)
		{
			m->Copy(p->LocalMatrix());
			m->Mul(this->_elem->State()->ParentMatrix());
			glbColor color = p->Color();
			color.Mul(this->_elem->Parent()->State()->NowColor());
			this->_particlesObj->Draw(this->_elem->ElemID(), m, &color);
			p = p->Next();
		}
		rMatWork.EndMatrix(iMatID);
	}


	void seoParticlesWork::Update(float fAddTime)
	{
		// o͒̃p[eBNXV
		ParticleOne *p = this->_particlesList;
		while(p != null)
		{
			ParticleOne *next = p->Next();
			// \Ȃ疢gpXg
			if (!p->Update(fAddTime))
			{
				// ŌȂ
				if(p == this->_lastParticles)
				{
					this->_lastParticles = p->Befor();
				}
				// 擪Ȃ
				if(p == this->_particlesList)
				{
					this->_particlesList = p->Next();
				}

				// gpp[eBNXg͂
				p->Cut();
				// gpXgɓo^
				this->InsertUnuseTop(p);
			}
			p = next;
		}

		seStateData *data = this->_elem->State()->GetData(emElemState::ParticleControl);
		bool bNext = true;
		// VԂɂȂȂo̓tOXV
		if ((data != null) && (data->MoveKey()))
		{
			switch (this->_partState->Play())
			{
				case ParticlesPlay::Play:
					this->_bPlay = true;
					bNext = true;
					break;
				case ParticlesPlay::PlayOne:
					this->_bPlay = true;
					bNext = false;
					break;
				case ParticlesPlay::Stop:
					this->_bPlay = false;
					this->_elapsedTime = 0.0f;
					break;
				// 
				default:
					break;
			}


			if(this->_partState->SuccessionRot() != ParticlesRot::Non)
			{
				this->_successionRot = this->_partState->SuccessionRot();
			}

			if (this->_partState->Priority() != ParticlesDrawPriority::Non)
			{
				this->_drawPriority = this->_partState->Priority();
			}

			if (this->_partState->OutType() != ParticlesOutType::Non)
			{
				this->_outType = this->_partState->OutType();
			}


			switch (this->_partState->SuccessionScale())
			{
				case ParticlesSuccession::Use:
					this->_successionScale = true;
					break;
				case ParticlesSuccession::NoUse:
					this->_successionScale = false;
					break;
				default:
					break;
			}
		}

		if (this->_bPlay && (this->_partState->OutCount() > 0.0f))
		{
			// oߎԂ甭p[eBNZo
			this->_elapsedTime += fAddTime;
			float fFrameTime = seRootScene::Instance()->RootScene()->Option()->FrameTime();

			// PxĐȂ1t[̎ԂZ
			if (this->_partState->Play() == ParticlesPlay::PlayOne)
			{
				this->_elapsedTime = fFrameTime;
			}


			const glbVector2 *bfoPos = this->_elem->State()->BfoPos();
			const glbVector2 *nowPos = this->_elem->State()->Pos()->Value();

			float fMstTime = this->_elapsedTime;

			// p[eBN܂ł̎
			float fPfs = fFrameTime / static_cast<float>(this->_partState->OutCount());
//			int iAddFrame = static_cast<int>(this->_elapsedTime / fPfs);

			// p[eBN
			for(;this->_elapsedTime >= fPfs; this->_elapsedTime -= fPfs)
			{
				float timeLeft = this->_elapsedTime - fPfs;

				// gp̈悪ꍇ͉I
				p = this->PeekNextUnuse();
				if (p == null) continue;

				// Öʒup[eBN̔ʒu`␳
				glbVector2 offset = *nowPos;
				offset.Sub(*bfoPos);
				offset.Scale(-(timeLeft / fMstTime));

				if (!p->Start(timeLeft, &offset)) continue;

				p = this->GetUnuseInstance();

				// ŏ̂P̏ꍇ
				if (this->_lastParticles == null)
				{
					this->_particlesList = this->_lastParticles = p;
				}
				else
				{
					// őSʂɔzȕꍇ
					if (this->_drawPriority == ParticlesDrawPriority::Top)
					{
						this->_lastParticles->AddOne(p);
						this->_lastParticles = p;
					}
					// ȊO͍ŉɔzu
					else
					{
						p->Next(this->_particlesList);
						this->_particlesList->Befor(p);
						this->_particlesList = p;
						p->Befor(null);
					}
				}

			}

//			this->_elapsedTime -= static_cast<float>(iAddFrame) * fPfs;
			this->_bPlay = bNext;
		}
	}


	// ========================================================================
	// p[eBNIuWFNg

	void seoParticles::Size( const glbVector2 *value)
	{
		this->_size = *value;
		this->_pSprite->UpdateSize(&this->_size);
	}

	void seoParticles::UV(const glbVector2 *value)
	{
		this->_uv = *value;
		this->_pSprite->UpdateUV(&this->_uv);
	}

	void seoParticles::ResourceID(int value)
	{
		this->iResourceID = value;
		this->bUpdateImage = true;
	}

	void seoParticles::UpdateImage()
	{
		if (!this->bUpdateImage)
			return;

		// Ȃ
		if (this->iResourceID == seResourceManager::InvalidID)
		{
			this->_pTexture = null;
		}
		else
		{
			this->_pTexture = static_cast<siResourceTexture*>(this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));

			// \[X͂邪łĂȂꍇ
			if ((this->_pTexture != null) && !this->_pTexture->IsEffective())
			{
				// XVIȂ
				this->_pTexture = null;
				this->_pSprite->UpdateTexture(null);
				return;
			}
		}

		this->_pSprite->UpdateTexture(this->_pTexture);
		this->bUpdateImage = false;
	}

	siResourceTexture *seoParticles::ResourceTexture()
	{
		this->UpdateImage();
		return this->_pTexture;
	}

	const seStateParticlesInfo *seoParticles::Info() const
	{
		return this->_pInfo;
	}

	void seoParticles::SetState(seStateParticlesInfo *state)
	{
		this->_pInfo->Set(state);
	}

	emObject::Type seoParticles::ElemType() const 
	{
		return emObject::Particles;
	}

	seoParticles::seoParticles()
		: _pSprite(null)
		, _color(255,255,255,255)		// J[
		, _size(10.0f, 10.0f)		// TCY
		, _uv(0.0f, 0.0f)			// UV
		, iResourceID(seResourceManager::InvalidID)		// \[XID
		, bUpdateImage(false)		// ResourceIDɂ킹ăeNX`̍XV
		, _pTexture(null)	// eNX`\[X擾
	{
		this->_pInfo = evc_new seStateParticlesInfo();
	}

	seoParticles::~seoParticles()
	{
		GLB_SAFE_DELETE(this->_pInfo);
		GLB_SAFE_DELETE(this->_pSprite);
	}



	void seoParticles::Init(int iObjectID, seSceneManager *pScene)
	{
		seObject::Init(iObjectID, pScene);
		// VID蓖Ăꂽ̂ōȂ
		GLB_SAFE_DELETE(this->_pSprite);

		this->_pSprite = seRootScene::Instance()->IOManager()->CreateTextureSprite(this->ObjectID());
		this->_pSprite->UpdateTexture(this->ResourceTexture());
		this->_pSprite->UpdateSize(&this->_size);
		this->_pSprite->UpdateUV(&this->_uv);
	}

	seObjectWork *seoParticles::CreateDrawWork(seElem *elem)
	{
		return evc_new seoParticlesWork(elem, this);
	}

	void seoParticles::Draw(int iElemID, const IGLMatrix *m, const glbColor *color)
	{
		this->UpdateImage();
		glbColor c = *color;
		c.Mul(this->Color());
		this->_pSprite->Draw(iElemID, m, &c);
	}

	void seoParticles::Draw(int iElemID, const IGLMatrix *m,
						const IGLMatrix *mUV, const glbColor *color)
	{
		this->UpdateImage();
		glbColor c = *color;
		c.Mul(this->Color());
		this->_pSprite->Draw(iElemID, m, &c, mUV);
	}

	void seoParticles::Load(IseStateInput *input, seSceneManager *manager)
	{
		seObject::Load(input, manager);
		this->_size = seIOUtil::ReadVector2(input);
		this->_uv = seIOUtil::ReadVector2(input);
		this->ResourceID(static_cast<int>(input->ReadShort()));
		this->_color = seIOUtil::ReadColor(input);
		this->_pInfo->Load(input);
	}

}

