#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/seResource/seResourceScene.h"
#include "seoScene.h"


using namespace glb;
namespace evc
{
	const glbVector2 seoScene::sDefaultSize = glbVector2(10.0f, 10.0f);

	seObject *seoSceneWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}


	seoSceneWork::seoSceneWork(seElem *pInElem, seoScene *pInObject)
	{
		this->pElem = pInElem;
		this->pObject = pInObject;
	
		this->pLoadControl = static_cast<seStateLoadControl*>(seStateUtil::CreateStateType(emElemState::Load));
		this->pSceneAnim = static_cast<seStateString*>(seStateUtil::CreateStateType(emElemState::SceneAnim));
	
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
		this->pElem->State()->LinkStateData(emElemState::Load, this->pLoadControl);
		this->pElem->State()->LinkStateData(emElemState::SceneAnim, this->pSceneAnim);
	}

	seoSceneWork::~seoSceneWork()
	{
		GLB_SAFE_DELETE( this->pLoadControl );
		GLB_SAFE_DELETE( this->pSceneAnim );
	}

	void seoSceneWork::Draw()
	{
		if (this->pScene == null) return;
		// Aj[V̗vȂȂ܂\łiKɂȂB
		if (this->pScene->ElemManager()->LastAnim() == seAnimManager::InvalidID)
			return;
	
		// wi`
		this->pObject->DrawBg( this->pElem->ElemID(),
							   this->pElem->NowMatrix(),
							   this->pElem->State()->NowColor() );
	
		// pScene.Root.Elem̐e}gNXɃf[^ݒ
		seElem *pRootElem = this->pScene->ElemManager()->RootElem();
		if (pRootElem != null)
		{
			this->pScene->ElemManager()->Succession(this->pElem->NowMatrix());
			this->pScene->ElemManager()->Draw();
		}
	}

	void seoSceneWork::Reset()
	{
		if (this->pScene == null) return;
		this->pScene->ElemManager()->PlayTime( 0.0f );
		this->pScene->ElemManager()->ResetAnim();
	}

	void seoSceneWork::DoEvent()
	{
		if (this->pScene == null) return;
		// Aj[V̗vȂȂ܂\łiKɂȂB
		if (this->pScene->ElemManager()->LastAnim() == seAnimManager::InvalidID)
			return;
	
		// qbg͏IĂ̂ŁACxĝݎs
		this->pScene->ElemManager()->DoEvent();
	}

	void seoSceneWork::Update(float fAddTime)
	{
		// [htOXVꂽȂ
		seStateData *pLoadData = pElem->State()->GetData(emElemState::Load);
		if (pLoadData->MoveKey())
		{
			seResourceScene *pResource = null;
	
			switch (this->pLoadControl->State())
			{
				// [hJn
				case seLoadControl::Load:
					// Ƀ[hς
					if (this->pScene != null)
						break;
					// V[̃[hJn
					pResource = this->pObject->ResourceScene();
					// AV[Ƀ[h
					pResource->Load(this->pObject->Scene());
					break;
	
				// V[[X
				case seLoadControl::Release:
					pResource = this->pObject->ResourceScene();
					pResource->Cancel();
					if (this->pScene != null)
					{
						this->pScene->Clear();
						this->pScene = null;
					}
					break;
				default:
					break;
			}
		}
	
	
		// V[`FbN
		if (this->pScene == null)
		{
			// V[\[Xgp\Ȃo^
			seResourceScene *pResource = this->pObject->ResourceScene();
			if (pResource->IsEffective())
			{
				this->pScene = pResource->Scene()->Clone();
				this->pScene->Parent( this->pObject->Scene() );
			}
			// V[ꍇ͂ȂɂȂB
			else
			{
				return;
			}
		}
	
		// XVtO
		bool bDoUpdate = false;
	
		// J[pV[ɔf
		seStateData *pData = pElem->State()->GetData(emElemState::SceneAnim);
		if (pData != 0)
		{
			bDoUpdate |= pData->MoveKey();
		}
	
	
		seElemManager *lem = this->pScene->ElemManager();
		lem->Color(this->pElem->State()->NowColor());
	
		// Aj[V̍XVB
		if (bDoUpdate)
		{
			seAnimManager *pAnimManager = this->pScene->AnimManager();
	
			int iAnimID = pAnimManager->FindID(this->pSceneAnim->Value());
			lem->SetAnim(iAnimID);
			// ANeBuȃAj[Vݒ
			pAnimManager->ActiveAnim( iAnimID );
			// ݃V[ǗĂ鎞ԂփAj[Vړ
			lem->PlayTime( 0.0f );
			lem->ResetAnim();
	
			bDoUpdate = false;
		}
		// ʏAj[V
		else
		{
			this->LocalScene()->UpdateAnim();
			lem->Update(this->pElem->NowMatrix(), fAddTime);
		}
	}

	const glbVector2 *seoSceneWork::GetSize()
	{
		seResourceScene *pSceneResource = this->pObject->ResourceScene();
		if (pSceneResource == null) return &seoScene::sDefaultSize;
		return pSceneResource->Scene()->Option()->ScreenSize();
	}


	void seoScene::ResourceID(int value)
	{
		this->iResourceID = value;
		this->pTargetScene = null;
	}

	seResourceScene *seoScene::ResourceScene()
	{
		if (this->pTargetScene == null)
		{
			// \[X̎擾
			seResourceScene *pSceneResource = static_cast<seResourceScene*>(
				seObject::Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(
																	this->iResourceID));
			if (pSceneResource == null) return null;
			if (!pSceneResource->IsEffective()) return null;
			this->pTargetScene = pSceneResource;
		}
		return this->pTargetScene;
	}

	seoScene::seoScene()
		: pColorSprite( null ) // af`p
		, iResourceID( seResourceManager::InvalidID )
		, pTargetScene( null )
		, sBgColor( glbColor(0,0,0,255) )
	{
	}

	seoScene::~seoScene()
	{
	}


	seObjectWork *seoScene::CreateDrawWork(seElem *pElem)
	{
		return evc_new seoSceneWork(pElem, this);
	}

	void seoScene::DrawBg(int iElemID, const IGLMatrix *m, const glbColor *sMulColor)
	{
		// wiXvCg쐬ĂȂꍇ̂
		if (this->pColorSprite == null)
		{
			seResourceScene *pResource = this->ResourceScene();
			if (pResource == null) return;
	
			seOption *pOption = pResource->Scene()->Option();
			this->pColorSprite = seRootScene::Instance()->IOManager()->CreateColorSprite(this->ObjectID());
			this->pColorSprite->UpdateSize(pOption->ScreenSize());
			this->sBgColor = *pOption->BgColor();
		}
	
		glbColor c = *sMulColor;
		c.Mul( &this->sBgColor );
		this->pColorSprite->Draw(iElemID, m, &c);
	}


	void seoScene::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->sBgColor = seIOUtil::ReadColor(pInput);
		this->ResourceID(static_cast<int>(pInput->ReadShort()));
	}
}
