#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/seResource/seResourceScene.h"
#include "seoSound.h"

using namespace glb;
namespace evc
{

	seObject *seoSoundWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}

	static glbVector2 sSoundSizeDmy( 10.0f, 10.0f );
	const glbVector2 *seoSoundWork::GetSize()
	{
		return &sSoundSizeDmy;
	}

	seoSoundWork::seoSoundWork(seElem *pInElem, seoSound *pInObject)
		: pObjectSound( null )
		, bDoUpdate( false )
		, eSoundState( seSoundControl::Non )
		, pElem( pInElem )
		, pObject( pInObject )
		, pVolume( null )
		, pPan( null )
		, pSoundControl( null )
	{
	
		this->pVolume = static_cast<seStateInt1*>(seStateUtil::CreateStateType(emElemState::Volume));
		this->pPan = static_cast<seStateInt1*>(seStateUtil::CreateStateType(emElemState::Pan));
		this->pSoundControl = static_cast<seStateSoundControl*>(seStateUtil::CreateStateType(emElemState::SoundControl));
	
		// Xe[^X̊gf[^NA
		this->pElem->State()->ClearStateData();
		this->pElem->State()->LinkStateData(emElemState::Volume, this->pVolume);
		this->pElem->State()->LinkStateData(emElemState::Pan, this->pPan);
		this->pElem->State()->LinkStateData(emElemState::SoundControl, this->pSoundControl);
	}

	seoSoundWork::~seoSoundWork()
	{
		GLB_SAFE_DELETE( this->pVolume );
		GLB_SAFE_DELETE( this->pPan );
		GLB_SAFE_DELETE( this->pSoundControl );
	}

	void seoSoundWork::Reset()
	{
		this->bDoUpdate = false;
		this->eSoundState = seSoundControl::Non;
		if (this->pObjectSound != null)
		{
			this->pObjectSound->Stop();
		}
	}

	void seoSoundWork::PlaySound(bool bLoop)
	{
		if (this->pObject->SoundTime() >= 0.0f)
		{
			this->pObjectSound->Play( this->pElem->ElemID(),
									  this->pObject->SoundStart(),
									  this->pObject->SoundTime(), bLoop );
		}
		else
		{
			this->pObjectSound->Play( this->pElem->ElemID(), 
									  this->pObject->SoundStart(), bLoop);
		}
	}

	void seoSoundWork::Draw()
	{
		if (this->pObjectSound == null)
		{
			this->pObjectSound = seRootScene::Instance()->IOManager()->CreateSound( 0, this->pObject->ResourceSound());
		}
	
		// {[ݒ
		int iVolume = this->pVolume->Value();
		int iPan = this->pPan->Value();
	
		this->pObjectSound->UpdateVolume(iVolume);
		this->pObjectSound->UpdatePan(iPan);
	
		// Đ[hłȂ悤ɂ悤B
		if ((seRootScene::Instance()->EditMode() == false) && this->bDoUpdate)
		{
			// ԕʏ
			switch (this->eSoundState)
			{
				// ~ɍĐȂB
				case seSoundControl::Play:
					this->PlaySound(false);
					break;
				case seSoundControl::PlayLoop:
					this->PlaySound(true);
					break;
				// Đɒ~Ȃ
				case seSoundControl::Stop:
					this->pObjectSound->Stop();
					break;
				// 
				case seSoundControl::Non:
					break;
			}
	
			this->bDoUpdate = false;
			this->eSoundState = seSoundControl::Non;
		}
	
		this->pObjectSound->Draw(this->pElem->NowMatrix(), iVolume, iPan);
	
	}

	void seoSoundWork::Update(float fAddTime)
	{
		seStateData *pData = pElem->State()->GetData(emElemState::SoundControl);
		// VԂɂȂȂ
		if ((pData != 0) && (pData->MoveKey()))
		{
			this->bDoUpdate = true;
			this->eSoundState = this->pSoundControl->State();
		}
	}


	// seoSound
	seoSound::seoSound()
		: iResourceID( seResourceManager::InvalidID )
		, pResourceSound( null )
		, fSoundStart( 0.0f )
		, fSoundTime( 0.0f )
	{
	}


	void seoSound::ResourceID(int value)
	{
		this->iResourceID = value;
		this->pResourceSound = null;
	}

	siResourceSound *seoSound::ResourceSound()
	{
		if (this->pResourceSound == null)
		{
			this->pResourceSound = static_cast<siResourceSound*>(
				this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));
		}
		return this->pResourceSound;
	}




	seObjectWork *seoSound::CreateDrawWork(seElem *pElem)
	{
		return evc_new seoSoundWork(pElem, this);
	}

	void seoSound::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->ResourceID(static_cast<int>(pInput->ReadShort()));
		this->fSoundStart = pInput->ReadFloat();
		this->fSoundTime = pInput->ReadFloat();
	}
}
