#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbDebug.h"
#include "glb/glbVector.h"
#include "glb/IGLMatrix.h"
#include "evc/common/seMatrixWork.h"
#include "evc/file/IseStateIO.h"
#include "evc/IO/siObject.h"
#include "evc/IO/siObjectFont.h"
#include "evc/IO/siIOManager.h"
#include "evc/scene/seRootScene.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seOption.h"
#include "evc/scene/seResourceManager.h"
#include "evc/scene/seElemManager.h"
#include "evc/scene/seAnimManager.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoFont/seoFont.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "evc/seElem/seElem.h"
#include "evc/seElem/seStateType.h"
#include "evc/seElem/seStateTypeEx.h"
#include "evc/seElem/seElemState/seElemState.h"
#include "evc/seElem/seElemState/seStateData.h"
#include "evc/seResource/seResourceScene.h"
#include "seoTextureSprite.h"


using namespace glb;
namespace evc
{
	seObject *seoTextureSpriteWork::Object()
	{
		return static_cast<seObject *>(this->pObject);
	}

	seoTextureSprite *seoTextureSpriteWork::TextureObject()
	{
		return (this->pDrawTexture == null) ? this->pObject
											: this->pDrawTexture;
	}

	const glbVector2 *seoTextureSpriteWork::GetSize()
	{
		return this->TextureObject()->Size();
	}

	seoTextureSpriteWork::seoTextureSpriteWork(seElem *pInElem, seoTextureSprite *pInObjct)
		: pObject( pInObjct ) // `IuWFNg
		, pElem( pInElem ) // Gg
		, pDrawTexture( null )		// ̕`eNX`
		, pUVMatrix( null )		// `惏[N
		, sUVOffset( glbVector2::Zero )
		// gXe[^X
		, pTexture( null )
		, pTextureControl( null )
		, pLoop( null )
		, pUV( null )
	{
		for( int i = 0; i < GLB_ARRAY_COUNT( this->apUV ); i++ )
		{
			this->apUV[i] = null;
		}

		this->pUVMatrix = seRootScene::Instance()->IOManager()->CreateMatrix();
	
		this->pTexture = static_cast<seStateSwitchInt1*>(seStateUtil::CreateStateType(emElemState::Texture));
		this->pTextureControl = static_cast<seStateTextureControl*>(seStateUtil::CreateStateType(emElemState::TextureControl));
		this->pLoop = static_cast<seStateLoop*>(seStateUtil::CreateStateType(emElemState::Loop));
		this->pUV = static_cast<seStateFloat2*>(seStateUtil::CreateStateType(emElemState::UV));
	
		// Xe[^X̊gf[^NA
		seElemState *pState = this->pElem->State(); 
		pState->ClearStateData();
		pState->LinkStateData(emElemState::Texture, this->pTexture);
		pState->LinkStateData(emElemState::TextureControl, this->pTextureControl);
		pState->LinkStateData(emElemState::Loop, this->pLoop);
		pState->LinkStateData(emElemState::UV, this->pUV);
	}

	seoTextureSpriteWork::~seoTextureSpriteWork()
	{
		GLB_SAFE_DELETE( this->pUVMatrix );
		GLB_SAFE_DELETE( this->pTexture );
		GLB_SAFE_DELETE( this->pTextureControl );
		GLB_SAFE_DELETE( this->pLoop );
		GLB_SAFE_DELETE( this->pUV );
	}

	void seoTextureSpriteWork::Reset()
	{
		this->pDrawTexture = null;
	}

	void seoTextureSpriteWork::Draw()
	{
		seoTextureSprite *pTextureObject = this->TextureObject();
	
		const glbColor *sColor = this->pElem->State()->NowColor();
		int iElemID = this->pElem->ElemID();
	
		// [vȂȂ
		if ((this->pLoop->LoopCountH() == 1) && (this->pLoop->LoopCountW() == 1))
		{
			pTextureObject->Draw(iElemID, this->pElem->NowMatrix(), this->pUVMatrix, sColor);
		}
		// [vȂ
		else
		{
				// ]ptu}gNX
				// E]
				// ㉺]
					// ㉺E]
			seUseWorkMatrix gw;
			const glbVector2 *sObjectSize = this->pObject->Size();
			IGLMatrix *m = gw.GetMatrix();
//			IGLMatrix *ms = gw.GetMatrix();
			int iHeightNum = glb::AbsValue(this->pLoop->LoopCountH());
			int iWidthNum = glb::AbsValue(this->pLoop->LoopCountW());
			float fAddW = static_cast<float>((this->pLoop->LoopCountW() >= 0) ? sObjectSize->X : -sObjectSize->X);
			float fAddH = static_cast<float>((this->pLoop->LoopCountH() >= 0) ? sObjectSize->Y : -sObjectSize->Y);
			int iHSwitchID = 0;
			int iVSwitchID = 0;
			this->apUV[0] = this->pUVMatrix;
			this->apUV[1] = gw.GetMatrix();
			this->apUV[2] = gw.GetMatrix();
			this->apUV[3] = gw.GetMatrix();
			IGLMatrix *mWork = gw.GetMatrix();
			if (this->pLoop->RevW())
			{
				iHSwitchID = 1;
				apUV[1]->SetScale(-1.0f, 1.0f);
				mWork->SetUV(static_cast<float>(this->sUVOffset.X), 0.0f);
				apUV[1]->MulR(mWork);
				apUV[1]->MulR(this->pUVMatrix);
			}
			if (this->pLoop->RevH())
			{
				iVSwitchID = 2;
				apUV[2]->SetScale(1.0f, -1.0f);
				mWork->SetUV(0.0f, static_cast<float>(this->sUVOffset.Y));
				apUV[2]->MulR(mWork);
				apUV[2]->MulR(this->pUVMatrix);
				if (this->pLoop->RevW())
				{
					apUV[3]->SetScale(-1.0f, -1.0f);
					mWork->SetUV( &this->sUVOffset);
					apUV[3]->MulR(mWork);
					apUV[3]->MulR(this->pUVMatrix);
				}
			}
			int iVSwitch = 0;
			for (int i = 0; i < iHeightNum; i++)
			{
				int iHSwitch = 0;
				for (int j = 0; j < iWidthNum; j++)
				{
					m->SetPos(j * fAddW, i * fAddH);
					m->MulR(this->pElem->NowMatrix());
					pTextureObject->Draw(iElemID, m, apUV[iVSwitch | iHSwitch], sColor);
					iHSwitch ^= iHSwitchID;
				}
				iVSwitch ^= iVSwitchID;
			}
		}
	}

	void seoTextureSpriteWork::Update(float fAddTime)
	{
		// eNX`XV`FbN
		seStateData *pTextureData = pElem->State()->GetData(emElemState::Texture);
		if (pTextureData->MoveKey())
		{
			int iValue = this->pTexture->Value();
			this->pDrawTexture = (iValue == seObjectManager::InvalidID) ?
					null : static_cast<seoTextureSprite*>(this->pObject->Scene()->ObjectManager()->seIDManager<seObject*>::Find(iValue));
		}
	
		// tu}gNX쐬
		this->pUVMatrix->SetUV(this->pUV->Value());
	
		// UṼTCYZo
		siResourceTexture *pImage = this->TextureObject()->ResourceTexture();
	
		if (pImage == null)
		{
			this->sUVOffset = glbVector2::Zero;
		}
		else
		{
			glbVector2 sRate = *this->pObject->Size();

			const glbVector2I *t = pImage->TextureSize();
			sRate.Div( glbVector2( static_cast<float>(t->X),
								   static_cast<float>(t->Y) ) );
			this->sUVOffset = *this->pObject->UV();
			this->sUVOffset.Scale(2.0f);
			this->sUVOffset.Add( sRate );
		}
	}

	// seoTextureSprite
	//

	seoTextureSprite::seoTextureSprite()
		: pSprite( null )
		, sSize( 10.0f, 10.0f )
		, sUV( glbVector2::Zero )
		, iResourceID( seResourceManager::InvalidID )
		, pResourceTexture( null )
		, sColor( &glbColor::White )
		, bUpdateImage( false )
	{

	}

	seoTextureSprite::~seoTextureSprite()
	{
		GLB_SAFE_DELETE( this->pSprite );
	}

	void seoTextureSprite::Size( const glbVector2 *value)
	{
		this->sSize = *value;
		this->pSprite->UpdateSize(&this->sSize);
	}

	void seoTextureSprite::UV( const glbVector2 *value)
	{
		this->sUV = *value;
		this->pSprite->UpdateUV(&this->sUV);
	}

	void seoTextureSprite::ResourceID(int value)
	{
		this->iResourceID = value;
		this->pResourceTexture = null;
		this->bUpdateImage = true;
	}

	siResourceTexture *seoTextureSprite::ResourceTexture()
	{
		this->UpdateImage();
		return this->pResourceTexture;
	}

	// ResourceIDɂ킹ăeNX`̍XV
	void seoTextureSprite::UpdateImage()
	{
		if ( !this->bUpdateImage ) return;

		// Ȃ
		if ( this->iResourceID == seResourceManager::InvalidID )
		{
			this->pResourceTexture = null;
		}
		else
		{
			this->pResourceTexture = static_cast<siResourceTexture*>(this->Scene()->ResourceManager()->seIDManager<siResourceBase*>::Find(this->iResourceID));

			// \[X͂邪łĂȂꍇ
			if ( (this->pResourceTexture != null) &&
				 !this->pResourceTexture->IsEffective() )
			{
				// XVIȂ
				this->pResourceTexture = null;
				this->pSprite->UpdateTexture( null );
				return;
			}
		}

		this->pSprite->UpdateTexture( this->pResourceTexture );
		this->bUpdateImage = false;
	}


	void seoTextureSprite::Init(int iInObjectID, seSceneManager *pInScene)
	{
		seObject::Init(iInObjectID, pInScene);
		// VID蓖Ăꂽ̂ōȂ
		GLB_SAFE_DELETE( this->pSprite );

		this->pSprite = seRootScene::Instance()->IOManager()->CreateTextureSprite(this->ObjectID());
		this->pSprite->UpdateTexture(this->ResourceTexture());
		this->pSprite->UpdateSize(&this->sSize);
		this->pSprite->UpdateUV(&this->sUV);
	}


	seObjectWork *seoTextureSprite::CreateDrawWork(seElem *pElem)
	{
		return evc_new seoTextureSpriteWork(pElem, this);
	}

	void seoTextureSprite::Draw( int iElemID, const IGLMatrix *m, const glbColor *sMulColor)
	{
		this->UpdateImage();
		glbColor c = *sMulColor;
		c.Mul( this->Color() );
		this->pSprite->Draw(iElemID, m, &c);
	}

	void seoTextureSprite::Draw( int iElemID, 
								 const IGLMatrix *m, 
								 const IGLMatrix *pUVMatrix, 
								 const glbColor *sMulColor)
	{
		this->UpdateImage();
		glbColor c = *sMulColor;
		c.Mul( this->Color() );
		this->pSprite->Draw(iElemID, m, &c, pUVMatrix);
	}

	void seoTextureSprite::Load(IseStateInput *pInput, seSceneManager *pManager)
	{
		seObject::Load(pInput, pManager);
		this->sSize = seIOUtil::ReadVector2(pInput);
		this->sUV = seIOUtil::ReadVector2(pInput);
		this->ResourceID(static_cast<int>(pInput->ReadShort()));
		this->sColor = seIOUtil::ReadColor(pInput);
	}
}
