#include "glbPch.h"
#include "glb/glbDefine.h"
#include "glb/glbString.h"
#include "glb/glbDebug.h"
#include "evc/file/bnInput.h"
#include "evc/file/IseStateIO.h"
#include "evc/common/seDefine.h"
#include "evc/scene/seSceneManager.h"
#include "evc/scene/seObjectManager.h"
#include "evc/scene/seResourceManager.h"
#include "evc/IO/siResource.h"
#include "evc/seObject/seObject.h"
#include "evc/seObject/ObjectParts/seoTextureSprite.h"
#include "seResourceScene.h"

using namespace glb;
namespace evc
{

	seResourceScene::seResourceScene(const glbString *strFileName, seSceneManager *pScene)
		: siResourceBase(strFileName)
	{
		this->pScene = pScene;
		this->eState = siResourceState::Effective;
	}

	seResourceScene::seResourceScene( const glbString *strFileName)
		: siResourceBase(strFileName)
		, pScene( null )
	{
	}


	// [h̑SĂɑ΂ďs
	void seResourceScene::DoFirstLoadAll(deAction1<siResourceBase*> *pFunc)
	{
		const est::map<int,siResourceBase*> &resources = *this->pScene->ResourceManager()->Piars();
		est::map<int,siResourceBase*>::const_iterator it = resources.begin();
		while(it != resources.end())
		{
			siResourceBase *pResource = it->second;
			switch( pResource->Type() )
			{
				// C[WȂ疳Ń[h
				case emResourceState::Image:
					(*pFunc)(pResource);
					break;
				// V[ŃI[g[hȂ烍[h
				case emResourceState::Scene:
					if ( pResource->IsAutoLoad() )
					{
						(*pFunc)(pResource);
					}
					break;
				// ȊO͖
				default:
					break;
			}
			it++;
		}

	}

	siResourceInit::Type seResourceScene::Init(const est::vector<glb::Byte> *buffer)
	{
		// ܂V[ĂȂȂ
		if (this->pScene == null)
		{
			this->pScene = evc_new seSceneManager(this->siResourceBase::Name());
			bnInput inReader;
			inReader.Open( buffer );
			this->pScene->Load( &inReader );
			inReader.Close();
	
			struct func : deAction1<siResourceBase*>
			{
				seSceneManager *pScene;
				func ( seSceneManager *s ) : pScene( s ) {}
				void operator()( siResourceBase *pResource )
				{
					pResource->Load(this->pScene);
				}
			};
			func f( this->pScene );
			// ǂݍ݂郊\[Xɑ΂ďs
			this->DoFirstLoadAll(&f);
		}
	
		// V[̍쐬ɎsĂꍇ̓G[
		if (this->pScene == null) return siResourceInit::Failure;
	
	
		// V[ŎgeNX`̃[hI`FbN
		// ǂݍ݂郊\[Xɑ΂ďs
		struct LoadCheck : deAction1<siResourceBase*>
		{
			bool bInited;
			bool bErr;
			LoadCheck() : bInited( true ), bErr( false ){}

			void operator()( siResourceBase *pResource )
			{
				pResource->Update();
				// IĂȂ̂Ȃ
				if (!pResource->IsEffective())
				{
					this->bInited = false;
					// \[X̓o^ɎsȂ
					if ( pResource->State() == siResourceState::Invalidity)
					{
						this->bErr = true;
					}
				}
			}
		};
		LoadCheck f;
		this->DoFirstLoadAll( &f );


		// V[ƑSẴeNX`[hς̏ԂȂ
		if (f.bInited)
		{
			const est::map<int,seObject*> &objs = *this->pScene->ObjectManager()->Piars();
			est::map<int,seObject*>::const_iterator it = objs.begin();
			while(it != objs.end())
			{
				seObject *pObj = it->second;
				if ( pObj->ElemType() == emObject::Texture )
				{
					seoTextureSprite *pTex = static_cast<seoTextureSprite*>(pObj);
					pTex->ResourceID( pTex->ResourceID() );
				}
				it++;
			}
		}

		// Pł\[X̏ɎsȂV[̓o^s
		if (f.bErr) return siResourceInit::Failure;
		return (f.bInited) ? siResourceInit::Success
						   : siResourceInit::Continuation;




	}


	bool seResourceScene::IsEnableScene()
	{
		if (this->pScene == null) return false;

		// V[̃eNX`SĎgp\ȂtrueȊOfalse				
		const est::map<int,siResourceBase*> &resources = *this->pScene->ResourceManager()->Piars();
		est::map<int,siResourceBase*>::const_iterator it = resources.begin();
		while(it != resources.end())
		{
			siResourceBase *pResource = it->second;
			if (pResource->Type() == emResourceState::Image)
			{
				if(!pResource->IsEffective())
					return false;
			}
		}
		return true;
	}

	seSceneManager *seResourceScene::PassesScene()
	{
		// ςłȂꍇ͏nsB
		if (!this->IsEffective())
		{
			GLB_ASSERT(false);
			return null;
		}
	
		seSceneManager *pRetScene = this->pScene;
		this->pScene = null;
	
		return pRetScene;
	}

	seResourceScene::~seResourceScene()
	{
		GLB_SAFE_DELETE( this->pScene );
	}
}
